C
C
C
      INTEGER   FUNCTION   LENSTR
     I                           (LEN,STR)
C
C     + + + PURPOSE + + +
C     LENSTR returns the position of the last non-blank character in the
C     array STR.  If STR contains all blanks, LENSTR returns a zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STR
C     STR    - character array of size LEN
C
C     + + + LOCAL VARIALBES + + +
      INTEGER     LENT,DONE
C
C     + + + END SPECIFICATIONS + + +
C
      DONE = 0
      LENT = LEN
  10  CONTINUE
        IF (STR(LENT).EQ.' ') THEN
          LENT = LENT- 1
        ELSE
          DONE = 1
        END IF
      IF (LENT.GT.0.AND.DONE.EQ.0) GO TO 10
C
      LENSTR= LENT
C
      RETURN
      END
C
C
C
      SUBROUTINE   LFTSTR
     I                    (LEN,
     M                     TITLE)
C
C     + + + PURPOSE + + +
C     Left justifies the characters within the character array TITLE.
C     Imbedded blanks are preserved.  Title is restricted to a maximum
C     size of 132.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 TITLE(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array TITLE, 1 <= LEN <= 132
C     TITLE  - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     L,IS,I,K,TLEN
      CHARACTER*1 BUFF(132),BLNK
C
C     + + + DATA INITIALIZATIONS + + +
      DATA BLNK/' '/
C
C     + + + END SPECIFICATIONS + + +
C
      TLEN = LEN
      IF (TLEN.GT.132) TLEN = 132
      I = 0
 10   CONTINUE
        I = I + 1
      IF (TITLE(I).EQ.BLNK.AND.I.LT.TLEN) GO TO 10
      IF (TITLE(I).EQ.BLNK.AND.I.EQ.TLEN) I= I+ 1
      IS = I
C     found beginning of non-blanks
C
      IF (IS.LE.TLEN .AND. IS.NE.1) THEN
C       title not all blanks and not currently left justified
        L = TLEN - (IS-1)
        DO 30 I = 1,TLEN
          BUFF(I) = TITLE(I)
          TITLE(I)= BLNK
 30     CONTINUE
        DO 40 I = 1,L
          K = IS + I - 1
          TITLE(I) = BUFF(K)
 40     CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   STRFND
     I                          (LEN,STR,FLEN,FSTR)
C
C     + + + PURPOSE + + +
C     STRFND returns the position in the array STR where the array FSTR
C     begins.  If FSTR is not contained in STR, STRFND returns a zero.
C     If FLEN is greater than LEN, STRFND returns a zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN,FLEN
      CHARACTER*1 STR(LEN),FSTR(FLEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array being searched
C     STR    - character array of size LEN to be searched
C     FLEN   - size of character array to search for
C     FSTR   - character array of size FLEN to be searched for
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J,FNDPOS
C
C     + + + END SPECIFICATIONS + + +
C
      FNDPOS= 0
      IF (FLEN.LE.LEN) THEN
        I= 0
        J= 0
 10     CONTINUE
          J= 0
 20       CONTINUE
            I= I+ 1
            J= J+ 1
            IF (J.EQ.FLEN.AND.STR(I).EQ.FSTR(J)) FNDPOS= I- FLEN+ 1
          IF (STR(I).EQ.FSTR(J).AND.FNDPOS.EQ.0.AND.J.LT.FLEN) GO TO 20
          I= I- J+ 1
        IF (I.LT.LEN-FLEN+1.AND.FNDPOS.EQ.0) GO TO 10
      END IF
C
      STRFND= FNDPOS
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   STRLNX
     I                           (NPTS,BUFF)
C
C     + + + PURPOSE + + +
C     STRLNX returns the number of characters in the array BUFF.  Leading
C     and trailing blanks are not included in the count, embedded blanks
C     are include in the count.  If there are no non-blank characters in
C     the array, STRLNX returns a 1.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     NPTS
      CHARACTER*1 BUFF(NPTS)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NPTS   - size of the character array BUFF
C     BUFF   - character array of size NPTS
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     K,KE,KB
      CHARACTER*1 BLNK
C
C     + + + DATA INITIALIZATIONS + + +
      DATA BLNK/' '/
C
C     + + + END SPECIFICATIONS + + +
C
      K = 1
 5    CONTINUE
        IF (BUFF(K).NE.BLNK) GO TO 6
        K = K + 1
      IF (K.LE.NPTS) GO TO 5
 6    CONTINUE
      KB = K -1
C
      K = NPTS
 7    CONTINUE
        IF (BUFF(K).NE.BLNK) GO TO 8
        K = K - 1
      IF (K.GE.1) GO TO 7
 8    CONTINUE
      KE = K
C
      K= KE- KB
      IF (K.LE.0) K=1
      STRLNX = K
C
      RETURN
      END
C
C
C
      SUBROUTINE   ZIPC
     I                  (LEN, ZIP,
     O                   X)
C
C     + + + PURPOSE + + +
C     Fill the character array X of size LEN with the given value ZIP.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 ZIP,X(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array ZIP
C     ZIP    - character to fill array X
C     X      - character array of size LEN to be filled with ZIP
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     L
C
C     + + + END SPECIFICATIONS + + +
C
      DO 100 L= 1, LEN
        X(L)= ZIP
 100  CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE   ZLJUST
     M                   (STRING)
C
C     + + + PURPOSE + + +
C     Left justifies the characters in STRING.  Leading blanks are
C     removed and the characters are shiftd to the left.  STRING is
C     padded with trailing blanks.  Embedded blanks are preserved.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STRING*(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRING - character variable
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J,K,L
C
C     + + + INTRINSICS + + +
      INTRINSIC   LEN
C
C     + + + END SPECIFICATIONS + + +
C
      IF (STRING(1:1) .EQ. ' ') THEN
C       has at least 1 leading blank
        L = LEN(STRING)
        I = 0
 10     CONTINUE
C         loop to look for other leading blanks
          I= I+ 1
        IF (I.LT.L .AND. STRING(I:I).EQ.' ') GO TO 10
C       remove leading blanks
Chnb  some compilers (OTG) complain about "STRING= STRING(I:L)", so ...
        K = 1
        DO 15, J=I,L
           STRING(K:K) = STRING(J:J)
           K = K + 1
 15     CONTINUE
        STRING(K:) = ' '
      END IF
C
      RETURN
      END
C
C
C
      INTEGER FUNCTION   ZLNTXT
     I                          (STRING)
C
C     + + + PURPOSE + + +
C     Determine the length of STRING, excluding trailing blanks and
C     nulls.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STRING*(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRING - character variable
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I,DONFG
      CHARACTER*1 NULL,CTMP
C
C     + + + INTRINSICS + + +
      INTRINSIC   LEN, CHAR
C
C     + + + END SPECIFICATIONS + + +
C
      NULL = CHAR(0)
      I    = LEN(STRING)
      DONFG= 0
      IF (I.GT.0) THEN
C       string is not all blanks
 10     CONTINUE
          CTMP= STRING(I:I)
          IF (CTMP.EQ.' ' .OR. CTMP.EQ.NULL) THEN
C           still nothing useful
            I= I- 1
          ELSE
C           a character
            DONFG= 1
          END IF
        IF (I.GT.0 .AND. DONFG.EQ.0) GOTO 10
      END IF
C
      ZLNTXT= I
C
      RETURN
      END
C
C
C
      SUBROUTINE   ZTRIM
     M                  (STRING)
C
C     + + + PURPOSE + + +
C     Left justify the characters in STRING.  All leading and embedded
C     blanks are removed.  String is padded with trailing blanks.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER STRING*(*)
C
C      + + + ARGUMENT DEFINITIONS + + +
C     STRING - character variable
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J
      CHARACTER TEMP*255
C
C     + + + INTRINSICS + + +
      INTRINSIC   LEN
C
C     + + + END SPECIFICATIONS + + +
C
      J = 0
      DO 100 I = 1, LEN(STRING)
C       look at each char
        IF (STRING(I:I) .NE. ' ') THEN
C         not blank, save it
          J = J + 1
          TEMP(J:J) = STRING(I:I)
        END IF
 100  CONTINUE
C
      IF (J .GT. 0) THEN
C       not all blank, return string with no blanks
        STRING = TEMP(1:J)
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   QUPCAS
     I                    (LEN,
     M                     STRING)
C
C     + + + PURPOSE + + +
C     Converts all lower case letters in STRING to upper case.  All
C     upper case letters are left in upper case.
C     To convert a character string from lower case to upper case
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STRING(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - length of character array STRING
C     STRING - character array of length LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,ICH
C
C     + + + INTRINSICS + + +
      INTRINSIC  ICHAR, MOD, CHAR
C
C     + + + END SPECIFICATIONS + + +
C
      DO 10 I = 1, LEN
        ICH= ICHAR(STRING(I))
        ICH= MOD(ICH,128)
        IF (ICH.GE.97 .AND. ICH.LE.122) THEN
C         character is lower case
          STRING(I)= CHAR(ICH-32)
        END IF
 10   CONTINUE
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   CKNBLV
     I                           (LEN,CBUF)
C
C     + + + PURPOSE + + +
C     Looks for the first non-blank character in CBUF.  CKNBLV returns
C     the position of the first non-blank character.  If CBUF contains
C     all blanks, CKNBLV returns a zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 CBUF(LEN)
C
C     + + + ARGUMENT DEFINITION + + +
C     LEN    - size of character array CBUF
C     CBUF   - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J
C
C     + + + END SPECIFICATIONS + + +
C
      I = 0
      J = 0
 10   CONTINUE
        I = I+ 1
        IF (CBUF(I) .NE. ' ') J = I
      IF (I.LT.LEN .AND. J.EQ.0) GO TO 10
C
      CKNBLV = J
C
      RETURN
      END
C
C
C
      SUBROUTINE   RHTSTR
     I                    (LEN,
     M                     TITLE)
C
C     + + + PURPOSE + + +
C     Right justifies the characters within the character array TITLE.
C     Embedded blanks are preserved.  
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 TITLE(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array TITLE, 1 <= LEN <= 132
C     TITLE  - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     IS,I,TLEN,I1,I2
      CHARACTER*1 BLNK
C
C     + + + DATA INITIALIZATIONS + + +
      DATA BLNK/' '/
C
C     + + + END SPECIFICATIONS + + +
C
C     find number of blanks on the right
      TLEN = LEN
      IF (TLEN.GT.132) TLEN = 132
      I = TLEN + 1
 10   CONTINUE
        I = I - 1
      IF (TITLE(I).EQ.BLNK.AND.I.GT.1) GO TO 10
      IS = I
C     found end of non-blanks
C
      IF (IS.LT.TLEN) THEN
C       title not currently right justified
        DO 30 I = 1,IS
          I1 = TLEN - I + 1
          I2 = IS - I + 1
          TITLE(I1) = TITLE(I2)
 30     CONTINUE
        DO 40 I = 1,TLEN-IS
          TITLE(I) = BLNK
 40     CONTINUE
      END IF
C
      RETURN
      END   
C
C
C
      SUBROUTINE   CARVAR
     I                    (LENA, CARY, LENV,
     O                     CVAR)
C
C     + + + PURPOSE + + +
C     Places the contents of the character*1 array CARY of size LENA
C     into the character string CVAR of length LEN.  If the length of
C     the array is greater than the length of the string (LENA > LENV)
C     CVAR will contain the first LENV characters from CARY.  If the
C     array is smaller than the string (LENA < LENV), then CVAR will
C     be padded at the end with blanks.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       LENA, LENV
      CHARACTER*1   CARY(LENA)
      CHARACTER*(*) CVAR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LENA   - size of input character*1 array CARY
C     CARY   - input character array of size LENA
C     LENV   - available length for output character variable CVAR
C     CVAR   - output character variable of length LENV
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     L, L1
      CHARACTER*1 BLNK
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  BLNK / ' ' /
C
C     + + + END SPECIFICATIONS + + +
C
      IF (LENA .GE. LENV) THEN
C       copy input array to output string, truncate if necessary
        DO 100 L = 1, LENV
          CVAR(L:L) = CARY(L)
 100    CONTINUE
      ELSE
C       copy input array to output string, pad with blanks as needed
        DO 200 L = 1, LENA
          CVAR(L:L) = CARY(L)
 200    CONTINUE
        L1 = LENA + 1
        DO 220 L = L1, LENV
          CVAR(L:L) = BLNK
 220    CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   CHKSTR
     I                           (LEN,NSTR,STR1,STR2)
C
C     + + + PURPOSE + + +
C     Searches the columns of the array STR2 for a match to the array
C     STR1.  If a match is found, CHKSTR returns the column number of the
C     first column in STR2 containing STR1.  If not match is found,
C     CHKSTR returns a zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN,NSTR
      CHARACTER*1 STR1(LEN),STR2(LEN,NSTR)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STR1 and number of rows in STR2
C     NSTR   - number of strings to be checked, number of columns in STR2
C     STR1   - character array of size LEN to searched
C     STR2   - character array of size LEN,NSTR to be searched
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J,MAT
C
C     + + + END SPECIFICATIONS + + +
C
      CHKSTR = 0
      J = 1
C
  10  CONTINUE
        I = 1
        MAT = 1
  20    CONTINUE
          IF (STR1(I).NE.STR2(I,J)) MAT = 0
          I = I + 1
        IF (I.LE.LEN.AND.MAT.EQ.1) GO TO 20
        IF (MAT.EQ.1) CHKSTR = J
        J = J + 1
      IF (J.LE.NSTR.AND.CHKSTR.EQ.0) GO TO 10
C
      RETURN
      END
C
C
C
      SUBROUTINE   CHRCHR
     I                    (LEN,STR1,
     O                     STR2)
C
C     + + + PURPOSE + + +
C     Copy LEN characters from array STR1 to array STR2.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STR1(LEN),STR2(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character arrays STR1 and STR2
C     STR1   - input character array of size LEN
C     STR2   - output character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I
C
C     + + + END SPECIFICATIONS + + +
C
      DO 10 I = 1,LEN
        STR2(I) = STR1(I)
 10   CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE   CHRDEL
     I                    (LEN,POS,
     M                     STRING)
C
C     + + + PURPOSE + + +
C     Deletes the character in array position POS in STRING and then sifts
C     the resto fo the array left one position.  The last character in the
C     STRING is set to blank.  If POS is greater than LEN, no action is
C     taken.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN,POS
      CHARACTER*1 STRING(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STRING
C     POS    - array position of the character to be deleted from STRING
C     STRING - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I
C
C     + + + END SPECIFICATIONS + + +
C
      IF (POS.LT.LEN) THEN
C       no problem with array dimensions
        DO 10 I = POS,LEN-1
          STRING(I) = STRING(I + 1)
 10     CONTINUE
        STRING(LEN) = ' '
      ELSE IF (POS.EQ.LEN) THEN
C       just delete last character
        STRING(LEN) = ' '
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   CHRINS
     I                    (LEN,COL,CHAR,
     M                     STRING)
C
C     + + + PURPOSE + + +
C     Inserts the character CHAR into array position COL in the character
C     array STRING.  First, array positions COL thru LEN-1 are shifted
C     forward one space.  Then CHAR is placed in array position COL.  The
C     original STRING(LEN) value is deleted in the process.  
C     WARNING:  CHRINS does not check that COL is a valid position.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN,COL
      CHARACTER*1 CHAR,STRING(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of array STRING
C     COL    - position in STRING that CHAR is to be inserted
C     CHAR   - character to be inserted in STRING
C     STRING - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J
C
C     + + + END SPECIFICATIONS + + +
C
      IF (COL.EQ.LEN) THEN
C       just fill last position
        STRING(LEN) = CHAR
      ELSE IF (COL.EQ.1 .AND. LEN.EQ.2) THEN
C       trap weird dimension case
        STRING(2) = STRING(1)
        STRING(1) = CHAR
      ELSE
C       all ok
        DO 10 I = COL,LEN-1
          J = LEN + COL- I- 1
          STRING(J+1) = STRING(J)
 10     CONTINUE
        STRING(COL) = CHAR
      END IF
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   CKNBLK
     I                           (LEN,CBUF)
C
C     + + + PURPOSE + + +
C     Checks the character array CBUF for the occurance of blanks.
C     CKNBLK returns a zero if the array contains all blanks.  CKNBLK
C     returns a 1 if there are any non-blank characters in CBUF.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 CBUF(LEN)
C
C     + + + ARGUMENT DEFINITION + + +
C     LEN    - size of character array CBUF
C     CBUF   - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   I,J
C
C     + + + END SPECIFICATIONS + + +
C
      J = 0
      DO 10 I = 1,LEN
        IF (CBUF(I) .NE. ' ') J = 1
 10   CONTINUE
C
      CKNBLK = J
C
      RETURN
      END
C
C
C
      SUBROUTINE   COPYC
     I                   (LEN, ZIP,
     O                    X)
C
C     + + + PURPOSE + + +
C     Copies the contents of character array ZIP to character array X.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 ZIP(LEN),X(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character arrays XIP and X
C     ZIP    - input character array of size LEN
C     X      - output character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     L
C
C     + + + END SPECIFICATIONS + + +
C
      DO 100 L= 1, LEN
        X(L)= ZIP(L)
 100  CONTINUE
C
      RETURN
      END
C
C
C
      SUBROUTINE   CTRSTR
     I                    (LEN,
     M                     TITLE)
C
C     + + + PURPOSE + + +
C     Centers the characters within the character array TITLE.  Embedded
C     blanks are preserved.  Leading and trailing blanks are balanced.
C     TITLE is restricted to a maximum size of 132.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 TITLE(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array TITLE, 1 <= LEN <= 132
C     TITLE  - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     L,IE,IS,I,J,K,B,TLEN
      CHARACTER*1 BUFF(132),BLNK
C
C     + + + DATA INITIALIZATIONS + + +
      DATA BLNK/' '/
C
C     + + + END SPECIFICATIONS + + +
C
      TLEN = LEN
      IF (TLEN.GT.132) TLEN = 132
      I = 0
 10   CONTINUE
        I = I + 1
      IF (TITLE(I).EQ.BLNK.AND.I.LT.TLEN) GO TO 10
      IS = I
C     found beginning of non-blanks
C
      I = TLEN + 1
 20   CONTINUE
        I = I - 1
      IF (TITLE(I).EQ.BLNK.AND.I.GT.1) GO TO 20
      IE = I
C     found end of non-blanks
C
      IF (IE.GE.IS) THEN
C       title not all blanks
        L = IE - (IS-1)
        B = (TLEN-L)/2
        DO 30 I = 1,TLEN
          BUFF(I) = TITLE(I)
          TITLE(I) = BLNK
 30     CONTINUE
        DO 40 I = 1,L
          J = B + I
          K = IS + I - 1
          TITLE(J) = BUFF(K)
 40     CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   CVARAR
     I                    (LENV, CVAR, LENA,
     O                     CARY)
C
C     + + + PURPOSE + + +
C     Places the contents of the character variable CVAR of expected
C     length LENV into the character*1 array CARY of length LENV.  If
C     the length of the variable is greater than the length of the array
C     (LENV > LENA), CARY will contain the first LENA characters of CVAR.
C     If the variable is shorter than the array (LENV < LENA), then CVAR
C     will be padded at the end with blanks.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       LENV, LENA
      CHARACTER*(*) CVAR
      CHARACTER*1   CARY(LENA)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LENV   - length of input character variable
C     CVAR   - input character variable of length LENV
C     LENA   - size of output character array
C     CARY   - output character array of size LENA
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   L, L1
      CHARACTER*1 BLNK
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  BLNK  / ' ' /
C
C     + + + END SPECIFICATIONS + + +
C
      IF (LENV .GE. LENA) THEN
C       copy variable to array, truncate if needed
        DO 100 L = 1, LENA
          CARY(L) = CVAR(L:L)
 100    CONTINUE
      ELSE
C       copy variable to array, padding with blanks
        DO 200 L = 1, LENV
          CARY(L) = CVAR(L:L)
 200    CONTINUE
        L1 = LENV + 1
        DO 220 L = L1, LENA
          CARY(L) = BLNK
 220    CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   DATCHR
     I                    (YR, MO, DY,
     O                     OLEN, DBUFF)
C
C     + + + PURPOSE + + +
C     Places a year, month, and day date into the character array DBUFF.
C     The month is represented as a number.  If MO is not a valid month
C     (1-12), the month and day will not be included in DBUFF.  If DY is
C     not a valid day (1-31), the day will not be included in DBUFF.
C     (Note that no check is made to verify that DY is a valid day for
C     the month MO.) OLEN is the actual number of characters used to
C     represent the date, the remainder of DBUFF is padded with blanks.
C     Examples:
C
C                YR   MO   DY   OLEN  DBUFF
C               1994  12   31    10   1994/12/31
C                92    1    1     6   92/1/1
C               1992  11    0     7   1992/11
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     YR, MO, DY, OLEN
      CHARACTER*1 DBUFF(10)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     YR     - year
C     MO     - month
C     DY     - day
C     OLEN   - number of array postions used for date
C     DBUFF  - output character array
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     LEN, LOC, JUST
      CHARACTER*1 SLSH,BLNK
C
C     + + + EXTERNALS + + +
      EXTERNAL    ZIPC, INTCHR
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  SLSH,BLNK  / '/',' ' /
C
C     + + + END SPECIFICATIONS + + +
C
      JUST= 1
      LOC = 1
      LEN = 10
      CALL ZIPC (LEN,BLNK,DBUFF)
      LEN = 4
      CALL INTCHR( YR, LEN, JUST, OLEN, DBUFF(LOC) )
      LOC = LOC + OLEN
      IF(MO .GT. 0  .AND.  MO .LT. 13) THEN
        DBUFF(LOC) = SLSH
        LOC = LOC + 1
        LEN = 2
        CALL INTCHR( MO, LEN, JUST, OLEN, DBUFF(LOC) )
        LOC = LOC + OLEN
        IF(DY .GT. 0  .AND. DY .LT. 32) THEN
          DBUFF(LOC) = SLSH
          LOC = LOC + 1
          LEN = 2
          CALL INTCHR( DY, LEN, JUST, OLEN, DBUFF(LOC) )
          LOC = LOC + OLEN
        END IF
      END IF
      OLEN = LOC - 1
C
      RETURN
      END
C
C
C
      SUBROUTINE   DATLST
     I                    (DATE,
     O                     DSTRNG,LEN,ERRCOD)
C
C     + + + PURPOSE + + +
C     Converts an integer date (year, month, day, hour, minute, and
C     second) to a character representation, with the month represented
C     as a 3-character abbreviation.  If the hour, minute, and second are
C     all zero, they are not included.  If the date array contains an
C     invalid date or time, the character representation will contain all
C     blanks.  The program does check that the day is valid for the month.
C     A 2-digit year is assumed to occur in the first century.  Examples:
C
C     <-- DATE(1-6) -->   <----- DSTRNG ------>   LEN
C     1986,2,14,10,30,0   1986 FEB. 14 10:30:00    21
C     92,12,31,0,0,0      92 DEC. 31               10
C     
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     DATE(6),LEN,ERRCOD
      CHARACTER*1 DSTRNG(21)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DATE   - date (year, month, day, hour, minute, second)
C     DSTRNG - output character array
C     LEN    - actual number of characters output to character array
C     ERRCOD - flag indicating valid date
C              0 - valid date
C              1 - invalid year
C              2 - invalid month
C              3 - invalid day
C              4 - invalid hour
C              5 - invalid minute
C              6 - invalid second
C              If the year or month is invalid, the remaining date
C              elements are not checked.  If the year and month are
C              valid, ERRCOD is set for the smallest invalid date element,
C              ie  if day and hour are both invalid ERRDOC =  4.
C
C     + + +   LOCAL VARIABLES   + + +
      INTEGER     JUST,OLEN,LOC,MO,J
      CHARACTER*1 MOCHAR(39),BLNK
C
C     + + + FUNCTIONS + + +
      INTEGER     DAYMON
C
C     + + + EXTERNALS + + +
      EXTERNAL   CHRCHR, DAYMON, INTCHR, ZIPC
C
C     + + + DATA INITIALIZATIONS + + +
      DATA MOCHAR/'J','A','N',    'F','E','B',    'M','A','R',    'A',
     1            'P','R',    'M','A','Y',    'J','U','N',    'J','U',
     2            'L',    'A','U','G',    'S','E','P',    'O','C','T',
     3                'N','O','V',    'D','E','C',    'E','R','R'    /
      DATA BLNK  /' '/
C
C     + + + END SPECIFICATIONS + + +
C
C     check values
      J= 21
      CALL ZIPC (J,BLNK,DSTRNG)
      ERRCOD = 0
C
      IF (DATE(1).LE.0.OR.DATE(1).GT.2100) ERRCOD = 1
      IF (DATE(2).LE.0.OR.DATE(2).GT.12) ERRCOD = 2
C
      IF (ERRCOD.EQ.0) THEN
        IF (DATE(3).LE.0.OR.DATE(3).GT.DAYMON(DATE(1),DATE(2)))
     1                                                 ERRCOD = 3
        IF (DATE(4).LT.0.OR.DATE(4).GT.24) ERRCOD = 4
        IF (DATE(5).LT.0.OR.DATE(5).GT.60) ERRCOD = 5
        IF (DATE(6).LT.0.OR.DATE(6).GT.60) ERRCOD = 6
C
      END IF
      IF (ERRCOD.EQ.0) THEN
C
C       put date in character array
C       add year
        JUST = 1
        LEN = 4
        LOC = 1
        IF (DATE(1).GT.9999.OR.DATE(1).LT.-999) DATE(1) = 9999
        CALL INTCHR (DATE(1),LEN,JUST,OLEN,DSTRNG(LOC))
        LOC = LOC + OLEN + 1
C
C       add month
        LEN = 3
        MO = 3*(DATE(2)-1) + 1
        IF (MO.LT.1.OR.MO.GT.34) MO = 37
        CALL CHRCHR (LEN,MOCHAR(MO),DSTRNG(LOC))
        LOC = LOC + LEN + 1
C
C       add day
        LEN = 2
        IF (DATE(3).GT.99.OR.DATE(3).LT.-9) DATE(3) = 99
        CALL INTCHR (DATE(3),LEN,JUST,OLEN,DSTRNG(LOC))
        LOC = LOC + OLEN + 1
C
C       add hour
        IF (DATE(4)+DATE(5)+DATE(6) .GT. 0) THEN
          LEN = 2
          IF (DATE(4).GT.99.OR.DATE(4).LT.0) DATE(4) = 99
          IF (DATE(4).LT.10) LEN = 1
          IF (LEN.EQ.1) THEN
            DSTRNG(LOC)= '0'
            LOC = LOC + 1
          END IF
          CALL INTCHR (DATE(4),LEN,JUST,OLEN,DSTRNG(LOC))
          LOC = LOC + OLEN
C
C         add colon
          DSTRNG(LOC)= ':'
          LOC = LOC + 1
C
C         add minute
          LEN = 2
          IF (DATE(5).GT.99.OR.DATE(5).LT.0) DATE(5) = 99
          IF (DATE(5).LT.10) LEN = 1
          IF (LEN.EQ.1) THEN
            DSTRNG(LOC)= '0'
            LOC = LOC + 1
          END IF
          CALL INTCHR (DATE(5),LEN,JUST,OLEN,DSTRNG(LOC))
          LOC = LOC + OLEN
C
C         add colon
          DSTRNG(LOC)= ':'
          LOC = LOC + 1
C
C         add second
          LEN = 2
          IF (DATE(6).GT.99.OR.DATE(6).LT.0) DATE(6) = 99
          IF (DATE(6).LT.10) LEN = 1
          IF (LEN.EQ.1) THEN
            DSTRNG(LOC)= '0'
            LOC = LOC + 1
          END IF
          CALL INTCHR (DATE(6),LEN,JUST,OLEN,DSTRNG(LOC))
          LOC = LOC + OLEN + 1
        END IF
        LEN = LOC - 2
      END IF
C
      RETURN
      END
