C
C
C
      SUBROUTINE   DECCHR
     I                    (REAIN, LENGTH,
     M                     JUST,
     O                     JLEN, STR)
C
C     + + + PURPOSE + + +
C     Converts a real number to a character array.  The number is left
C     or right justified in the array based on the value of JUST.  If the
C     number will not fit in the array, exponential notation is used and
C     the number is right justified in the array.  For left-justified
C     numbers, STR is padded with trailing blanks.  For right-justified
C     numbers, STR is padded with leading blanks.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      LENGTH, JUST, JLEN
      REAL         REAIN
      CHARACTER*1  STR(LENGTH)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     REAIN  - real value to be converted to a character array
C     LENGTH - available size for output character array STR
C     JUST   - output justification
C              0 - right justified
C              1 - left justified
C              will be forced to 0 if an exponent is required
C     JLEN   - actual number of characters placed in string, includes
C              any leading blanks if number is right justified.
C     STR    - output character array of size LENGTH
C
C     + + + COMMON BLOCKS + + +
C     numeric constants
      INCLUDE 'const.inc'
C 
C     + + + LOCAL VARIABLES + + + 
      INTEGER      I0, I1, I2, I, J, K, L, M, PREC, EXPO, WHDIGS, 
     $             EXDIGS, FDIGS, NOCNT, KLEN
      REAL         R1, R2, R3, ARTMP, RSUB, MINFIX, MAXFIX, X
      CHARACTER*1  FMTBUF(12), BUF(79),  ZERO,  BLNK,  POINT,  STAR
      CHARACTER*12 FMT
      CHARACTER*79 BUFSTR
      LOGICAL      FLOAT, FRAC, NEG, KILL, NOFIT
C
C     + + + LOCAL DEFINITIONS + + +
C     NOCNT  - for real values less than 1.0, NOCNT is the number of
C              zeros immediately following the decimal point (which
C              "don't count" in terms of the number of significant
C              digits representable by the machine); for real values
C              greater than or equal to 1.0, NOCNT = 0
C
C     + + + EQUIVALENCES + + +
      EQUIVALENCE (BUF,BUFSTR)
      EQUIVALENCE (FMT,FMTBUF)
C
C     + + + FUNCTIONS + + +
      INTEGER      RWDIGS
      REAL         FPART, RNOP
      LOGICAL      NONZER, NZFRAC
C
C     + + + INTRINSICS + + +
      INTRINSIC    IABS, ABS, ALOG10, REAL, LEN, INDEX, 
     $             INT, AINT, ANINT, NINT
C
C     + + + EXTERNALS + + +
      EXTERNAL     INTCHR, NUMINI, RWDIGS
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  I0, I1, I2, ZERO, BLNK, POINT, STAR
     .    /  0,  1,  2,  '0',  ' ',   '.',  '*' /
C
C     + + + STATEMENT FUNCTION DEFINITIONS + + +
C     absolute value of the FRACtional PART (of a real number)
      FPART(X)  = ABS(X) - ABS(AINT(X))
C     NON-ZERo real number? - true or false  N.B. X should not be < 0.0
      NONZER(X) = X .GT. 0.0
C     Non-Zero FRACtional part of a real number? - true or false
      NZFRAC(X) = NONZER(FPART(X))
C     number of Whole DIGitS - for X > 0;
C     double precision needed in the following to ensure correct calc.
C     of log for DG/UX
Cmyg  WDIGS(X)  = IDINT(DLOG10(DBLE(X*RP1MIN))) + 1 
C     to keep Ryan/McFarland optimization honest
      RNOP(X) = X
C
C     + + + END SPECIFICATIONS + + +
C
C     may need to init machine-dependent numeric constants (const.inc)
      CALL NUMINI
C
C     save time with the common case of REAIN .eq. 0
      IF (ABS(REAIN) .LE. R0MIN) THEN
         IF (LENGTH .GE. 3) THEN
            BUFSTR = '0.0'
         ELSE IF (LENGTH .EQ. 2) THEN
            BUFSTR = '0.'
         ELSE IF (LENGTH .EQ. 1) THEN
            BUFSTR = '0'
         ELSE
            NOFIT = .TRUE.
         END IF
      ELSE
C
C        the number of decimal digits of precision offered can be found
C        in the variable RPREC of the ICONST common block.
         PREC = RPREC
C
C        is the number negative?  set NEG and define MAXFIX/MINFIX
         IF (REAIN .LT. 0.0) THEN
            NEG = .TRUE.
            MAXFIX = 10.0**(LENGTH-1)
            MINFIX = 1.0/10.0**(LENGTH-1)
         ELSE
            NEG = .FALSE.
            MAXFIX = 10.0**LENGTH
            MINFIX = 1.0/10.0**LENGTH
         END IF
C
C        this copy of the absolute value of the number is used frequently
         ARTMP = ABS(REAIN)
C
C        using scientific notation, how many digits will be in the exponent?
C        the answer will be in EXDIGS.
         R1 = ALOG10(ARTMP*RP1MIN)
C        (RNOP must be used to keep Ryan/McFarland optimization from causing
C        problems.  If RNOP is not used, and RM optimization is turned on,
C        then R1 is left on the stack as an 80-bit number instead of being
C        popped and pushed.)
         R1 = RNOP(R1)
         EXPO = INT(R1)
         IF (R1 .LT. 0.0) THEN
            IF (NZFRAC(R1)) THEN
               EXPO = EXPO - 1
            END IF
         END IF
         IF (EXPO .EQ. 0) THEN
            EXDIGS = 1
         ELSE
            EXDIGS = INT(ALOG10(REAL(IABS(EXPO))*RP1MIN)) + 1
         END IF
C
C        how many digits in the whole part of the number?
         IF (ARTMP .GE. 1.0) THEN
            WHDIGS = RWDIGS(ARTMP)
         ELSE
C           one digit to allow room for 0
            WHDIGS = 1
         END IF
C
         IF (ARTMP.LT.MINFIX .OR. ARTMP.GT.MAXFIX 
     $       .OR. WHDIGS.GT.PREC) THEN
C           use scientific notation
            FLOAT = .FALSE.
C           force right justification with scientific notation
            JUST = 0
         ELSE
C           use floating point notation
            FLOAT  = .TRUE.
C           does the number have a non-zero fractional part?
            FRAC   = NZFRAC(REAIN)
C           here, in the context of a floating point number, EXDIGS is
C           a count of EXtra whole DIGits (not digits in the exponent).
            EXDIGS = 0
            NOCNT  = 0
            IF (FRAC) THEN
C              if the fractional part were rounded, would the number of
C              of whole digits increase?  (EXtra whole DIGitS)
               IF (ANINT(FPART(REAIN)) .GT. 0.0) THEN
C                  IF ((ARTMP+1.0) .GE. 1.0) THEN
                     IF (RWDIGS(ARTMP+1.0) .GT. WHDIGS) EXDIGS = 1
C                  END IF
               END IF
               IF (ARTMP .LT. 1.0) THEN
C                 since there is a non-zero fractional part (FRAC = TRUE),
C                 and the absolute value of the number is less than 1.0,
C                 how many consecutive zeros immediately follow the decimal
C                 point?  The NOCNT (no count) zeros don't count against the
C                 precision available.
                  R1 = ABS(ALOG10(ARTMP*RP1MIN))
                  IF (NZFRAC(R1)) THEN
                     NOCNT = INT (R1)
                  ELSE
                     NOCNT = INT (R1) - 1
                  END IF
               END IF
            END IF
         END IF
C
         IF (FLOAT) THEN
C           using floating point notation
            RSUB = REAIN
C
C           first, assume it's not negative, taking into account the '.':
            FDIGS = LENGTH - (WHDIGS + 1)
            IF (FDIGS-NOCNT .GT. PREC) FDIGS = PREC+NOCNT
C           if it is negative, there will be less room for fractional digits
            IF (NEG) FDIGS = FDIGS - 1
C           if the number of WHole digits is one, it might just be a zero
C           in which case, for purposes of precision, we can consider the
C           number of whole digits to be zero.
            IF (WHDIGS .EQ. 1) THEN
C              if the whole number IS a zero, then maybe it would be better
C              to add an extra digit of precision to the fraction rather than
C              display the zero whole number.  However, don't do this if
C              the precision would be exceeded.
               IF (AINT(ARTMP) .LE. 0.0 .AND. FDIGS.LT.PREC+NOCNT) THEN
                  WHDIGS = 0
C                 we could just add that extra digit of precision to the
C                 fraction, but if it's just zero too, it looks better to
C                 keep the zero to the left of the decimal.
                  IF (FRAC) THEN
C                    r1 gets the number that we had planned to display, but
C                    raised to a power of 10 such that it is a whole number.
                     R1 = ANINT(ARTMP*10.0**FDIGS)
C                    r2 gets a fraction representing digits which we had
C                    not planned to display and r3 is that fraction times 10.
                     R2 = (ARTMP*10.0**FDIGS) - R1
                     R3 = R2*10
C                    now, if 1) the number we were not going to display is,
C                    when rounded, non-zero, or if 2) the number passed in
C                    is negative, but everything we had planned on displaying
C                    was zero, then we add the extra digit of precision.
C                    in the first case, we just end up dropping the zero to
C                    the left of the decimal and adding an extra digit of
C                    precision; in the second case, the extra digit of
C                    precision is added only if the digits that were to be
C                    displayed previously were all zeros and the number
C                    itself is negative, so if the extra digit is zero, it
C                    is displayed to make up for the fact that the negative
C                    sign (for which room has been given) won't be displayed,
C                    but if the extra digit is non-zero, the negative sign
C                    will be displayed and the zero before the decimal will
C                    be lost.
                     IF (NINT(R3).NE.0 .OR. 
     $                  (NEG .AND. R1.LE.1.0E-20)) THEN
                           FDIGS  = FDIGS + 1
                     END IF
                  END IF
C                 if nocnt, the number of zero digits immediately
C                 following the decimal point (which don't count against the
C                 number of digits of precision representable by the machine),
C                 is greater than zero (this can only be the case if the
C                 absolute value of the number is less than one, as here)
C                 then we must ensure that the number of digits displayed
C                 does not exceed the actual machine precision
               END IF
            END IF
C
            IF ((WHDIGS + FDIGS - NOCNT) .GT. PREC) THEN
               FDIGS = PREC - WHDIGS + NOCNT
            END IF
C
            NOFIT = .FALSE.
            FMT = '(F  .  )'
            IF (FDIGS .GT. 0) THEN
C               expensive internal write replaced, but this is what it did
C               WRITE(FMT,96) LENGTH,FDIGS
               CALL INTCHR (LENGTH,I2,I0,KLEN,FMTBUF(3))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(6))
            ELSE IF (FDIGS .EQ. 0) THEN
               RSUB = ANINT (RSUB)
C               expensive internal write replaced, but this is what it did
C               WRITE(FMT,96) LENGTH,FDIGS
               CALL INTCHR (LENGTH,I2,I0,KLEN,FMTBUF(3))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(6))
            ELSE IF (FDIGS .EQ. -1) THEN
               RSUB = ANINT (RSUB)
               FDIGS = 0
C               expensive internal write replaced, but this is what it did
C               WRITE(FMT,96) LENGTH+1,FDIGS
               I = LENGTH + 1
               CALL INTCHR (I,I2,I0,KLEN,FMTBUF(3))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(6))
            ELSE
               NOFIT = .TRUE.
            END IF
C
         ELSE
C           using scientific notation
            RSUB  = REAIN
            NOFIT = .FALSE.
            IF (NEG) THEN
               IF (EXDIGS+5 .LE. LENGTH) THEN
                  FDIGS = LENGTH-(EXDIGS+5)
                  IF (FDIGS+1 .GT. PREC) FDIGS = PREC - 1
               ELSE
                  NOFIT = .TRUE.
               END IF
            ELSE
               IF (EXDIGS+4 .LE. LENGTH) THEN
                  FDIGS = LENGTH-(EXDIGS+4)
                  IF (FDIGS+1 .GT. PREC) FDIGS = PREC - 1
               ELSE
                  NOFIT = .TRUE.
               END IF
            END IF
C            expensive internal write replaced, but this is what it did
C            IF (.NOT. NOFIT) WRITE(FMT,95) LENGTH,FDIGS,EXDIGS
            IF (.NOT. NOFIT) THEN
               FMT = '(1PE  .  E )'
               CALL INTCHR (LENGTH,I2,I0,KLEN,FMTBUF(5))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(8))
               CALL INTCHR (EXDIGS,I1,I0,KLEN,FMTBUF(11))
            END IF
         END IF
C
C        here we either put asterisks or use 'write' to convert the number
         IF (NOFIT) THEN
            JLEN = 0
            DO 400 I = 1,LENGTH
               BUF(I) = STAR
  400       CONTINUE
         ELSE
C           some compilers don't like blanks embedded in the format string
C           as they are now, so first squeeze it together, and then use it
C           to format the given number.
            I = INDEX(FMT,BLNK)
            IF (I .GT. 0) THEN
               J = I
               L = LEN(FMT)
               DO 420 K = I,L
                  IF (FMTBUF(K) .NE. BLNK) THEN
                     FMTBUF(J) = FMTBUF(K)
                     J = J + 1
                  END IF
  420          CONTINUE
               DO 430 K = J,L
                  FMTBUF(K) = BLNK
  430          CONTINUE
            END IF
C
            I = INDEX(FMT,BLNK)
            IF (I .EQ. 0) I = LEN(FMT)
C
            WRITE(BUFSTR,FMT(1:I)) RSUB
C
         END IF
C
C        in either form, at most 1 trailing zero should be left.
C        make J point to the '.' if there is one
         J = INDEX (BUFSTR,POINT)
         IF (FLOAT) THEN
            IF (J .NE. 0) THEN
C              kludge to change " .0" to "0.0" in some cases on some 'chines
               IF ((J-1) .GT. 0) THEN
                  IF (BUF(J-1) .EQ. BLNK) THEN
                     BUF(J-1) = ZERO
                  END IF
               END IF
C              we don't want to truncate any zeros if there's no '.'
C              make K point to the last zero (if there is one)
               I = 0
               K = 0
  510          CONTINUE
               I = INDEX (BUFSTR(K+1:79),ZERO)
               IF (I .NE. 0) THEN
                  K = K + I
                  GO TO 510
               END IF
               KILL = .TRUE.
               IF (K .NE. 79) THEN
C                 only kill zeros that are not significant.  if the last
C                 zero is followed by something other than a blank,
C                 then you can't get rid of it.
                  IF (BUF(K+1) .NE. BLNK) KILL = .FALSE.
               END IF
               IF (K .GT. J+1 .AND. KILL) THEN
C                 only truncate zeros if they're to the right of the decimal
C                 and don't get rid of a zero that is immediately to the
C                 right of the decimal point.
                  DO 520 I = K,J+2,-1
                     IF (BUF(I) .EQ. ZERO) THEN
                        BUF(I) = BLNK
                     ELSE
C                       once a non-zero is found, get out of the loop
                        GO TO 525
                     END IF
  520             CONTINUE
  525             CONTINUE
               END IF
            END IF
         ELSE
C           get rid of trailing zeros in scientific notation
            IF (J .NE. 0) THEN
C              we don't won't to truncate any zeros if there's no '.'
C              make L point to the 'E'
               L = INDEX (BUFSTR,'E')
               IF (L .NE. 0) THEN
C                 make K point to the last digit before the 'E'
                  K = L - 1
                  IF (K .GT. J+1) THEN
C                    only truncate zeros between '.' and 'E', and don't
C                    get rid of the zero (if there is one) immediately to the
C                    right of the decimal.
                     M = J+2
                     DO 620 I = K,J+2,-1
                        IF (BUF(I) .EQ. ZERO) THEN
C                          don't truncate the last trailing zero
                           BUF(I) = BLNK
                        ELSE
                           M = I + 1
                           GO TO 625
                        END IF
  620                CONTINUE
  625                CONTINUE
C                    a non-zero was found, slide the exponent down to
C                    fill in the gap left by the truncated zeros.
                     DO 630 I = M,LENGTH
                        IF (L .LE. LENGTH) THEN
                           BUF(I) = BUF(L)
                        ELSE
                           BUFSTR(I:LENGTH) = BLNK
                           GO TO 635
                        END IF
                        L = L + 1
  630                CONTINUE
  635                CONTINUE
                  END IF
               END IF
            END IF
         END IF
      END IF
C
C     copy the final string, blanks and all
      IF (JUST .NE. 0) THEN
C        left justified (i.e., JUST != 0)
         J = 1
         DO 710 I = 1,LENGTH
            IF (BUF(J) .NE. BLNK) GO TO 715
            J = J + 1
  710    CONTINUE
  715    CONTINUE
         JLEN = INDEX (BUFSTR(J:LENGTH),BLNK)
         IF (JLEN .EQ. 0) THEN
            JLEN = LENGTH - J + 1
         ELSE
            JLEN = JLEN - 1
         END IF
         K = 1
         DO 720 I = J,LENGTH
            STR(K) = BUF(I)
            K = K + 1
  720    CONTINUE
         DO 730 I = K,LENGTH
            STR(I) = BLNK
  730    CONTINUE
      ELSE
C        right justified (i.e., JUST == 0)
         J = LENGTH
         DO 810 I = LENGTH,1,-1
            IF (BUF(J) .NE. BLNK) GO TO 815
            J = J - 1
  810    CONTINUE
  815    CONTINUE
         DO 820 I = 1,LENGTH-J
            STR(I) = BLNK
  820    CONTINUE
         K = 1
         DO 830 I = LENGTH-J+1,LENGTH
            STR(I) = BUF(K)
            K = K + 1
  830    CONTINUE
C        with right justified strings, JLEN is always LENGTH
         JLEN = LENGTH
      END IF
C
      RETURN
C
C      expensive internal writes were replaced, but this is what they did
C  95  FORMAT('(1PE',I2,'.',I2,'E',I1,')')
C  96  FORMAT('(F',I2,'.',I2,')')
C
      END
C
C
C
      SUBROUTINE   DPRCHR
     I                    (DPRIN, LENGTH,
     M                     JUST,
     O                     JLEN, STR)
C
C     + + + PURPOSE + + +
C     Converts a double precision number to a character array.  The
C     number is left or right justified in the array based on the value
C     of JUST.  If the number will not fit in the array, exponential
C     notation is used and the number is right justified in the array.
C     For left-justified numbers, STR is padded with trailing blanks.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      LENGTH, JUST, JLEN
      DOUBLE PRECISION  DPRIN
      CHARACTER*1  STR(LENGTH)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DPRIN  - double precision value to be converted to a character array
C     LENGTH - available size for output character array STR
C     JUST   - output justification
C              0 - right justified
C              1 - left justified
C              will be forced to 0 if an exponent is required
C     JLEN   - actual number of characters placed in string, includes
C              any leading blanks if number is right justified.
C     STR    - output character array of size LENGTH
C
C     + + + COMMON BLOCKS + + +
C     numeric constants
      INCLUDE 'const.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      I0, I1, I2, I, J, K, L, M, PREC, EXPO, WHDIGS, 
     $             EXDIGS, FDIGS, NOCNT, KLEN
      DOUBLE PRECISION  R1, R2, R3, ARTMP, RSUB, MINFIX, MAXFIX, X
      CHARACTER*1  FMTBUF(12), BUF(79),  ZERO,  BLNK,  POINT,  STAR
      CHARACTER*12 FMT
      CHARACTER*79 BUFSTR
      LOGICAL      FLOAT, FRAC, NEG, KILL, NOFIT
C
C     + + + LOCAL DEFINITIONS + + +
C     NOCNT  - for values less than 1.0, NOCNT is the number of
C              zeros immediately following the decimal point (which
C              "don't count" in terms of the number of significant
C              digits representable by the machine); for values
C              greater than or equal to 1.0, NOCNT = 0
C
C     + + + EQUIVALENCES + + +
      EQUIVALENCE (BUF,BUFSTR)
      EQUIVALENCE (FMT,FMTBUF)
C
C     + + + FUNCTIONS + + +
      INTEGER      DWDIGS
      DOUBLE PRECISION  FPART
      LOGICAL      NONZER, NZFRAC
C
C     + + + INTRINSICS + + +
      INTRINSIC    IABS, DABS, DLOG10, DBLE, LEN, INDEX, 
     $             IDINT, DINT, DNINT, IDNINT
C
C     + + + EXTERNALS + + +
      EXTERNAL     INTCHR, NUMINI, DWDIGS
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  I0, I1, I2,  ZERO, BLNK, POINT, STAR
     .    /  0,  1,  2,   '0',  ' ',   '.',  '*' /
C
C     + + + STATEMENT FUNCTION DEFINITIONS + + +
C     absolute value of the FRACtional PART (of a real number)
      FPART(X)  = DABS(X) - DABS(DINT(X))
C     NON-ZERo real number? - true or false  N.B. X should not be < 0.0
      NONZER(X) = X .GT. 0.0
C     Non-Zero FRACtional part of a real number? - true or false
      NZFRAC(X) = NONZER(FPART(X))
C     number of Whole DIGitS - for X > 0
Cmyg  WDIGS(X)  = IDINT(DLOG10(X*DP1MIN)) + 1
C
C     + + + END SPECIFICATIONS + + +
C
C     may need to init machine-dependent numeric constants (const.inc)
      CALL NUMINI
C
C     save time with the common case of DPRIN .eq. 0
      IF (DABS(DPRIN) .LE. D0MIN) THEN
         IF (LENGTH .GE. 3) THEN
            BUFSTR = '0.0'
         ELSE IF (LENGTH .EQ. 2) THEN
            BUFSTR = '0.'
         ELSE IF (LENGTH .EQ. 1) THEN
            BUFSTR = '0'
         ELSE
            NOFIT = .TRUE.
         END IF
      ELSE
C
C        the number of decimal digits of precision offered can be found
C        in the variable DPREC of the ICONST common block.
         PREC = DPREC
C
C        is the number negative?  set NEG and define MAXFIX/MINFIX
         IF (DPRIN .LT. 0.0) THEN
            NEG = .TRUE.
            MAXFIX = DBLE(10.0)**(LENGTH-1)
            MINFIX = DBLE(1.0/10.0)**(LENGTH-1)
         ELSE
            NEG = .FALSE.
            MAXFIX = DBLE(10.0)**LENGTH
            MINFIX = DBLE(1.0/10.0)**LENGTH
         END IF
C
C        this copy of the absolute value of the number is used frequently
         ARTMP = DABS(DPRIN)
C
C        using scientific notation, how many digits will be in the exponent?
C        the answer will be in EXDIGS.
         R1 = DLOG10(ARTMP*DP1MIN)
         EXPO = IDINT(R1)
         IF (R1 .LT. 0.0) THEN
            IF (NZFRAC(R1)) THEN
               EXPO = EXPO - 1
            END IF
         END IF
         IF (EXPO .EQ. 0) THEN
            EXDIGS = 1
         ELSE
            EXDIGS = IDINT(DLOG10(DBLE(IABS(EXPO))*DP1MIN)) + 1
         END IF
C
C        how many digits in the whole part of the number?
         IF (ARTMP .GE. 1.0) THEN
            WHDIGS = DWDIGS(ARTMP)
         ELSE
C           one digit to allow room for 0
            WHDIGS = 1
         END IF
C
         IF (ARTMP.LT.MINFIX .OR. ARTMP.GT.MAXFIX
     .      .OR. WHDIGS.GT.PREC) THEN
C           use scientific notation
            FLOAT = .FALSE.
C           force right justification with scientific notation
            JUST = 0
         ELSE
C           use floating point notation
            FLOAT  = .TRUE.
C           does the number have a non-zero fractional part?
            FRAC   = NZFRAC(DPRIN)
C           here, in the context of a floating point number, EXDIGS is
C           a count of EXtra whole DIGits (not digits in the exponent).
            EXDIGS = 0
            NOCNT  = 0
            IF (FRAC) THEN
C              if the fractional part were rounded, would the number of
C              of whole digits increase?  (EXtra whole DIGitS)
               IF (DNINT(FPART(DPRIN)) .GT. 0.0) THEN
C                  IF ((ARTMP+DBLE(1.0)) .GE. 1.0) THEN
                     IF (DWDIGS(ARTMP+DBLE(1.0)) .GT. WHDIGS) EXDIGS = 1
C                  END IF
               END IF
               IF (ARTMP .LT. 1.0) THEN
C                 since there is a non-zero fractional part (FRAC = TRUE),
C                 and the absolute value of the number is less than 1.0,
C                 how many consecutive zeros immediately follow the decimal
C                 point?  The NOCNT (no count) zeros don't count against the
C                 precision available.
                  R1 = DABS(DLOG10(ARTMP*DP1MIN))
                  IF (NZFRAC(R1)) THEN
                     NOCNT = IDINT (R1)
                  ELSE
                     NOCNT = IDINT (R1) - 1
                  END IF
               END IF
            END IF
         END IF
C
         IF (FLOAT) THEN
C           using floating point notation
            RSUB = DPRIN
C
C           first, assume it's not negative, taking into account the '.':
            FDIGS = LENGTH - (WHDIGS + 1)
            IF (FDIGS-NOCNT .GT. PREC) FDIGS = PREC+NOCNT
C           if it is negative, there will be less room for fractional digits
            IF (NEG) FDIGS = FDIGS - 1
C           if the number of WHole digits is one, it might just be a zero
C           in which case, for purposes of precision, we can consider the
C           number of whole digits to be zero.
            IF (WHDIGS .EQ. 1) THEN
C              if the whole number IS a zero, then maybe it would be better
C              to add an extra digit of precision to the fraction rather than
C              display the zero whole number.  However, don't do this if
C              the precision would be exceeded.
               IF (DINT(ARTMP) .LE. 0.0 .AND. FDIGS.LT.PREC+NOCNT) THEN
                  WHDIGS = 0
C                 we could just add that extra digit of precision to the
C                 fraction, but if it's just zero too, it looks better to
C                 keep the zero to the left of the decimal.
                  IF (FRAC) THEN
C                    r1 gets the number that we had planned to display, but
C                    raised to a power of 10 such that it is a whole number.
                     WRITE(99,*) 'ARTMP =',ARTMP
                     WRITE(99,*) 'LENGTH=',LENGTH
                     WRITE(99,*) 'FDIGS =',FDIGS 
                     WRITE(99,*) 'DBLE(10.0)**FDIGS =',DBLE(10.0)**FDIGS
                     WRITE(99,*) 'ARTMP*DBLE(10.0)**FDIGS =',
     $                            ARTMP*DBLE(10.0)**FDIGS
                     WRITE(99,*) 'DNINT(ARTMP*DBLE(10.0)**FDIGS) =',
     $                            DNINT(ARTMP*DBLE(10.0)**FDIGS)
                     WRITE(99,*) '--'
                     WRITE(99,*) ' '
                     R1 = DNINT(ARTMP*DBLE(10.0)**FDIGS)
C                    r2 gets a fraction representing digits which we had
C                    not planned to display and r3 is that fraction times 10.
                     R2 = (ARTMP*DBLE(10.0)**FDIGS) - R1
                     R3 = R2*10
C                    now, if 1) the number we were not going to display is,
C                    when rounded, non-zero, or if 2) the number passed in
C                    is negative, but everything we had planned on displaying
C                    was zero, then we add the extra digit of precision.
C                    in the first case, we just end up dropping the zero to
C                    the left of the decimal and adding an extra digit of
C                    precision; in the second case, the extra digit of
C                    precision is added only if the digits that were to be
C                    displayed previously were all zeros and the number
C                    itself is negative, so if the extra digit is zero, it
C                    is displayed to make up for the fact that the negative
C                    sign (for which room has been given) won't be displayed,
C                    but if the extra digit is non-zero, the negative sign
C                    will be displayed and the zero before the decimal will
C                    be lost.
                     IF (IDNINT(R3).NE.0 .OR. 
     .                  (NEG .AND. R1.LE.1.0D-20)) THEN
                           FDIGS  = FDIGS + 1
                     END IF
                  END IF
C                 if nocnt, the number of zero digits immediately
C                 following the decimal point (which don't count against the
C                 number of digits of precision representable by the machine),
C                 is greater than zero (this can only be the case if the
C                 absolute value of the number is less than one, as here)
C                 then we must ensure that the number of digits displayed
C                 does not exceed the actual machine precision
               END IF
            END IF
C
            IF ((WHDIGS + FDIGS - NOCNT) .GT. PREC) THEN
               FDIGS = PREC - WHDIGS + NOCNT
            END IF
C
            NOFIT = .FALSE.
            FMT = '(F  .  )'
            IF (FDIGS .GT. 0) THEN
C               expensive internal write replaced, but this is what it did
C               WRITE(FMT,96) LENGTH,FDIGS
               CALL INTCHR (LENGTH,I2,I0,KLEN,FMTBUF(3))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(6))
            ELSE IF (FDIGS .EQ. 0) THEN
               RSUB = DNINT (RSUB)
C               expensive internal write replaced, but this is what it did
C               WRITE(FMT,96) LENGTH,FDIGS
               CALL INTCHR (LENGTH,I2,I0,KLEN,FMTBUF(3))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(6))
            ELSE IF (FDIGS .EQ. -1) THEN
               RSUB = DNINT (RSUB)
               FDIGS = 0
C               expensive internal write replaced, but this is what it did
C               WRITE(FMT,96) LENGTH+1,FDIGS
               I = LENGTH + 1
               CALL INTCHR (I,I2,I0,KLEN,FMTBUF(3))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(6))
            ELSE
               NOFIT = .TRUE.
            END IF
C
         ELSE
C           using scientific notation
            RSUB  = DPRIN
            NOFIT = .FALSE.
            IF (NEG) THEN
               IF (EXDIGS+5 .LE. LENGTH) THEN
                  FDIGS = LENGTH-(EXDIGS+5)
                  IF (FDIGS+1 .GT. PREC) FDIGS = PREC - 1
               ELSE
                  NOFIT = .TRUE.
               END IF
            ELSE
               IF (EXDIGS+4 .LE. LENGTH) THEN
                  FDIGS = LENGTH-(EXDIGS+4)
                  IF (FDIGS+1 .GT. PREC) FDIGS = PREC - 1
               ELSE
                  NOFIT = .TRUE.
               END IF
            END IF
C            expensive internal write replaced, but this is what it did
C            IF (.NOT. NOFIT) WRITE(FMT,95) LENGTH,FDIGS,EXDIGS
            IF (.NOT. NOFIT) THEN
               FMT = '(1PE  .  E )'
               CALL INTCHR (LENGTH,I2,I0,KLEN,FMTBUF(5))
               CALL INTCHR (FDIGS,I2,I0,KLEN,FMTBUF(8))
               CALL INTCHR (EXDIGS,I1,I0,KLEN,FMTBUF(11))
            END IF
         END IF
C
C        here we either put asterisks or use 'write' to convert the number
         IF (NOFIT) THEN
            JLEN = 0
            DO 400 I = 1,LENGTH
               BUF(I) = STAR
  400       CONTINUE
         ELSE
C           some compilers don't like blanks embedded in the format string
C           as they are now, so first squeeze it together, and then use it
C           to format the given number.
            I = INDEX(FMT,BLNK)
            IF (I .GT. 0) THEN
               J = I
               L = LEN(FMT)
               DO 420 K = I,L
                  IF (FMTBUF(K) .NE. BLNK) THEN
                     FMTBUF(J) = FMTBUF(K)
                     J = J + 1
                  END IF
  420          CONTINUE
               DO 430 K = J,L
                  FMTBUF(K) = BLNK
  430          CONTINUE
            END IF
C
            I = INDEX(FMT,BLNK)
            IF (I .EQ. 0) I = LEN(FMT)
C
            WRITE(BUFSTR,FMT(1:I)) RSUB
C
         END IF
C
C        in either form, at most 1 trailing zero should be left.
C        make J point to the '.' if there is one
         J = INDEX (BUFSTR,POINT)
         IF (FLOAT) THEN
            IF (J .NE. 0) THEN
C              kludge to change " .0" to "0.0" in some cases on some 'chines
               IF ((J-1) .GT. 0) THEN
                  IF (BUF(J-1) .EQ. BLNK) THEN
                     BUF(J-1) = ZERO
                  END IF
               END IF
C              we don't want to truncate any zeros if there's no '.'
C              make K point to the last zero (if there is one)
               I = 0
               K = 0
  510          CONTINUE
               I = INDEX (BUFSTR(K+1:79),ZERO)
               IF (I .NE. 0) THEN
                  K = K + I
                  GO TO 510
               END IF
               KILL = .TRUE.
               IF (K .NE. 79) THEN
C                 only kill zeros that are not significant.  if the last
C                 zero is followed by something other than a blank,
C                 then you can't get rid of it.
                  IF (BUF(K+1) .NE. BLNK) KILL = .FALSE.
               END IF
               IF (K .GT. J+1 .AND. KILL) THEN
C                 only truncate zeros if they're to the right of the decimal
C                 and don't get rid of a zero that is immediately to the
C                 right of the decimal point.
                  DO 520 I = K,J+2,-1
                     IF (BUF(I) .EQ. ZERO) THEN
                        BUF(I) = BLNK
                     ELSE
C                       once a non-zero is found, get out of the loop
                        GO TO 525
                     END IF
  520             CONTINUE
  525             CONTINUE
               END IF
            END IF
         ELSE
C           get rid of trailing zeros in scientific notation
            IF (J .NE. 0) THEN
C              we don't won't to truncate any zeros if there's no '.'
C              make L point to the 'E'
               L = INDEX (BUFSTR,'E')
               IF (L .NE. 0) THEN
C                 make K point to the last digit before the 'E'
                  K = L - 1
                  IF (K .GT. J+1) THEN
C                    only truncate zeros between '.' and 'E', and don't
C                    get rid of the zero (if there is one) immediately to the
C                    right of the decimal.
                     M = J+2
                     DO 620 I = K,J+2,-1
                        IF (BUF(I) .EQ. ZERO) THEN
C                          don't truncate the last trailing zero
                           BUF(I) = BLNK
                        ELSE
                           M = I + 1
                           GO TO 625
                        END IF
  620                CONTINUE
  625                CONTINUE
C                    a non-zero was found, slide the exponent down to
C                    fill in the gap left by the truncated zeros.
                     DO 630 I = M,LENGTH
                        IF (L .LE. LENGTH) THEN
                           BUF(I) = BUF(L)
                        ELSE
                           BUFSTR(I:LENGTH) = BLNK
                           GO TO 635
                        END IF
                        L = L + 1
  630                CONTINUE
  635                CONTINUE
                  END IF
               END IF
            END IF
         END IF
      END IF
C
C     copy the final string, blanks and all
      IF (JUST .NE. 0) THEN
C        left justified (i.e., JUST != 0)
         J = 1
         DO 710 I = 1,LENGTH
            IF (BUF(J) .NE. BLNK) GO TO 715
            J = J + 1
  710    CONTINUE
  715    CONTINUE
         JLEN = INDEX (BUFSTR(J:LENGTH),BLNK)
         IF (JLEN .EQ. 0) THEN
            JLEN = LENGTH - J + 1
         ELSE
            JLEN = JLEN - 1
         END IF
         K = 1
         DO 720 I = J,LENGTH
            STR(K) = BUF(I)
            K = K + 1
  720    CONTINUE
         DO 730 I = K,LENGTH
            STR(I) = BLNK
  730    CONTINUE
      ELSE
C        right justified (i.e., JUST == 0)
         J = LENGTH
         DO 810 I = LENGTH,1,-1
            IF (BUF(J) .NE. BLNK) GO TO 815
            J = J - 1
  810    CONTINUE
  815    CONTINUE
         DO 820 I = 1,LENGTH-J
            STR(I) = BLNK
  820    CONTINUE
         K = 1
         DO 830 I = LENGTH-J+1,LENGTH
            STR(I) = BUF(K)
            K = K + 1
  830    CONTINUE
C        with right justified strings, JLEN is always LENGTH
         JLEN = LENGTH
      END IF
C
      RETURN
C
C      expensive internal writes were replaced, but this is what they did
C  95  FORMAT('(1PE',I2,'.',I2,'E',I1,')')
C  96  FORMAT('(F',I2,'.',I2,')')
C
      END
C
C
C
      INTEGER   FUNCTION   RWDIGS
     I                           (RVAL)
C
C     + + + PURPOSE + + +
C     RWDIGS returns the number of whole digits in the real value.  The
C     real value is expected to be greater than 0, no check is made to
C     verify that this is true.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL         RVAL
C
C     + + + ARGUMENT DEFINITIONS + + +
C     RVAL   - real value
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      LOGVAL
C
C     + + + INTRINSICS + + +
      INTRINSIC    INT, ALOG10
C
C     + + + END SPECIFICATIONS + + +
C
      LOGVAL = INT(ALOG10(RVAL))
      IF (.NOT. (RVAL.GE.10.**LOGVAL .AND. RVAL.LT.10.**(LOGVAL+1)))
     $  LOGVAL = LOGVAL + 1
C
      RWDIGS = LOGVAL + 1
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   DWDIGS
     I                           (DVAL)
C
C     + + + PURPOSE + + +
C     DWDIGS returns the number of whole digits in the double precision
C     value.  The value is expected to be greater than 0, no check is 
C     made to verify that this is true.
C
C     + + + DUMMY ARGUMENTS + + +
      DOUBLE PRECISION  DVAL
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DVAL   - double precision value
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      LOGVAL
C
C     + + + INTRINSICS + + +
      INTRINSIC    IDINT, DLOG10
C
C     + + + END SPECIFICATIONS + + +
C
      LOGVAL = IDINT(DLOG10(DVAL))
      IF (.NOT. (DVAL.GE.10.D0**LOGVAL .AND. DVAL.LT.10.D0**(LOGVAL+1)))
     $  LOGVAL = LOGVAL + 1
C
      DWDIGS = LOGVAL + 1
C
      RETURN
      END
C
C
C
      SUBROUTINE   DECCHX
     I                    (REAIN,LEN,SIGDIG,DECPLA,
     O                     STR)
C
C     + + + PURPOSE + + +
C     Converts a real number to a character array.  The number is right
C     justified in the array.  The number will be represente with SIGDIG
C     significant digits and DECPLA decimal places, if there is room in
C     STR.  If there is not room, the number will be represented using
C     exponential notation.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN,SIGDIG,DECPLA
      REAL        REAIN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     REAIN  - real value to be converted to a character array
C     LEN    - available size for output character array
C     SIGDIG - significant digits for output
C     DECPLA - number of decimal places for output
C               0 - no decimal places
C              <0 - force exponential output
C     STR    - output character array of size LEN
C
C     + + + COMMON BLOCKS + + +
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      I,J,K,L,JUST,TSDIG,DPTFG,LDZRO,ITMP,MCRY,FORCE,
     1             LDECPL,EXPFLG,FRSTD,L20,L1
      REAL         RWRIT
      CHARACTER*20 UBBUF
      CHARACTER*9  UFMTL
      CHARACTER*1  UFMT(9),USBUF(20),C1
C
C     + + + EQUIVALENCES + + +
      EQUIVALENCE (UBBUF,USBUF),(UFMTL,UFMT)
C
C     + + + FUNCTIONS + + +
      INTEGER      CHRDIG
      CHARACTER*1  DIGCHR
C
C     + + + INTRINSICS + + +
      INTRINSIC   ABS
C
C     + + + EXTERNALS + + +
      EXTERNAL   CHRCHR, CHRDIG, DIGCHR, INTCHR, CHRDEL, CHRINS
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  L1,L20/1,20/,  C1/'1'/
C
C     + + + END SPECIFICATIONS + + +
C
      TSDIG= SIGDIG
      IF (TSDIG.EQ.0) TSDIG= 5
C
      IF (DECPLA.LT.0) THEN
C       force exponential instead of zero
        FORCE = 1
        LDECPL= -DECPLA
      ELSE
C       dont force exponential
        FORCE = 0
        LDECPL= DECPLA
      END IF
      JUST= 0
C
      UFMT(1)= '('
      UFMT(2)= ' '
      UFMT(3)= ' '
      UFMT(4)= 'F'
      I= 2
      CALL INTCHR (LEN,I,JUST,J,UFMT(5))
      UFMT(7)= '.'
      UFMT(8)= DIGCHR(LDECPL)
      UFMT(9)= ')'
C
      WRITE (UBBUF,UFMTL) REAIN
      IF (CHRDIG(USBUF(1)).LT.0.AND.USBUF(1).NE.' '.AND.
     1    USBUF(1).NE.'.'     .AND.USBUF(1).NE.'-') THEN
C       real number didn't fit in user specified format, try E
        EXPFLG = 1
      ELSE IF (FORCE.EQ.1) THEN
C       check to see if value written is zero
        READ (UBBUF,UFMTL) RWRIT
Chnb is following condition appropriate (for extremely large magnitude numbers
Chnb that might have a large absolute difference?)
        IF ((ABS(RWRIT).LT.1.0E-10).OR.ABS(RWRIT-REAIN).GT.1.E-6)
     1    EXPFLG= 1
      ELSE
C       no exponent needed here
        EXPFLG = 0
      END IF
      IF (EXPFLG.EQ.1) THEN
C       try exponential format
        UFMT(2)= '1'
        UFMT(3)= 'P'
        UFMT(4)= 'E'
        I= LEN- 6
        IF (REAIN.LT.0) I= I- 1
        IF (TSDIG-1.LT.I) THEN
C         don't want too many digits in exp output
          I= TSDIG- 1
        END IF
        UFMT(8)= DIGCHR(I)
        WRITE (UBBUF,UFMTL) REAIN
      ELSE IF (TSDIG.GT.0) THEN
C       it fit, remove insignificant digits, if any
        I= 0
        J= 0
        DPTFG= 0
        LDZRO= 1
        FRSTD= 0
 10     CONTINUE
          I= I+ 1
C         have we passed the decimal point
          IF (USBUF(I).EQ.'.') THEN
            DPTFG= I
          ELSE
C           are we on a digit?
            ITMP= CHRDIG(USBUF(I))
            IF (ITMP.GT.0.OR.(ITMP.EQ.0.AND.LDZRO.EQ.0)) THEN
C             set counter for first 0-9 digit
              IF (FRSTD .EQ. 0) FRSTD= I
              LDZRO= 0
C             J is counter for significant digits
              J= J+ 1
              IF (J.GT.TSDIG) THEN
C               we have enough significant digits
                IF (ITMP.GE.5.AND.J.EQ.TSDIG+1) THEN
C                 we need to carry from the first insig digit
                  K= I- 1
C                 K is next digit back
 20               CONTINUE
                    IF (K.EQ.DPTFG) K= K-1
                    L= CHRDIG(USBUF(K))+ 1
                    IF (L.EQ.10) THEN
                      IF (K .EQ. FRSTD) THEN
C                       need to shift over since 9 rounded to 10
                        CALL CHRINS (L20,K,C1,USBUF)
                        MCRY = 0
                        L = 0
                        IF (USBUF(1) .EQ. ' ') THEN
C                         shift back if 1st char blank
                          CALL CHRDEL (L20, L1, USBUF)
                          FRSTD = FRSTD - 1
                        ELSE
C                         can't shift so reset pointers
                          I = I + 1
                          K = K + 1
                        END IF
                      ELSE
                        MCRY= 1
                        L= 0
                      END IF
                    ELSE
                      MCRY= 0
                    END IF
                    USBUF(K)= DIGCHR(L)
                    K= K- 1
                  IF (MCRY.EQ.1) GO TO 20
                END IF
C               now get rid of insig digits
                IF (DPTFG.EQ.0) THEN
C                 before the decimal point
                  USBUF(I)= '0'
                ELSE
C                 after the decimal point
                  USBUF(I)= ' '
                END IF
              END IF
            END IF
          END IF
        IF (I.LT.LEN) GO TO 10
      END IF
C
C     fill in output buffer
      CALL CHRCHR (LEN,USBUF,STR)
C
      RETURN
      END
C
C
C
      CHARACTER*1 FUNCTION   DIGCHR
     I                             (DIG)
C
C     DIGCHR returns the character equivalent of a single digit.  If
C     the integer provided is not within the valid range 0 thru 9,
C     DIGCHR will return a zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   DIG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DIG    - digit to be converted to a character
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I
      CHARACTER*1 TDIG(9),TZERO
C
C     + + + DATA INITIALIZATIONS + + +
      DATA TDIG  /'1','2','3','4','5','6','7','8','9'/
      DATA TZERO /'0'/
C
C     + + + END SPECIFICATIONS + + +
C
      I = 1
      DIGCHR = TZERO
  10  CONTINUE
        IF (I.EQ.DIG) DIGCHR = TDIG(I)
        I = I + 1
      IF (I.LT.10.AND.DIGCHR.EQ.TZERO) GO TO 10
C
      RETURN
      END
C
C
C
      SUBROUTINE   INTCHR
     I                    (INTIN,LENA,JUST,
     O                     JLEN,STRNG)
C
C     + + + PURPOSE + + +
C     Converts an integer number to a character array.  The number is
C     left or right justified in the array based on the value of JUST.
C     The maximum number of digits for the integer is 9.  For numbers
C     that are left justified, STRNG is padded with trailing blanks.
C     For numbers that are right justified, STRNG is padded with leading
C     blanks.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LENA,INTIN,JLEN,JUST
      CHARACTER*1 STRNG(LENA)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     INTIN  - integer value to be converted to a character array
C     LENA   - available size for output character array
C     JUST   - output justification
C              0 - right justified in the field
C              1 - left justified in the field
C     JLEN   - actual number of characters output to character array
C     STRNG  - output character array of size LENA
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I,IM,IST,NUMCHR, LEN
      INTEGER*4   JTMP1,JTMP2,PTEN,CHKSIZ,REMTMP,INTTMP
      CHARACTER*1 BLNK,ZERO,MINUS,FILCHR,STR(9)
C
C     + + + FUNCTIONS + + +
      CHARACTER*1 DIGCHR
C
C     + + + INTRINSICS + + +
      INTRINSIC   ALOG10, INT, MOD, REAL
C
C     + + + EXTERNALS + + +
      EXTERNAL    DIGCHR, ZIPC, COPYC
C
C     + + + DATA INITIALIZATIONS + + +
      DATA  MINUS, BLNK, ZERO, FILCHR
     &     /  '-',  ' ',  '0',    '*' /
C
C     + + + END SPECIFICATIONS + + +
C
      INTTMP = INTIN
      LEN  = LENA
      IF (LEN.GT.9) LEN = 9
      CALL ZIPC (LEN,BLNK,STR)
      IF (INTTMP.EQ.0) THEN
C       process zero
        IF (JUST.EQ.0) THEN
          STR(LEN) = ZERO
          JLEN = LEN
        ELSE
          STR(1) = ZERO
          JLEN = 1
        END IF
      ELSE
        IST = 1
        IF (INTTMP.LT.0) THEN
          IST   = 2
          INTTMP= -INTTMP
          STR(1)= MINUS
        END IF
Chnb  guess what, the integral portion of the log base 10 of a number
Chnb  like 9,999,999 is different than that for 9,999,999*1.00001
Chnb  (therefore, the following code won't work!)
        NUMCHR = INT (ALOG10(REAL(INTTMP)*1.00001))
        IF (JUST.EQ.0) THEN
          JLEN = LEN
        ELSE
          JLEN = NUMCHR + IST
          IF (JLEN.GT.LEN) JLEN = LEN
        END IF
C                 **** ADDED BECAUSE OF HP BUG IN SSB #5838 ***
        JTMP1 = 10
        JTMP2 = (JLEN - IST + 1)
C       IF (JTMP2.GT.4) WRITE (*,'("INTCHR,JTMP2:",I8)') JTMP2
        PTEN = JTMP1** JTMP2
        CHKSIZ = INTTMP- PTEN
        IF (CHKSIZ.GE.0) THEN
C         resulting string will be too long- fill with stars
          CALL ZIPC (JLEN,FILCHR,STR)
        ELSE
Chnb  Enough is enough.  I hadn't said anything to this point, but
Chnb  filling the character array with the most significant digit
Chnb  first is just too ridiculous.  Change this to
Chnb  "DO 130 I = JLEN,IST,-1" or something like that.
          DO 130 I = IST, JLEN
            PTEN = PTEN/ 10
            REMTMP = MOD(INTTMP,PTEN)
            IM = (INTTMP-REMTMP)/PTEN
            STR(I) = DIGCHR(IM)
            IM = I- 1
            IF (IM.GT.0) THEN
              IF (STR(I).EQ.ZERO) THEN
                IF (STR(IM).EQ.BLNK.OR.STR(IM).EQ.MINUS) THEN
                  STR(I) = STR(IM)
                  STR(IM)= BLNK
                END IF
              END IF
            ELSE
              IF (STR(1).EQ.ZERO) STR(1) = BLNK
            END IF
            INTTMP = REMTMP
 130      CONTINUE
        END IF
      END IF
C
      CALL ZIPC ( LENA, BLNK, STRNG )
      IF (LEN .EQ. LENA) THEN
C       available field length within max allowed length
        CALL COPYC ( LEN, STR, STRNG )
      ELSE IF (JUST .NE. 0) THEN
C       available field length bigger than max, left justify
        CALL COPYC ( LEN, STR, STRNG )
      ELSE
C       available field length bigger than max, right justify
        I = LENA - LEN + 1
        CALL COPYC (LEN, STR, STRNG(I) )
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   ADCOMA
     I                   (LEN,
     M                    STR)
C
C     + + + PURPOSE + + +
C     This routine places a comma(s) in the real number in the string
C     every 3 digits.  String may include sign, decimal point, and
C     decimal digits.  If there is not enough room in the string, the
C     string is returned as it was entered.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STR
C     STR    - character array of size LEN
C
C     + + + PARAMETERS + + + 
      INTEGER   MAXTMP
      PARAMETER (MAXTMP=50)
C 
C     + + + LOCAL VARIABLES + + + 
      INTEGER   LOC,I1,NEG,SHFT,OLEN,I,IDIG,IOLD,INEW,JUST
      CHARACTER*1 TEMP(MAXTMP),DOT,MINUS,BLNK
C
C     + + + FUNCTIONS + + + 
      INTEGER  LENSTR, STRFND
C
C     + + + EXTERNAL + + + 
      EXTERNAL  LENSTR, LFTSTR, RHTSTR, STRFND, ZIPC, COPYC
C
C     + + + DATA INITIALIZATIONS + + + 
      DATA MINUS/'-'/,  DOT/'.'/,  BLNK/' '/, I1/1/
C
C     + + + END SPECIFICATIONS + + + 
C
C     check if right of left justified
      IF (STR(LEN) .EQ. ' ') THEN
C       assume string was left justified
        JUST = 0
      ELSE
        JUST = 1
      END IF
C     shift to left
      CALL LFTSTR (LEN,STR)
C     locate position of decimal point if any
      LOC = STRFND (LEN,STR,I1,DOT)
C     check for minus sign
      IF (STR(1) .EQ. MINUS) THEN               
        NEG = 1
      ELSE
        NEG = 0
      END IF
C     determine length of string
      OLEN = LENSTR (LEN,STR) 
C     reset LOC for cases of no decimal point
      IF (LOC .LE. 0) LOC = OLEN + 1
C     determine number of commas needed, 10000 or greater
      SHFT = (LOC-2-NEG)/3
C     blank out temp buffer
      CALL ZIPC (MAXTMP,BLNK,TEMP)        
C
      IF (OLEN+SHFT .LE. LEN) THEN
C       sufficient room for commas
        IF (OLEN+SHFT .LE. MAXTMP) THEN
C         sufficient room for temp buffer
          IDIG = 0
          DO 10 I = 1,OLEN
C           loop thru the character string, right to left
C           IOLD is position in string without commas
C           INEW is position in string with commas
            IOLD = OLEN - I + 1
            INEW = IOLD + SHFT
            IF (IOLD .GE. LOC) THEN
C             found decimal digit or point, just shift over
              TEMP(INEW) = STR(IOLD)
            ELSE IF (STR(IOLD) .EQ. '-') THEN
C             INEW should be equal to IOLD because SHFT should be 0
              TEMP(INEW) = STR(IOLD)
            ELSE IF (IDIG  .LT. 3) THEN
C             within cluster of 3, not ready to add comma
              TEMP(INEW) = STR(IOLD)
C             count digits
              IDIG = IDIG + 1
            ELSE
C             must be time to add a comma, IDIG = 3
              TEMP(INEW) = ','
              TEMP(INEW-1) = STR(IOLD)
              IDIG = 1
              SHFT = SHFT - 1
            END IF
 10       CONTINUE
C
C         replace with new string
          CALL COPYC (LEN,TEMP,STR)
          IF (JUST .EQ. 1 .AND. LEN .GT. SHFT+OLEN) THEN
C           was right justified and blank spaces remain
C           so right justify
            CALL RHTSTR (LEN,STR)
          END IF
        END IF
      END IF
C
      RETURN
      END                   
C
C
C
      SUBROUTINE   CHDECE
     I                   (LEN,STR,
     O                    RVAL,ERRFLG)
C
C     + + + PURPOSE + + +
C     Convert the character array STR to its real equivalent.  STR is
C     expected to contain a right-justified number and may include a
C     sign, decimal point, exponent, and decimla digits (+, -, ., D, E,
C     and 0-9).  If the character array contains an invalid character
C     or cannot be converted ERRFLG will be returned with a value of 1,
C     RVAL is set to -ROMAX in this case.  (Note:  ROMAX, the largest
C     representable number, is determined in subroutine NUMINI.  It is
C     machine dependent.)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN,ERRFLG
      REAL        RVAL
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STR
C     STR    - character array containing real value
C     RVAL   - real value
C     ERRFLG - flag indicating success of conversion
C              0 - successful
C              1 - unsuccessful
C
C     + + + COMMON BLOCKS + + +
C     numeric constants
      INCLUDE 'const.inc'
C
C     + + + LOCAL VARIABLES + + +
      REAL       ERRREA
C
C     + + + FUNCTIONS + + +
      REAL       CHRDEC
C
C     + + + EXTERNALS + + +
      EXTERNAL   CHRDEC, NUMINI
C
C     + + + END SPECIFICATIONS + + +
C
C     may need to init maching dependent numeric constants (const.inc)
      CALL NUMINI
C
      ERRREA = -R0MAX
      RVAL  = CHRDEC (LEN,STR)
      ERRFLG= 0
      IF (RVAL.LE.ERRREA) THEN
C       ivalid real
        ERRFLG= 1
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   CHINTE
     I                   (LEN,STR,
     O                    IVAL,ERRFLG)
C
C     + + + PURPOSE + + +
C     Convert the character array STR to its integer equivalent.  STR is
C     expected to contain a right-justified integer value and may include
C     a sign and decimal digits (+, -, and 0-9).  If the character array
C     contains an invalide character or cannot be converted, ERRFLG will
C     be returned with a value of 1, IVAL is set to 0 in this case.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN,ERRFLG,IVAL
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STR
C     STR    - character array of size LEN containing an integer value
C     IVAL   - integer value
C     ERRFLG - flag indicating success of conversion
C              0 - successful
C              1 - unsuccessful
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     DIG,I,SIGN
      CHARACTER*1 CDIG
C
C     + + + FUNCTIONS + + +
      INTEGER     CHRDIG
C
C     + + + EXTERNALS + + +
      EXTERNAL    CHRDIG
C
C     + + + END SPECIFICATIONS + + +
C
      IVAL  = 0
      ERRFLG= 0
      SIGN  = 1
      I = 1
  10  CONTINUE
        CDIG = STR(I)
        IF ((CDIG.EQ.'-' .OR. CDIG.EQ.'+' .OR. CDIG.EQ.' ') .AND.
     1       IVAL.EQ.0) THEN
C         replace blank, - sign, or + sign with a 0 if no number yet
          IF (CDIG.EQ.'-') THEN
C           change sign
            SIGN = -1
          END IF
          CDIG = '0'
        END IF
        DIG = CHRDIG(CDIG)
        IVAL= IVAL* 10+ DIG
        I = I + 1
      IF (I.LE.LEN .AND. DIG.GE.0) GO TO 10
C
      IF (DIG.EQ.-1) THEN
C       invalid integer
        ERRFLG= 1
      END IF
C
      IVAL= IVAL* SIGN
C
      RETURN
      END
C
C
C
      REAL   FUNCTION   CHRDEC
     I                         (LEN,STR)
C
C     + + + PURPOSE + + +
C     Convert the character array STR to its real equivalent.  STR is
C     expected to contain a right-justified number and may include a
C     sign, decimal point, exponent, and decimal digits (+, -, ., D, E,
C     and 0-9).  If the character array contains an invalid character
C     or cannot be converted CHRDEC will return -ROMAX.  (Note:  ROMAX,
C     the largest representable number, is determined in subroutine
C     NUMINI.  It is machine dependent.)  Leading blanks are ignored.
C     Trailing blanks are treated as zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of array STR
C     STR    - character array of size LEN containing a real value
C
C     + + + COMMON BLOCKS + + +
C     numeric constants
      INCLUDE 'const.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     CHKORD(6,6),ERFLG,EXPFLG,KIND,OKIND,POS,FRCFLG
      REAL        DECSGN,EXPSGN,EXPON,FRC,SUM,TEN,TVAL,ERRREA
      CHARACTER*1 DIG
C
C     + + + FUNCTIONS + + +
      REAL      RNOP
      INTEGER   CHRDIG
C
C     + + + EXTERNALS + + +
      EXTERNAL   CHRDIG, NUMINI
C
C     + + + DATA INITIALIZATIONS + + +
C
C                   DOWN OKIND, ACROSS KIND
      DATA CHKORD /
     1              1, 2, 0, 4, 0, 6,
     2              1, 2, 0, 4, 0, 6,
     3              1, 2, 3, 0, 0, 6,
     4              0, 0, 3, 4, 0, 6,
     5              1, 2, 0, 0, 0, 6,
     6              0, 0, 0, 0, 0, 0/
C
C     + + + STATEMENT FUNCTION DEFINITIONS + + +
C     No OPperation, used to keep Ryan/McFarland optimization honest
      RNOP(ERRREA) = ERRREA
C
C     + + + END SPECIFICATIONS + + +
C
C     may need to init maching dependent numeric constants (const.inc)
      CALL NUMINI
C
      ERRREA = -R0MAX
      ERFLG = 0
      EXPFLG = 0
      FRCFLG = 0
      OKIND = 6
      POS = 1
      EXPSGN = 1.0
      DECSGN = 1.0
      TEN = 10.
      FRC = 1.0
      SUM = 0.0
      EXPON = 0.0
C
  10  CONTINUE
        DIG = STR(POS)
        KIND = 0
        IF (DIG.EQ.'+') KIND = 1
        IF (DIG.EQ.'-') KIND = 2
        IF (DIG.EQ.'.') KIND = 3
        IF (DIG .EQ. 'D' .OR. DIG .EQ. 'd' .OR.
     1      DIG .EQ. 'E' .OR. DIG .EQ. 'e') KIND = 4
C       treat a blank as a zero
        IF (DIG.EQ.' ') THEN
          IF (OKIND.EQ.6) THEN
C           leading blank
            KIND = 6
          ELSE
C           not a leading blank - assume zero
            KIND = 5
            TVAL = 0
          END IF
        END IF
C
        IF (KIND.EQ.0) THEN
          TVAL = CHRDIG(DIG)
          IF (TVAL.GE.0.0) KIND = 5
        END IF
C
        IF (KIND.EQ.0) THEN
          ERFLG = 5
        ELSE
          ERFLG = CHKORD(KIND,OKIND)
        END IF
C
 40     CONTINUE
        IF (ERFLG.NE.0) THEN
C         this is a quick exit
          CHRDEC = ERRREA
          RETURN
        END IF
C
        GO TO (110,120,130,140,150,160),KIND
C
 110    CONTINUE
C         process a plus sign
          GO TO 200
C
 120    CONTINUE
C         process a minus sign
          IF (EXPFLG.EQ.0) THEN
            DECSGN = -1.0
          ELSE
            EXPSGN = -1.0
          END IF
          GO TO 200
C
 130    CONTINUE
C         process a decimal point
          IF (EXPFLG.EQ.1) THEN
            ERFLG = 3
            GO TO 40
          END IF
C
C         no fractional exponents allowed
          IF (FRCFLG.EQ.1) THEN
            ERFLG = 3
            GO TO 40
          END IF
C
C         no double decimal point
          FRCFLG = 1
          GO TO 200
C
 140    CONTINUE
C         process an exponent (E, e, D, or d)
          IF (EXPFLG.EQ.1) THEN
            ERFLG = 4
            GO TO 40
          END IF
C
C         no double exponent
          EXPFLG = 1
          GO TO 200
C
 150    CONTINUE
C         process a number
          IF (EXPFLG.EQ.0) THEN
            IF (FRCFLG.EQ.0) THEN
              SUM = (SUM* TEN) + TVAL
            ELSE
              FRC = FRC/ TEN
              FRC = RNOP(FRC)
              SUM = SUM + (FRC* TVAL)
            END IF
          ELSE
            EXPON = (EXPON* TEN) + TVAL
          END IF
          GO TO 200
C
 160    CONTINUE
C         process a leading blank
          GO TO 200
C
 200    CONTINUE
C       repeat until finished
        OKIND = KIND
        POS = POS + 1
      IF (POS.LE.LEN) GO TO 10
      CHRDEC = DECSGN* SUM* TEN** (EXPSGN* EXPON)
C
      RETURN
      END
C
C
C
      DOUBLE PRECISION   FUNCTION   CHRDPR
     I                                    (LEN,STR)
C
C     + + + PURPOSE + + +
C     Convert the character array STR to its double precision equivalent.
C     STR is expected to contain a right-justified number and may include
C     a sign, decimal point, exponent, and decimal digits (+, -, ., D, E,
C     and 0-9).  If the character array contains an invalid character or
C     cannot be converted CHRDPR will return -DOMAX.  (Note:  DOMAX, the
C     largest representable double precission number, is determined in
C     subroutine NUMINI.  It is machine dependent.)  Leading blanks areC
C     ignored.  Trailing blanks are treated as zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of array STR
C     STR    - character array of length LEN containing a real value
C
C     + + + COMMON BLOCKS + + +
C     numeric constants
      INCLUDE 'const.inc'
C
C     + + + LOCAL VARIABLES + + +
      REAL*8      DECSGN,EXPSGN,EXPON,FRC,SUM,TEN,TVAL
      INTEGER     CHKORD(6,6),ERFLG,EXPFLG,KIND,OKIND,POS,FRCFLG
      CHARACTER*1 DIG
      DOUBLE PRECISION  ERRREA
C
C     + + + FUNCTIONS + + +
      INTEGER   CHRDIG
C
C     + + + EXTERNALS + + +
      EXTERNAL   CHRDIG, NUMINI
C
C                   DOWN OKIND, ACROSS KIND
      DATA CHKORD /
     1              1, 2, 0, 4, 0, 6,
     2              1, 2, 0, 4, 0, 6,
     3              1, 2, 3, 0, 0, 6,
     4              0, 0, 3, 4, 0, 6,
     5              1, 2, 0, 0, 0, 6,
     6              0, 0, 0, 0, 0, 0/
C
C     + + + END SPECIFICATIONS + + +
C
C     may need to init maching dependent numeric constants (const.inc)
      CALL NUMINI
C
      ERRREA = -D0MAX
      ERFLG = 0
      EXPFLG = 0
      FRCFLG = 0
      OKIND = 6
      POS = 1
      EXPSGN = 1.0
      DECSGN = 1.0
      TEN = 10.
      FRC = 1.0
      SUM = 0.0
      EXPON = 0.0
C
  10  CONTINUE
        DIG = STR(POS)
        KIND = 0
        IF (DIG.EQ.'+') KIND = 1
        IF (DIG.EQ.'-') KIND = 2
        IF (DIG.EQ.'.') KIND = 3
        IF (DIG .EQ. 'D' .OR. DIG .EQ. 'd' .OR.
     1      DIG .EQ. 'E' .OR. DIG .EQ. 'e') KIND = 4
C       treat a blank as a zero
        IF (DIG.EQ.' ') THEN
          IF (OKIND.EQ.6) THEN
C           leading blank
            KIND = 6
          ELSE
C           not a leading blank - assume zero
            KIND = 5
            TVAL = 0
          END IF
        END IF
C
        IF (KIND.EQ.0) THEN
          TVAL = CHRDIG(DIG)
          IF (TVAL.GE.0.0) KIND = 5
        END IF
C
        IF (KIND.EQ.0) THEN
          ERFLG = 5
        ELSE
          ERFLG = CHKORD(KIND,OKIND)
        END IF
C
 40     CONTINUE
        IF (ERFLG.NE.0) THEN
C         this is a quick exit
          CHRDPR = ERRREA
          RETURN
        END IF
C
        GO TO (110,120,130,140,150,160),KIND
C
 110    CONTINUE
C         process a plus sign
          GO TO 200
C
 120    CONTINUE
C         process a minus sign
          IF (EXPFLG.EQ.0) THEN
            DECSGN = -1.0
          ELSE
            EXPSGN = -1.0
          END IF
          GO TO 200
C
 130    CONTINUE
C         process a decimal point
          IF (EXPFLG.EQ.1) THEN
            ERFLG = 3
            GO TO 40
          END IF
C
C         no fractional exponents allowed
          IF (FRCFLG.EQ.1) THEN
            ERFLG = 3
            GO TO 40
          END IF
C
C         no double decimal point
          FRCFLG = 1
          GO TO 200
C
 140    CONTINUE
C         process an exponent (E, e, D, or d)
          IF (EXPFLG.EQ.1) THEN
            ERFLG = 4
            GO TO 40
          END IF
C
C         no double exponent
          EXPFLG = 1
          GO TO 200
C
 150    CONTINUE
C         process a number
          IF (EXPFLG.EQ.0) THEN
            IF (FRCFLG.EQ.0) THEN
              SUM = (SUM* TEN) + TVAL
            ELSE
              FRC = FRC/ TEN
              SUM = SUM + (FRC* TVAL)
            END IF
          ELSE
            EXPON = (EXPON* TEN) + TVAL
          END IF
          GO TO 200
C
 160    CONTINUE
C         process a leading blank
          GO TO 200
C
 200    CONTINUE
C       repeat until finished
        OKIND = KIND
        POS = POS + 1
      IF (POS.LE.LEN) GO TO 10
      CHRDPR = DECSGN* SUM* TEN** (EXPSGN* EXPON)
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   CHRDIG
     I                           (CHR)
C
C     + + + PURPOSE + + +
C     CHRDIG returns the integer equivalent of a single character.  The
C     expected characters are '0' - '9'.  If CHR contains a character
C     other than expected, CHRDIG returns a -1.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*1 CHR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     CHR    - a single character
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     I
      CHARACTER*1 TDIG(10)
C
C     + + + DATA INITIALIZATIONS + + +
      DATA TDIG  /'0','1','2','3','4','5','6','7','8','9'/
C
C     + + + END SPECIFICATIONS + + +
C
      CHRDIG = -1
      I = 1
  10  CONTINUE
        IF (CHR.EQ.TDIG(I)) CHRDIG = I- 1
        I = I + 1
      IF (CHRDIG.EQ.-1.AND.I.LE.10) GO TO 10
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   CHRINT
     I                           (LEN,STR)
C
C     + + + PURPOSE + + +
C     CHRINT returns the integer equivalent of the character array STR.
C     STR is expected to contain a right-justified integer value and may
C     include a sign and digits (+, -, and 0-9).  CHRINT returns a zero
C     if the character array contains an invalid character or cannot be
C     converted.  Leading blanks are ignored and trailing blanks are
C     treated as a zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STR
C     STR    - character array of size LEN containing an integer value
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     ERRINT
C
C     + + + FUNCTIONS + + +
      INTEGER     CRINTE
C
C     + + + EXTERNALS + + +
      EXTERNAL   CRINTE
C
C     + + + END SPECIFICATIONS + + +
C
      ERRINT= 0
      CHRINT= CRINTE(ERRINT,LEN,STR)
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   CRINTE
     I                           (ERRINT,LEN,STR)
C
C     + + + PURPOSE + + +
C     CRINTE returns the integer equivalent of the character array STR.
C     STR is expected to contain a right-justified integer value and may
C     include a sign and digits (+, -, and 0-9).  CRINTE returns ERRINT
C     if the character array contains an invalid character or cannot be
C     converted.  Leading blanks are ignored and trailing blanks are
C     treated as a zero.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER*4     ERRINT,LEN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ERRINT - value to be returned when the integer value cannot be
C              determined
C     LEN    - size of character array STR
C     STR    - character array of size LEN containing an integer value
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     DIG,I,SIGN,ITMP
      CHARACTER*1 CDIG
      LOGICAL     NEARMAX
C
C     + + + FUNCTIONS + + +
      INTEGER     CHRDIG
C
C     + + + EXTERNALS + + +
      EXTERNAL   CHRDIG
C
C     + + + END SPECIFICATIONS + + +
C
      ITMP= 0
      SIGN= 1
      I   = 0
      NEARMAX = .FALSE.
  10  CONTINUE
        I   = I + 1
        CDIG= STR(I)
C       replace a blank with a zero
        IF ((CDIG.EQ.'-' .OR. CDIG.EQ.'+' .OR. CDIG.EQ.' ') .AND.
     1       ITMP.EQ.0) THEN
C         replace blank, - sign, or + sign with a 0 if no number yet
          IF (CDIG.EQ.'-') THEN
C           change sign
            SIGN= -1
          END IF
          CDIG = '0'
        END IF
        DIG = CHRDIG(CDIG)
        IF (NEARMAX) THEN
          IF ((DIG .GT. 8) .OR. (DIG .EQ. 8 .AND. SIGN .EQ. 1)) THEN
C            the last digit cannot be greater than 8, and it can only
C            be 8 if the number is negative.
             I = I - 1
             GO TO 20
          END IF
        END IF
        ITMP= ITMP* 10+ DIG
        IF (I.LT.LEN .AND. DIG.GE.0) THEN
          IF (ITMP .EQ. 214748364) THEN
C            right at the border, an additional digit may be possible
             NEARMAX = .TRUE.
          ELSE IF (ITMP .GT. 214748364) THEN
C            no room for another digit, no matter what it is
             GO TO 20
          END IF
          GO TO 10
        END IF
C
 20   CONTINUE
C
      IF (DIG.EQ.-1 .OR. I.LT.LEN) THEN
C       bad conversion
        CRINTE = ERRINT
      ELSE
C       ok conversion
        CRINTE = ITMP* SIGN
      END IF
C
      RETURN
      END
C
C
C
      INTEGER   FUNCTION   CRINTX
     I                           (LEN,STR)
C
C     + + + PURPOSE + + +
C     CRINTX returns the integer equivalent of the character array STR.
C     STR is expected to contain a right-justified integer value and may
C     include a sign and digits (+, -, and 0-9).  CRINTX returns -999
C     if the character array contains an invalid character or cannot be
C     converted.  Leading blanks are ignored and trailing blanks are
C     treated as a zero.

C     Convert a character array to its integer equivalent.
C     The integer is expected to be right justified in STR.
C     For an invalid integer, -999 is returned.  Valid characters
C     are '0' - '9', '+', and '-' .  Leading blanks are ignored.
C     Trailing blanks are treated as 0.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     LEN
      CHARACTER*1 STR(LEN)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LEN    - size of character array STR
C     STR    - character array of size LEN
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     ERRINT
C
C     + + + FUNCTIONS + + +
      INTEGER     CRINTE
C
C     + + + EXTERNALS + + +
      EXTERNAL   CRINTE
C
C     + + + END SPECIFICATIONS + + +
C
      ERRINT= -999
      CRINTX= CRINTE(ERRINT,LEN,STR)
C
      RETURN
      END
C
C
C
      SUBROUTINE   NUMINI
C
C     + + + PURPOSE + + +
C     Initialize machine dependent floating point constants.
C
C     + + + COMMON BLOCKS + + +
C     numeric constants
      INCLUDE 'const.inc'
C
C     + + + SAVES + + +
      INTEGER       INITFG
      SAVE          INITFG
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       PRECFG
      INTEGER*4     R2PREC,D2PREC,TI
      REAL*4        R1,R2,R3,R4,  TR
      REAL*8        D1,D2,D3,D4
C
C     + + + FUNCTIONS + + +
      REAL*4   RNOP
      REAL*8   DNOP

C     + + + INTRINSICS + + +
      INTRINSIC  INT,LOG10
C
C     + + + EQUIVALENCE STATEMENTS + + +
      EQUIVALENCE (TR,TI)
C
C     + + + DATA INITIALIZATIONS + + +
      DATA INITFG /0/
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT(' UNKNOWN DOUBLE PRECISION FORMAT, using default double pre
     .cision values!')
 2010 FORMAT(' UNKNOWN MACHINE TYPE, using default precision values!')
C
C     + + + STATEMENT FUNCTION DEFINITIONS + + +
C     No OPperation, used to keep Ryan/McFarland optimization honest
      RNOP(R1) = R1
      DNOP(D1) = D1
C
C     + + + END SPECIFICATIONS + + +
C
      IF (INITFG.EQ.0) THEN
C       Need to calculate machine dependent numeric constants.
C       Set flag to do so only once.
        INITFG= 1
C
C       Determine the number of decimal digits of REAL precision number and
C       the smallest REAL greater than 1.0.
C       First find the number of significant binary digits, then convert
C       it to the number of significant decimal digits.  Any machine used
C       today is going to have more than 7 binary digits of precision
C       (actually, we're cheating, because 1 is added to R2PREC after
C       it is tested.  This usually results in 7 decimal digits of
C       precision, which is usually the case, whereas strictly
C       speaking only 6 decimal digits are guaranteed, and 6 is
C       usually the result if R2PREC is initialized to 6).
        R2PREC = 7
        R1 = 1.0
        R2 = 0.0078125
 100    CONTINUE
C         loop until PRECFG=1
          PRECFG= 0
          R3 = R1 + R2
C         this 'nop' keeps Ryan/McFarland optimization honest.  Without it,
C         the precision of an 80 bit floating point register is computed
C         (instead of a 4-byte real) when R/M optimization is turned on.
C         You can comment out the call to NOP if you observe that by
C         doing so DECCHR doesn't provide extra digits of precision.
          R4 = RNOP(R3)
C         The following conditional causes a warning on the PRIME,
C         but this statement needs to be left as is since we are looking
C         for the case of the two real numbers being exactly the same
C         to help determine the precision of the machine.
          IF (R1 .NE. R3) THEN
             RP1MIN = R3
             R2PREC = R2PREC + 1
             R2 = R2 / 2.0
             PRECFG= 1
          END IF
        IF (PRECFG.EQ.1) GO TO 100
C
        RPREC = INT(LOG10(2.0**R2PREC))
C
C       Determine the number of decimal digits of the typical DOUBLE precision
C       number and the smallest DOUBLE greater than 1.0D0.
        D2PREC = 7
        D1 = 1.0D0
        D2 = 0.0078125D0
 200    CONTINUE
C         loop until PRECFG=1
          PRECFG= 0
          D3 = D1 + D2
C         this 'nop' keeps Ryan/McFarland optimization honest.  Without it,
C         the precision of an 80 bit floating point register is computed
C         (instead of a 4-byte real) when R/M optimization is turned on.
C         You can comment out the call to NOP if you observe that by
C         doing so DECCHR doesn't provide extra digits of precision.
          D4 = DNOP(D3)
C         The following conditional causes a warning on the PRIME,
C         but this statement needs to be left as is since we are looking
C         for the case of the two real numbers being exactly the same
C         to help determine the precision of the machine.
          IF (D1 .NE. D3) THEN
             DP1MIN = D3
             D2PREC = D2PREC + 1
             D2 = D2 / 2.0D0
             PRECFG= 1
          END IF
        IF (PRECFG.EQ.1) GO TO 200
C
        DPREC = INT(LOG10(2.0D0**D2PREC))
C
        TR = 1.0
        IF (TI .EQ. 1065353216) THEN
C         this should be the case for the Sun or Ryan/McFarland
          R1   = 1.0E-19
          R0MIN = 1.1754945E-19 * R1
          R1   = 1.0E+19
          R0MAX = 3.40282347E+19 * R1
C         On DG (unix) system, RP1MIN is calculated a little too small
C         to have the needed correcting effect in DECCHR/DPRCHR when
C         using LOG10 intrinsics on those machines.  Thus, increase it a
C         little bit to force correction. (also see DP1MIN below)
          RP1MIN= RP1MIN**4
          D1   = 1.0D-28
          D0MIN = (2.22507385850720219D-28 * D1**10)
          D1   = 1.0D+28
          D0MAX = (1.7976931348623157D0 * D1**11)
          DP1MIN= DP1MIN**4
        ELSE IF (TI .EQ. 16512) THEN
C         this should be the case for the VAX
          R1   = 1.0E-20
          R0MIN = 2.9387359E-19 * R1
          R1   = 1.0E+18
          R0MAX = 1.7014117E+20 * R1
          IF (DPREC .EQ. 17) THEN
C            this should be the case for the default /NOG_FLOAT compiler option
             D1   = 1.0D-20
             D0MIN = 2.938735877055719D-19 * D1
             D1   = 1.0E+19
             D0MAX = 1.7014118346046923D+19 * D1
          ELSE IF (DPREC .EQ. 16) THEN
C            this should be the case for the /G_FLOAT option
             D3 = 1.0D-21
             D4 = 1.0D-20
             D0MIN = (5.562684646268008D-20)*(D4**5)*(D3**9)
             D3 = 1.0D+21
             D4 = 1.0D+20
             D0MAX = (8.988465674311578D+21) * (D3**6) * (D4**8)
          ELSE
C            assume D_FLOAT real*8 type
             WRITE(*,2000)
             D1   = 1.0D-20
             D0MIN = 2.938735877055719D-19 * D1
             D1   = 1.0E+19
             D0MAX = 1.7014118346046923D+19 * D1
          END IF
        ELSE
C         not quite sure of machine type
          WRITE(*,2010)
          R1    = 1.0E-19
          R0MIN = 1.1754945E-19 * R1
          R1    = 1.0E+18
C         On PRIME system, RP1MIN is calculated a little too small
C         to have the needed correcting effect in DECCHR/DPRCHR when
C         using LOG10 intrinsics on those machines.  Thus, increase it a
C         little bit to force correction. (also see DP1MIN below)
          RP1MIN= RP1MIN**4
          R0MAX = 1.7014117E+20 * R1
          D1    = 1.0D-20
          D0MIN = 2.938735877055719D-19 * D1
          D1    = 1.0E+19
          D0MAX = 1.7014118346046923D+19 * D1
          DP1MIN= DP1MIN**4
        ENDIF
      END IF
C
      RETURN
      END
