C
C
C
      SUBROUTINE   FTKBUF
     I                    (LINE, PARRAY,
     O                     ERRCOD)
C
C     + + + PURPOSE + + +
C     This routine fills the token buffer contained in the 'ctokes.inc'
C     common by sequentially picking tokens off of the character string
C     passed.  If there are continution lines, this routine does NOT
C     load the continuation character from column 6 into the token
C     buffer.
C
C     + + + HISTORY + + +
C     This routine was written in support of a portable SYSDOC program
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) LINE
      INTEGER       PARRAY, ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LINE   - character string to be analyzed
C     PARRAY - position in array to start loading token information
C     ERRCOD - error code returned to calling routine
C               0 - operation was successful
C              -1 - operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokea.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      APOS
      CHARACTER*80 CTEMP1, CTEMP2
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + EXTERNALS + + +
      EXTERNAL GTOKEN, ZIP
C
C     + + + END SPECIFICATIONS + + +
C
      ERRCOD = 0
      CALL ZIP(' ',CTEMP1)
      CALL ZIP(' ',CTEMP2)
      APOS   = PARRAY - 1
C
C     loop to call gtoken until gtoken fails
      ITEMS = 0
 100  CONTINUE
      ITEMS = ITEMS + 1
        IF (ITEMS .LE. LEN(LINE)) THEN
          APOS = APOS + 1
          CALL GTOKEN (LINE, ITEMS, TKBUFF(APOS), TKTYPE(APOS),
     +      TKSIZE(APOS), TKSTAR(APOS), TKEND(APOS), EOLFLG, ERRCOD)
C
          IF (ERRCOD .NE. -2) THEN
C           requested token is within range - otherwise
C           gtoken returns an errcod of -2 when requested
C           token is out of range
            IF (ERRCOD .EQ. 0) THEN
C             gtoken was successful - otherwise gtoken returns
C             errcod of -1
              IF (.NOT.EOLFLG) THEN
                IF (TKSTAR(APOS) .EQ. 6) THEN
C                 token was found in column 6, discard the character in
C                 column 6 by left shifting the token in TKBUFF(APOS)
                  IF (TKSIZE(APOS) .GT. 1) THEN
                    CTEMP1 = TKBUFF(APOS)
                    CTEMP2(1:TKSIZE(APOS)-1) = CTEMP1(2:TKSIZE(APOS))
C                   otherwise, if token is only 1 character in length
C                   reposition buffer pointer to overwrite this token
C                   on next call to GTOKEN
                  ELSE
                    APOS = APOS - 1
                  END IF
                END IF
              END IF
            END IF
          END IF
        END IF
      IF (ERRCOD .EQ. 0 .AND. (.NOT.EOLFLG)) GOTO 100
C
      IF (EOLFLG .OR. ERRCOD .EQ. -2 ) THEN
C       last call to gtoken found token requested to be out of range of
C       the valid number of tokens on the line, decrement token count
C       by 1 to indicate actual number of tokens on the line
        ITEMS = APOS - 1
        ERRCOD = 0
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   LOADBF
     O                   (ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to load single token buffer from token arrays
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER  ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ERRCOD - error code returned to calling routine
C               0 - operation was successful
C              -1 - operation failed, token does not exist
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokea.inc'
      INCLUDE 'ctokeb.inc'
C
C     + + + END SPECIFICATIONS + + +
C
      IF (TNUM .GT. 0 .AND. TNUM .LE. BUFSIZ) THEN
C       tnum is within range of token buffer
        TOKEN  = TKBUFF(TNUM)
        TTYPE  = TKTYPE(TNUM)
        TOKLEN = TKSIZE(TNUM)
        TSTART = TKSTAR(TNUM)
        TEND   = TKEND (TNUM)
        IF (TNUM .LT. ITEMS) THEN
C         preload next token
          NTOKEN = TKBUFF(TNUM+1)
        ELSE
C         this is the last token
C         set next-token to dots
          NTOKEN = '..........'
        END IF
        ERRCOD = 0
      ELSE
C       tnum is out of range of token buffer
        ERRCOD = -1
      END IF
      RETURN
      END
C
C
C
      SUBROUTINE   CLTBUF
C
C     + + + PURPOSE + + +
C     routine to clear contents of temp xref buffer
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokec.inc'
C
C     + + + EXTERNALS + + +
      EXTERNAL ZIP
C
C     + + + END SPECIFICATIONS + + +
C
C     the following variables, arrays, and char strings do not get
C     cleared here their previous values are still valid until
C     changed elsewhere
C       CALL ZIP (' ',TMODUL)
C       TMODTY   = ' '
C       TMODNU   = 0
C       TARGCN   = 0
C
C     clear the following temporary storage buffers
      CALL ZIP (' ',TSYMNM)
      TCLASS   = ' '
      TDEFIN   = .FALSE.
      TATRB1   = ' '
      TATRB2   = 0
      TATRB3   = ' '
      TDIM1    = 0
      TDIM2    = 0
      TDIM3    = 0
      TDIM4    = 0
      CALL ZIP (' ',TCOMNM)
      TLINEN   = 0
      TLNUSE   = ' '
      TAPOS    = 0
C
      RETURN
      END
C
C
C
      SUBROUTINE   XTBUF
C
C     + + + PURPOSE + + +
C     routine to transfer contents of temp xref buffer to main
C     xref buffer
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'cxref.inc'
      INCLUDE 'ctokeb.inc'
      INCLUDE 'ctokec.inc'
      INCLUDE 'cdebug.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER YORN
C
C     + + + INPUT FORMATS + + +
C1001   FORMAT(A1)
C
C     + + + OUTPUT FORMATS + + +
 2000   FORMAT('Symbol=',A32,/,
     +      'Class=',A1,' Defined=',I2,' Atrib1=',A1,' Atrib2=',I2,
     +      ' Atrib3=',A1,/,
     +      'Dim1=',I6,' Dim2=',I6,' Dim3=',I6,' Dim4=',I6,/,
     +      'Common=',A10,' LineNo=',I5,' Use=',A1,' ArgPos=',I2)
CBS  +         ' Program Module................',A12,/,
CBS  +         ' Module Type...................',A1,/,
CBS  +         ' Module Number.................',I3,/,
CBS  +         ' Number of Aguments in Module..',I3,/,
CBS  +         ' Symbol Name...................',A32,/,
CBS  +         ' Symbol Class..................',A1,/,
CBS  +         ' Defined Here (0=No, 1=Yes)..',I3,/,
CBS  +         ' Attribute 1 ..................',A1,/,
CBS  +         ' Attribute 2 ..................',I3,/,
CBS  +         ' Attribute 3 ..................',A1,/,
CBS  +         ' Dimension 1 ..................',I3,/,
CBS  +         ' Dimension 2 ..................',I3,/,
CBS  +         ' Dimension 3 ..................',I3,/,
CBS  +         ' Dimension 4 ..................',I3,/,
CBS  +         ' Included in COMMON block .....',A10,/,
CBS  +         ' Found on Source Code line No..',I5,/,
CBS  +         ' Use of Symbol on this line ...',A1,/,
CBS  +         ' Position of Dummy Argument ...',I3)
C2001   FORMAT(/,' <CR> to continue.....')
C
C     + + + END SPECIFICATIONS + + +
C
      MODULE(SYMNUM) = TMODUL
      MODTYP(SYMNUM) = TMODTY
      MODNUM(SYMNUM) = TMODNU
      ARGCNT(SYMNUM) = TARGCN
      SYMNAM(SYMNUM) = TSYMNM
      CLASS(SYMNUM)  = TCLASS
      DEFIND(SYMNUM) = TDEFIN
      ATRIB1(SYMNUM) = TATRB1
      ATRIB2(SYMNUM) = TATRB2
      ATRIB3(SYMNUM) = TATRB3
      DIM1(SYMNUM)   = TDIM1
      DIM2(SYMNUM)   = TDIM2
      DIM3(SYMNUM)   = TDIM3
      DIM4(SYMNUM)   = TDIM4
      COMNAM(SYMNUM) = TCOMNM
      LINENO(SYMNUM) = TLINEN
      LINUSE(SYMNUM) = TLNUSE
      ARGPOS(SYMNUM) = TAPOS
C
      IF (DEBUG .GT. 2) THEN
        YORN = 0
        IF (TDEFIN) YORN = 1
        WRITE(*,2000)
CBS  +      TMODUL,
CBS  +      TMODTY,
CBS  +      TMODNU,
CBS  +      TARGCN,
     +      TSYMNM,
     +      TCLASS,
     +      YORN,
     +      TATRB1,
     +      TATRB2,
     +      TATRB3,
     +      TDIM1,
     +      TDIM2,
     +      TDIM3,
     +      TDIM4,
     +      TCOMNM,
     +      TLINEN,
     +      TLNUSE,
     +      TAPOS
CBS     WRITE (*,2001)
CBS     READ(*,1001) GO
      END IF
      RETURN
      END
C
C
C
      SUBROUTINE   CPTBUF
C
C     + + + PURPOSE + + +
C     routine to copy reusable contents of previous main
C     buffer xfer to temp xref buffer
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'cxref.inc'
      INCLUDE 'ctokeb.inc'
      INCLUDE 'ctokec.inc'
C
C     + + + END SPECIFICATIONS + + +
C
      TMODUL = MODULE(SYMNUM)
      TMODTY = MODTYP(SYMNUM)
      TMODNU = MODNUM(SYMNUM)
C
C     not reusable
C       TARGCN = ARGCNT(SYMNUM)
C
      TSYMNM = SYMNAM(SYMNUM)
      TCLASS = CLASS(SYMNUM)
      TDEFIN = DEFIND(SYMNUM)
      TATRB1 = ATRIB1(SYMNUM)
      TATRB2 = ATRIB2(SYMNUM)
      TATRB3 = ATRIB3(SYMNUM)
C
C     not reusable
C       TDIM1  = DIM1(SYMNUM)
C       TDIM2  = DIM2(SYMNUM)
C       TDIM3  = DIM3(SYMNUM)
C       TDIM4  = DIM4(SYMNUM)
C
      TCOMNM = COMNAM(SYMNUM)
      TLINEN = LINENO(SYMNUM)
      TLNUSE = LINUSE(SYMNUM)
C
C     not reusable
C       TAPOS  = ARGPOS(SYMNUM)
C
      RETURN
      END
C
C
C
      SUBROUTINE   FNDLNO
     O                    (NXTFLG, ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to capture line numbers
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER ERRCOD
      LOGICAL NXTFLG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NXTFLG - flag to tell calling routine that no additional
C              tokens in the token buffer need to be processed
C              .TRUE. if processing of current buffer is complete
C              .FALSE. otherwise
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokea.inc'
      INCLUDE 'ctokeb.inc'
      INCLUDE 'ctokec.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     ILABEL, CRINTE
      CHARACTER*1 TTOKEN(32)
C
C     + + + EXTERNALS + + +
      EXTERNAL CRINTE, XTBUF, CLTBUF, ERROUT, CVARAR
C
C     + + + END SPECIFICATIONS + + +
C
      ERRCOD = 0
      NXTFLG = .FALSE.
      CALL CVARAR (32, TOKEN(1:32), 32, TTOKEN(1))
      ILABEL = CRINTE(-1,TOKLEN,TTOKEN)
C
      IF (ILABEL .GT. 0) THEN
        CALL CLTBUF
        SYMNUM = SYMNUM + 1
        TSYMNM = TOKEN(1:TOKLEN)
        TDEFIN = .TRUE.
        TLINEN = PRLINE
        TATRB1 = 'E'
        IF (NTOKEN .EQ. 'FORMAT') THEN
          TATRB1 = 'F'
          NXTFLG = .TRUE.
        ELSE IF (NTOKEN .EQ. 'CONTINUE') THEN
          TATRB1 = 'E'
          NXTFLG = .TRUE.
        END IF
        CALL XTBUF
      ELSE
        ERRCOD = -1
        CALL ERROUT (' Error in Source Code','FNDLNO')
        IF (TNUM .EQ. ITEMS) NXTFLG = .TRUE.
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   FNDENT
     I                   (SOURCE,
     O                    NXTFLG, ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to find and capture entry point information
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       ERRCOD
      CHARACTER*(*) SOURCE
      LOGICAL       NXTFLG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     SOURCE - source code file name void of any suffix
C     NXTFLG - flag to tell calling routine that the token
C              buffer has been successfully processed and that
C              the next line can be fetched and processed.
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'cxref.inc'
      INCLUDE 'ctokea.inc'
      INCLUDE 'ctokeb.inc'
      INCLUDE 'ctokec.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     LOOP, OLDNUM, APOS, ITEMNO, TSYMNO, ERRFLG
      CHARACTER*1 DEFTYP
      LOGICAL     FUNFLG, ARGS, FOUNDE, AGAIN, GETOUT
C
C     + + + EXTERNALS + + +
      EXTERNAL LOADBF, CLTBUF, XTBUF, BUMPTK, PRTENT, FNDDEF,
     +         ERROUT, DUMPBF
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT('        PROGRAM   ',A8)
C
C     + + + END SPECIFICATIONS + + +
C
      FUNFLG = .FALSE.
      NXTFLG = .FALSE.
      GETOUT = .FALSE.
      OLDNUM = TNUM
C
C     clear temporary buffer
      CALL CLTBUF
C
      FOUNDE = .FALSE.
C     loop until entry point located
      LOOP = 0
      ERRCOD = 0
 100  CONTINUE
        LOOP = LOOP + 1
        IF (LOOP .LE. ITEMS) THEN
          TNUM = LOOP
          CALL LOADBF(ERRFLG)
          IF (TOKEN(1:TOKLEN) .EQ. 'PROGRAM') THEN
C           entry point found - 'PROGRAM'
            IF (TNUM .EQ. 1) THEN
C             set routine type to 'M' = main program
              TMODTY = 'M'
              ITEMNO = TNUM
              FOUNDE = .TRUE.
            ELSE
C             entry point name, 'PROGRAM', was not first item
C             on line - ERROR
              ERRCOD = -2
            END IF
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'SUBROUTINE') THEN
C           entry point found - 'SUBROUTINE'
            IF (TNUM .EQ. 1) THEN
C             set routine type to 'S' = subroutine
              TMODTY = 'S'
              ITEMNO = TNUM
              FOUNDE = .TRUE.
            ELSE
C             entry point name, 'SUBROUTINE', was not first
C             item on line - ERROR
              ERRCOD = -2
            END IF
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'FUNCTION') THEN
C           entry point found - 'FUNCTION'
            ITEMNO = TNUM
            IF (TNUM .EQ. 1) THEN
C             set routine type to 'F' = function
              TMODTY = 'F'
              FOUNDE = .TRUE.
            ELSE
C             entry point was not first item on line -
C             look for function type
              FUNFLG = .TRUE.
              TNUM = 1
              CALL LOADBF(ERRFLG)
              IF (TOKEN(1:TOKLEN) .EQ. 'INTEGER') THEN
C               set routine type to 'I' = integer function
                TMODTY = 'I'
                FOUNDE = .TRUE.
                IF (TNUM .LT. ITEMNO-2) THEN
                  CALL BUMPTK(1,ERRFLG)
                  IF (TOKEN(1:TOKLEN) .EQ. '2') THEN
C                   set routine type to '2' = integer*2 function
                    TMODTY = '2'
                  ELSE IF (TOKEN(1:TOKLEN) .EQ. '4') THEN
C                   set routine type to '4' = integer*4 function
                    TMODTY = '4'
                  ELSE
C                   set routine type to 'J' = undetermined
C                   integer type function
                    TMODTY = 'J'
                  END IF
                END IF
              ELSE IF (TOKEN(1:TOKLEN) .EQ. 'DOUBLE') THEN
                TMODTY = 'D'
                FOUNDE = .TRUE.
              ELSE IF (TOKEN(1:TOKLEN) .EQ. 'COMPLEX') THEN
C               set routine type to 'X' = complex function
                TMODTY = 'X'
                FOUNDE = .TRUE.
              ELSE IF (TOKEN(1:TOKLEN) .EQ. 'REAL') THEN
C               set routine type to 'R' = real function
                TMODTY = 'R'
                FOUNDE = .TRUE.
                IF (TNUM .LT. ITEMNO-2) THEN
                  CALL BUMPTK(1,ERRFLG)
                  IF (TOKEN(1:TOKLEN) .EQ. '4') THEN
C                   set routine type to 'R' = real function
                    TMODTY = 'R'
                  ELSE IF (TOKEN(1:TOKLEN) .EQ. '8') THEN
C                   set routine type to 'D' = double precision
C                   function
                    TMODTY = 'D'
                  END IF
                END IF
              ELSE IF (TOKEN(1:TOKLEN) .EQ. 'LOGICAL') THEN
C             set routine type to 'L' = logical function
                TMODTY = 'L'
                FOUNDE = .TRUE.
              ELSE IF (TOKEN(1:TOKLEN) .EQ. 'CHARACTER') THEN
C             set routine type to 'C' = character function
                TMODTY = 'C'
                FOUNDE = .TRUE.
              END IF
            END IF
          END IF
          IF (FOUNDE .AND. ERRCOD .EQ. 0) THEN
C           entry point was found
            TNUM   = ITEMNO + 1
            CALL LOADBF(ERRFLG)
            SYMNUM = SYMNUM + 1
            TSYMNO = SYMNUM
            TMODUL = TOKEN(1:TOKLEN)
            TMODNU = RTNNUM
            TSYMNM = TOKEN(1:TOKLEN)
            TDEFIN = .TRUE.
            TLINEN = PRLINE
            TATRB3 = 'E'
            IF (FUNFLG) THEN
C             entry point was a function - capture type (e.g.integer)
              CALL XTBUF
              CALL CLTBUF
              SYMNUM = SYMNUM + 1
              TSYMNM = TOKEN(1:TOKLEN)
              TLINEN = PRLINE
              TDEFIN = .TRUE.
              TATRB1 = ATRIB1(SYMNUM-1)
              TATRB2 = ATRIB2(SYMNUM-1)
            END IF
            CALL XTBUF
            CALL PRTENT(ERRFLG)
C           get argument list
            APOS   = 0
            ARGS   = .FALSE.
C           set loop counter to position of the word
C           following 'PROGRAM',
C           'SUBROUTINE',or 'FUNCTION'
            LOOP   = ITEMNO + 1
C           loop until left parin is found
 200        CONTINUE
              AGAIN = .FALSE.
              LOOP = LOOP + 1
              IF (LOOP .GT. ITEMS) THEN
                NXTFLG = .TRUE.
              ELSE IF (TKSTAR(LOOP) .GT. 6) THEN
C               token begins after column 6
C               look for 'D' type token (DELIMITER) one character
C               in length and examine it to see if it is a left parin
                IF (TKTYPE(LOOP) .EQ. 'D' .AND. TKSIZE(LOOP) .EQ. 1
     +              .AND. TKBUFF(LOOP) .EQ. '(') THEN
C                 arguments are present
                  ARGS = .TRUE.
C                 loop to read arguments
 300              CONTINUE
                    LOOP = LOOP + 1
                    IF (TKBUFF(LOOP) .EQ. ')' .OR. LOOP .EQ. ITEMS) THEN
C                     this is the last token
                      NXTFLG = .TRUE.
                    ELSEIF (TKBUFF(LOOP) .NE. ',' .AND.
     +                   TKSTAR(LOOP) .NE. 6) THEN
C                     token is not a comma or a continution character
C                     found a dummy argument, increment argument
C                     position counter
                      APOS = APOS + 1
C                     determine default variable type
                      TNUM = LOOP
                      CALL LOADBF(ERRFLG)
                      CALL CLTBUF
                      SYMNUM = SYMNUM + 1
                      TSYMNM = TOKEN(1:TOKLEN)
                      CALL FNDDEF (TSYMNM, DEFTYP, ERRCOD)
                      IF (ERRCOD .NE. 0) THEN
                        CALL ERROUT (' Error in call to FNDDEF',
     +                               'FNDENT')
                        CALL DUMPBF
                        GETOUT = .TRUE.
                      ELSE
                        TATRB1  = DEFTYP
                      END IF
C                     log argument information
                      TCLASS = 'A'
                      TAPOS  = APOS
                      TLINEN = PRLINE
                      CALL XTBUF
                    END IF
                  IF (.NOT.GETOUT .AND. .NOT.NXTFLG) GOTO 300
                END IF
              ELSE
                AGAIN = .TRUE.
              END IF
            IF (.NOT.GETOUT .AND. AGAIN) GOTO 200
C
C           if there are dummy arguments and the last token is not a
C           right parin, generate an error code
            IF (.NOT.GETOUT .AND. APOS .GT. 0
     +          .AND. TKBUFF(LOOP) .NE. ')') ERRCOD = -1
C
C           if a left parin was found but no arguements were found
C           generate an error code
            IF (.NOT.GETOUT .AND. APOS .EQ. 0
     +          .AND. ARGS) ERRCOD = -1
C
C           log dummy argument count
            IF (.NOT.GETOUT) ARGCNT(TSYMNO) = APOS
C
          END IF
        ELSE
C         entry point was not found, consider this PROGRAM MAIN
          FOUNDE = .TRUE.
          TMODTY = 'M'
          SYMNUM = SYMNUM + 1
          TSYMNO = SYMNUM
          TMODUL = SOURCE
          TMODNU = RTNNUM
          TSYMNM = SOURCE
          TDEFIN = .TRUE.
          TLINEN = PRLINE - 1
          TATRB3 = 'E'
          CALL XTBUF
          WRITE(*,2000) SOURCE
CBS       entry point not found, set errcod and return to calling routine
CBS       IF (ERRCOD .EQ. 0) ERRCOD = -1
        END IF
      IF (.NOT.GETOUT .AND. .NOT.FOUNDE .AND. ERRCOD .EQ. 0) GOTO 100
C
      TNUM = OLDNUM
      CALL LOADBF(ERRFLG)
      RETURN
      END
C
C
C
      SUBROUTINE   GINCNM (
     I                     PATH, PTHLEN,
     O                     INAME, INAME2, ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to get the filename of an INCLUDE file
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) INAME, INAME2, PATH
      INTEGER       ERRCOD, PTHLEN
C
C     + + + ARGUMENT DEFINITIONS + + +
C     PATH   - path to directory where source code is located
C     PTHLEN - length of path name
C     INAME  - file name of INCLUDE or $INSERT file
C     INAME2 - concatenation of PATH and INAME
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokea.inc'
      INCLUDE 'ctokeb.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER INCSTR, INCEND, IPOS, ICNT, LOOP, IPOINT, ERRFLG, IP2
      LOGICAL AGAIN
C
C     + + + EXTERNALS + + +
      EXTERNAL BUMPTK, ZIP
C
C     + + + END SPECIFICATIONS + + +
C
      ERRCOD = 0
      INCSTR = 0
      INCEND = 0
      CALL ZIP (' ',INAME)
      CALL ZIP (' ',INAME2)
      INAME2 = PATH
C
C     loop until filename is found
C
      AGAIN = .TRUE.
 100  CONTINUE
        CALL BUMPTK(1,ERRFLG)
        IF (ERRFLG .EQ. 0) THEN
C         process token
          IF (TOKEN .EQ. 1H') THEN
C           register the location of the single quotation marks
            IF (INCSTR .EQ. 0) THEN
C             location of starting element of filename
              INCSTR = TNUM + 1
            ELSE
C             location of ending element of filename
              INCEND = TNUM - 1
              AGAIN = .FALSE.
            END IF
          ELSE IF (TOKEN .EQ. 'INSERT') THEN
C           register the location of the token
            IF (INCSTR .EQ. 0) THEN
C             location of starting element of filename
              INCSTR = TNUM + 1
              INCEND = ITEMS
              AGAIN = .FALSE.
            END IF
          END IF
        ELSE
C         token is out of range
          ERRCOD = -1
        END IF
      IF (AGAIN .AND. ERRCOD .EQ. 0) GOTO 100
C
      IF (ERRCOD.EQ.0) THEN
C       get filename
C       set pointer to first element of filename
        IPOS = INCSTR
C       set counter to number of elements which make up the filename
        ICNT = INCEND - INCSTR + 1
C       set pointer to INAME character position to start filling
        IPOINT = 1
C       set pointer to INAME2 character position to start filling
        IP2 = PTHLEN + 1
C
C       loop to concatonate the filename out of the token pieces
        DO 200 LOOP = 1, ICNT, 1
C         build the filename
          INAME(IPOINT:(IPOINT+TKSIZE(IPOS))) = TKBUFF(IPOS)
C         increment character pointer
          IPOINT = IPOINT + TKSIZE(IPOS)
C         build the pathname
          INAME2(IP2:(IP2+TKSIZE(IPOS))) = TKBUFF(IPOS)
C         increment character pointer
          IP2 = IP2 + TKSIZE(IPOS)
C         increment token buffer pointer
          IPOS = IPOS + 1
 200    CONTINUE
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   FNDSPC (
     O                     DONFLG, NXTFLG, ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to find and capture source code specification
C     statement information
C
C     + + + DUMMY ARGUMENTS + + +
      LOGICAL       NXTFLG, DONFLG
      INTEGER       ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     DONFLG - flag to tell calling routine that this routine
C              has located the end of the specification statements
C              in the source code file
C     NXTFLG - flag to tell calling routine that the token
C              buffer has been successfully processed and that
C              the next line can be fetched and processed.
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokea.inc'
      INCLUDE 'ctokeb.inc'
      INCLUDE 'ctokec.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER LOOP, ERRFLG
      LOGICAL FOUNDR
C
C     + + + EXTERNALS + + +
      EXTERNAL LOADBF, CLTBUF, REVIMP, ERROUT, FNDEXT, FNDINT, FNDEQU,
     +         FNDCOM, GVARI, GVTYPE, FNDPAR, DUMPBF, BUMPTK
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT('  ERROR - ENTRY is not supported',' Module =',A12,
     +       '  on Line ',I5)
C
C     + + + END SPECIFICATIONS + + +
C
      ERRCOD = 0
      DONFLG = .FALSE.
      NXTFLG = .FALSE.
C
C     clear temporary buffer
      CALL CLTBUF
      TLINEN = PRLINE
C
C     loop until reserved word is located
      FOUNDR = .FALSE.
      LOOP = 0
 100  CONTINUE
        LOOP = LOOP + 1
        IF (LOOP .LE. ITEMS) THEN
          TNUM = LOOP
          CALL LOADBF(ERRFLG)
          IF (TOKEN(1:TOKLEN) .EQ. '=') THEN
C           we have entered executable code, bail out
            FOUNDR = .TRUE.
            DONFLG = .TRUE.
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'CHARACTER') THEN
            TATRB1 = 'C'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'DOUBLEPRECISION') THEN
            TATRB1 = 'D'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'COMPLEX') THEN
            TATRB1 = 'X'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'INTEGER') THEN
            TATRB1 = 'I'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'LOGICAL') THEN
            TATRB1 = 'L'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'REAL') THEN
            TATRB1 = 'R'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'DOUBLE') THEN
            CALL BUMPTK(1,ERRFLG)
            IF (TOKEN(1:TOKLEN) .EQ. 'PRECISION') THEN
              TATRB1 = 'D'
            ELSE
              CALL BUMPTK(-1,ERRFLG)
            END IF
          END IF
C
          IF (TATRB1 .NE. ' ') THEN
C           reserved word found - 'REAL','DOUBLE PRECISION',
C             'COMPLEX','INTEGER','LOGICAL','CHARACTER'
            FOUNDR = .TRUE.
C           set flag to indicate variables are defined on source
C           code line being processed
            TDEFIN = .TRUE.
C           get variable type
            CALL GVTYPE (ERRCOD)
            IF (ERRCOD .NE. 0) THEN
              ERRCOD = -1
              CALL ERROUT (' Error in call to GVTYPE','FNDSPC')
              CALL DUMPBF
            ELSE
C             get variable information
              CALL GVARI (NXTFLG, ERRCOD)
              IF (ERRCOD .NE. 0) THEN
                ERRCOD = -1
                CALL ERROUT (' Error in call to GVARI','FNDSPC')
                CALL DUMPBF
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'DIMENSION') THEN
C             reserved word found - 'DIMENSION'
              FOUNDR = .TRUE.
C             get variable type
              CALL GVTYPE (ERRCOD)
              IF (ERRCOD .NE. 0) THEN
                ERRCOD = -1
                CALL ERROUT (' Error in call to GVTYPE','FNDSPC')
                CALL DUMPBF
              ELSE
C               get variable information
                CALL GVARI (NXTFLG, ERRCOD)
                IF (ERRCOD .NE. 0) THEN
                  ERRCOD = -1
                  CALL ERROUT (' Error in call to GVARI','FNDSPC')
                  CALL DUMPBF
                END IF
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'PARAMETER') THEN
C             reserved word found - 'PARAMETER'
              TATRB3 = 'P'
              FOUNDR = .TRUE.
              TDEFIN = .FALSE.
              CALL FNDPAR (NXTFLG, ERRCOD)
              IF (ERRCOD .NE. 0 .OR. (.NOT.NXTFLG)) THEN
                CALL ERROUT (' Error in call to FNDPAR','FNDSPC')
                CALL DUMPBF
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'COMMON') THEN
C             reserved word found - 'COMMON'
              FOUNDR = .TRUE.
              TDEFIN = .FALSE.
              CALL FNDCOM (NXTFLG, ERRCOD)
              IF (ERRCOD .NE. 0 .OR. (.NOT.NXTFLG)) THEN
                CALL ERROUT (' Error in call to FNDCOM','FNDSPC')
                CALL DUMPBF
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'EXTERNAL') THEN
C           reserved word found - 'EXTERNAL'
              FOUNDR = .TRUE.
              CALL FNDEXT (NXTFLG, ERRCOD)
              IF (ERRCOD .NE. 0 .OR. .NOT.NXTFLG) THEN
                CALL ERROUT (' Error in call to FNDEXT','FNDSPC')
                CALL DUMPBF
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'INTRINSIC') THEN
C             reserved word found - 'INTRINSIC'
              FOUNDR = .TRUE.
              CALL FNDINT (DONFLG, NXTFLG, ERRCOD)
              IF (ERRCOD .NE. 0 .OR. (.NOT.NXTFLG)) THEN
                CALL ERROUT (' Error in call to FNDINT','FNDSPC')
                CALL DUMPBF
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'EQUIVALENCE') THEN
C             reserved word found - 'EQUIVALENCE'
              FOUNDR = .TRUE.
              CALL FNDEQU (DONFLG, NXTFLG, ERRCOD)
              IF (ERRCOD .NE. 0 .OR. (.NOT.NXTFLG)) THEN
                CALL ERROUT (' Error in call to FNDEQU','FNDSPC')
                CALL DUMPBF
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'SAVE') THEN
              FOUNDR = .TRUE.
            ELSE IF (TOKEN(1:TOKLEN) .EQ. 'ENTRY') THEN
              WRITE (*,2000) TMODUL, PRLINE
              FOUNDR = .TRUE.
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'IMPLICIT') THEN
              CALL REVIMP (NXTFLG, ERRCOD)
              IF (ERRCOD .NE. 0 .OR. .NOT.NXTFLG) THEN
                CALL ERROUT (' Error in call to REVIMP','FNDSPC')
                CALL DUMPBF
              ELSE
                FOUNDR = .TRUE.
              END IF
            END IF
          END IF
C
          IF (.NOT.FOUNDR) THEN
            IF (TOKEN(1:TOKLEN) .EQ. 'FORMAT' .AND.
     +           LOOP .LE. 2 .AND. (.NOT.YESEXE)) THEN
C             format statement preceeds executable code
C             MAKXRF prints these lines to the .XRF file
C             for further processing by SYSDOC
              FOUNDR = .TRUE.
            END IF
          END IF
        ELSE
C         reserved word not found, consider this the end of the
C         specification section of the source code
          DONFLG = .TRUE.
          FOUNDR = .TRUE.
        ENDIF
      IF (.NOT.FOUNDR) GOTO 100
      NXTFLG = .TRUE.
      RETURN
      END
C
C
C
      SUBROUTINE   DUMPX
     I                  (SRTNAM, OUNIT,
     O                   ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to dump xref listing information contained in 'cxref.inc'
C     common to file opened on unit = OUNIT)
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       OUNIT, ERRCOD
      CHARACTER*(*) SRTNAM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     SRTNAM - temporary filename used by sort routine
C     OUNIT  - file unit on which the .XRF file is opened for write
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'cxref.inc'
      INCLUDE 'ctokeb.inc'
      INCLUDE 'ctokec.inc'
      INCLUDE 'cdebug.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       LOOP1, TUNIT1, TUNIT2, YORN1, YORN2,
     +              KEYS, KEYBGN(3), KEYEND(3), BUFNUM,
     +              SLEN, CPOS, ERCOD2, IOSTA0, IOSTA1, IOSTA2
      CHARACTER*100 OLINE
      CHARACTER*10  INTBUF
      CHARACTER*1   TINT(20)
      LOGICAL       FIRST
C
C     + + + EXTERNALS + + +
      EXTERNAL SRTSYM, ERROUT, GUNIT, DMPBUF, ZIP, INTCHR,
     +         DOCDEL, CARVAR
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT(A12,A1,I4,I2,A32,A1,I1,A1,I4,A1,I6,I6,I6,I6,A10,I5,A1,I2)
 2001 FORMAT ('C',/,'BEGIN CROSS REFERENCE')
 2002 FORMAT ('END CROSS REFERENCE')
 2003 FORMAT ( A )
 3000 FORMAT(' MAIN BUFFER - ',/,
     +    ' Symbol=',A32,T40,' Class=',A1,T60,' Defined=',I2,/,
     +    ' Atr1=',A1,T20,' Atr2=',I4,T40,' Atr3=',A1,/,
     +    ' Dim1=',I6,T20,' Dim2=',I6,T40,' Dim3=',I6,T60,' Dim4=',I6,/,
     +    ' Common=',A10,T20,' LineNo=',I5,T40,' Use=',A1,T60,
     +    ' ArgPos=',I2,T20,' Numuse=',I5,T40,' Numm=',I5)
 3001 FORMAT(' TEMP BUFFER -',/,
     +    ' Symbol=',A32,T40,' Class=',A1,T60,' Defined=',I2,/,
     +    ' Atr1=',A1,T20,' Atr2=',I2,T40,' Atr3=',A1,/,
     +    ' Dim1=',I6,T20,' Dim2=',I6,T40,' Dim3=',I6,T60,' Dim4=',I6,/,
     +    ' Common=',A10,T20,' LineNo=',I5,T40,' Use=',A1,T60,
     +    ' ArgPos=',I2)
 3002 FORMAT(' AFTER CONSOLIDATION',/,
     +    ' Symbol=',A32,T40,' Class=',A1,T60,' Defined=',I2,/,
     +    ' Atr1=',A1,T20,' Atr2=',I2,T40,' Atr3=',A1,/,
     +    ' Dim1=',I6,T20,' Dim2=',I6,T40,' Dim3=',I6,T60,' Dim4=',I6,/,
     +    ' Common=',A10,T20,' LineNo=',I5,T40,' Use=',A1,T60,
     +    ' ArgPos=',I2,T20,' Numuse=',I5,T40,' Numm=',I5)
C
C
C     + + + END SPECIFICATIONS + + +
C
C     get available temporary file unit number
      CALL GUNIT (TUNIT1, ERRCOD)
C
C     open scratch file
      OPEN (UNIT=TUNIT1, STATUS='SCRATCH',
     +    ACCESS='SEQUENTIAL', IOSTAT=ERRCOD)
      IF (ERRCOD .NE. 0)
     +    CALL ERROUT (' Error in call to OPEN','DUMPX')
C
C     open xxxxxx.SRT file for sort output
      CALL GUNIT (TUNIT2, ERRCOD)
      OPEN (UNIT=TUNIT2, FILE=SRTNAM, ACCESS='SEQUENTIAL',
     +    IOSTAT=ERRCOD)
      IF (ERRCOD .NE. 0)
     +    CALL ERROUT (' Error in call to OPEN','DUMPX')
C
C     copy main buffer location 1 information to temporary buffer
      BUFNUM = 1
      CALL DMPBUF (BUFNUM)
C
C     initialize OLINE
      CALL ZIP (' ', OLINE)
C
C     determine type of routine we are working with
      IF (TMODTY .EQ. 'M') THEN
        OLINE(14:25) = 'MAIN PROGRAM'
      ELSE IF (TMODTY .EQ. 'S') THEN
        OLINE(14:23) = 'SUBROUTINE'
      ELSE IF (TMODTY .EQ. 'F') THEN
        OLINE(14:21) = 'FUNCTION'
      ELSE IF (TMODTY .EQ. 'D') THEN
        OLINE(14:38) = 'DOUBLE PRECISION FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. 'R') THEN
        OLINE(14:26) = 'REAL FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. 'I') THEN
        OLINE(14:29) = 'INTEGER FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. '2') THEN
        OLINE(14:31) = 'INTEGER*2 FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. '4') THEN
        OLINE(14:31) = 'INTEGER*4 FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. 'J') THEN
        OLINE(14:31) = 'INTEGER*? FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. 'C') THEN
        OLINE(14:31) = 'CHARACTER FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. 'L') THEN
        OLINE(14:29) = 'LOGICAL FUNCTION'
CBS     OLINE(23:23) = TMODTY
      ELSE IF (TMODTY .EQ. 'X') THEN
        OLINE(14:29) = 'COMPLEX FUNCTION'
CBS     OLINE(23:23) = TMODTY
      END IF
C
      OLINE(1:12) = TMODUL(1:12)
C
C     write module name and type to the xref file
      WRITE (OUNIT, 2001)
      WRITE (OUNIT, 2003) OLINE
C
C     write module information to first record position of
C     sort output file so that it does not get sorted and
C     therefore remains in the first buffer position when
C     sorted information is read back into memory.
      YORN1 = 0
      IF (DEFIND(1)) YORN1 = 1
      WRITE (TUNIT2, 2000) MODULE(1), MODTYP(1),
     +        MODNUM(1), ARGCNT(1), SYMNAM(1), CLASS(1),
     +        YORN1, ATRIB1(1), ATRIB2(1), ATRIB3(1),
     +        DIM1(1), DIM2(1), DIM3(1), DIM4(1),
     +        COMNAM(1), LINENO(1), LINUSE(1), ARGPOS(1)
C
C     write remainder of main buffer to scratch file
      DO 100 LOOP1 = 2, SYMNUM
        YORN1 = 0
        IF (DEFIND(LOOP1)) YORN1 = 1
        WRITE (TUNIT1, 2000) MODULE(LOOP1), MODTYP(LOOP1),
     +      MODNUM(LOOP1), ARGCNT(LOOP1), SYMNAM(LOOP1), CLASS(LOOP1),
     +      YORN1, ATRIB1(LOOP1), ATRIB2(LOOP1), ATRIB3(LOOP1),
     +      DIM1(LOOP1), DIM2(LOOP1), DIM3(LOOP1), DIM4(LOOP1),
     +      COMNAM(LOOP1), LINENO(LOOP1), LINUSE(LOOP1), ARGPOS(LOOP1)
 100  CONTINUE
C
C     rewind the scratch file prior to calling sort routine
      REWIND TUNIT1
C
C     set keys to sort on symbol name and line number
      KEYS = 2
      KEYBGN(1) = 20
      KEYEND(1) = 51
      KEYBGN(2) = 94
      KEYEND(2) = 98
C
C     Sort the file
      CALL SRTSYM (TUNIT1, TUNIT2, KEYS, KEYBGN, KEYEND, ERRCOD)
        IF (ERRCOD .NE. 0)
     +    CALL ERROUT (' Error in call to SUBSRT','DUMPX')
C
C     rewind the sort output file in preparation for
C     information consolidation
      REWIND TUNIT2
C
C     read module information (first record) from sort output file
      READ (TUNIT2, 2000, IOSTAT=IOSTA0)
     +    MODULE(1), MODTYP(1),
     +    MODNUM(1), ARGCNT(1), SYMNAM(1), CLASS(1),
     +    YORN1, ATRIB1(1), ATRIB2(1), ATRIB3(1),
     +    DIM1(1), DIM2(1), DIM3(1), DIM4(1),
     +    COMNAM(1), LINENO(1), LINUSE(1), ARGPOS(1)
C
      IF (IOSTA0 .EQ. 0) THEN
C       initialize counter to indicate first record has been read
        LOOP1 = 1
        DEFIND (1) = .FALSE.
        IF (YORN1 .EQ. 1) DEFIND (1) = .TRUE.
C
C       loop to read and consolidate records
 250    CONTINUE
C         increment counter to read next record
          LOOP1 = LOOP1 + 1
C         initialize use and modification counters
          NUMUSE(LOOP1) = 0
          NUMM(LOOP1)   = 0
          FIRST = .FALSE.
C
C         read record from sort output file
          READ (TUNIT2, 2000, IOSTAT=IOSTA1)
     +        MODULE(LOOP1), MODTYP(LOOP1), MODNUM(LOOP1),
     +        ARGCNT(LOOP1), SYMNAM(LOOP1), CLASS(LOOP1), YORN1,
     +        ATRIB1(LOOP1), ATRIB2(LOOP1), ATRIB3(LOOP1),
     +        DIM1(LOOP1), DIM2(LOOP1), DIM3(LOOP1), DIM4(LOOP1),
     +        COMNAM(LOOP1), LINENO(LOOP1), LINUSE(LOOP1),
     +        ARGPOS(LOOP1)
C
          IF (IOSTA1 .EQ. 0) THEN
            FIRST = .TRUE.
            IF (LINUSE(LOOP1) .EQ. 'A' .OR.
     +           LINUSE(LOOP1) .EQ. ' ') NUMUSE(LOOP1) = 1
            IF (YORN1 .EQ. 1) NUMUSE(LOOP1) = NUMUSE(LOOP1) -1
            IF (LINUSE(LOOP1) .EQ. 'M') NUMM(LOOP1)   = 1
C
            IF (DEBUG .GT. 1) WRITE(*,3000)
     +            SYMNAM(LOOP1),
     +            CLASS(LOOP1),
     +            YORN1,
     +            ATRIB1(LOOP1),
     +            ATRIB2(LOOP1),
     +            ATRIB3(LOOP1),
     +            DIM1(LOOP1),
     +            DIM2(LOOP1),
     +            DIM3(LOOP1),
     +            DIM4(LOOP1),
     +            COMNAM(LOOP1),
     +            LINENO(LOOP1),
     +            LINUSE(LOOP1),
     +            ARGPOS(LOOP1),
     +            NUMUSE(LOOP1),
     +            NUMM(LOOP1)
C
 400        CONTINUE
C             read next record into temp buffer
              READ (TUNIT2, 2000, IOSTAT=IOSTA2)
     +               TMODUL, TMODTY, TMODNU, TARGCN,
     +               TSYMNM, TCLASS, YORN2, TATRB1, TATRB2, TATRB3,
     +               TDIM1, TDIM2, TDIM3, TDIM4, TCOMNM, TLINEN, TLNUSE,
     +               TAPOS
C
              IF (IOSTA2 .EQ. 0) THEN
                IF (DEBUG .GT. 1) WRITE(*,3001)
     +              TSYMNM,
     +              TCLASS,
     +              YORN2,
     +              TATRB1,
     +              TATRB2,
     +              TATRB3,
     +              TDIM1,
     +              TDIM2,
     +              TDIM3,
     +              TDIM4,
     +              TCOMNM,
     +              TLINEN,
     +              TLNUSE,
     +              TAPOS
C
C
C               if both records pertain to same symbol, consolidate
C               information into first record
                IF (TSYMNM .EQ. SYMNAM(LOOP1)) THEN
                  IF (CLASS (LOOP1) .EQ. ' ')  CLASS (LOOP1)  = TCLASS
                  IF (ATRIB1 (LOOP1) .EQ. ' ') ATRIB1 (LOOP1) = TATRB1
                  IF (YORN2 .EQ. 1 .AND.
     +                 TATRB1 .NE. ' ')        ATRIB1 (LOOP1) = TATRB1
                  IF (ATRIB2 (LOOP1) .EQ. 0)   ATRIB2 (LOOP1) = TATRB2
                  IF (ATRIB3 (LOOP1) .EQ. ' ') ATRIB3 (LOOP1) = TATRB3
                  IF (DIM1 (LOOP1) .EQ. 0)     DIM1 (LOOP1)   = TDIM1
                  IF (DIM2 (LOOP1) .EQ. 0)     DIM2 (LOOP1)   = TDIM2
                  IF (DIM3 (LOOP1) .EQ. 0)     DIM3 (LOOP1)   = TDIM3
                  IF (DIM4 (LOOP1) .EQ. 0)     DIM4 (LOOP1)   = TDIM4
                  IF (COMNAM (LOOP1) .EQ. '            ')
     +                                         COMNAM (LOOP1) = TCOMNM
                  IF (ARGPOS(LOOP1) .EQ. 0)    ARGPOS (LOOP1) = TAPOS
                  IF (YORN2 .EQ. 1)  NUMUSE(LOOP1) = NUMUSE(LOOP1) - 1
                  IF (LINENO(LOOP1) .NE. TLINEN) THEN
                    IF (LINUSE(LOOP1) .EQ. ' ')
     +                   LINUSE (LOOP1) = TLNUSE
                    IF (TLNUSE .EQ. 'A' .OR.
     +                   TLNUSE .EQ. ' ' .OR.
     +                   TLNUSE .EQ. 'I')
     +                   NUMUSE(LOOP1) = NUMUSE(LOOP1) + 1
                    IF (TLNUSE .EQ. 'M')
     +                   NUMM(LOOP1) = NUMM(LOOP1) + 1
                    IF (TLNUSE .EQ. 'M')
     +                   LINUSE (LOOP1) = 'M'
                  ELSE
                    IF (.NOT.DEFIND(LOOP1) .AND. YORN2 .EQ. 1)
     +                  DEFIND(LOOP1) = .TRUE.
                  END IF
C
                  IF (DEBUG .GT. 1) WRITE(*,3002)
     +              SYMNAM(LOOP1),
     +              CLASS(LOOP1),
     +              YORN1,
     +              ATRIB1(LOOP1),
     +              ATRIB2(LOOP1),
     +              ATRIB3(LOOP1),
     +              DIM1(LOOP1),
     +              DIM2(LOOP1),
     +              DIM3(LOOP1),
     +              DIM4(LOOP1),
     +              COMNAM(LOOP1),
     +              LINENO(LOOP1),
     +              LINUSE(LOOP1),
     +              ARGPOS(LOOP1),
     +              NUMUSE(LOOP1),
     +              NUMM(LOOP1)
C
                ELSE
C                 records pertain to different symbols
C                 backspace input file one record
                  BACKSPACE (TUNIT2)
                  FIRST = .FALSE.
                END IF
              ELSE
                SYMNUM = LOOP1
              END IF
            IF (FIRST .AND. IOSTA2 .EQ. 0) GOTO 400
          ELSE IF (IOSTA1 .GT. 0) THEN
C           read error
            SYMNUM = LOOP1 - 1
          END IF
        IF (.NOT. FIRST) GOTO 250
      END IF
C
      IF (IOSTA0 .EQ. 0 .AND. IOSTA1 .GE. 0) THEN
        DO 800 LOOP1 = 2, SYMNUM
          CALL ZIP (' ', OLINE)
C
C         symbol name
          OLINE(1:32) = SYMNAM(LOOP1)
C
C         symbol class - constant, common block, argument, intrinsic
          OLINE(34:34)   = CLASS(LOOP1)
C
C         attribute 3 - function, subroutine, parameter
          OLINE(36:36) = ATRIB3(LOOP1)
C
C         common name
          OLINE(38:47) = COMNAM(LOOP1)
C
C         use of symbol - argument, modified, common block name
          OLINE(49:49) = LINUSE(LOOP1)
C
C         argument position
          CALL INTCHR (ARGPOS(LOOP1), 3, 0, SLEN, TINT(1))
          CALL CARVAR (SLEN, TINT(1), SLEN, OLINE(51:53))
C
C         number of uses of symbol
          CALL INTCHR (NUMUSE(LOOP1), 3, 0, SLEN, TINT(1))
          CALL CARVAR (SLEN, TINT(1), SLEN, OLINE(55:57))
C
C         number of modifications to symbol
          CALL INTCHR (NUMM(LOOP1), 3, 0, SLEN, TINT(1))
          CALL CARVAR (SLEN, TINT(1), SLEN, OLINE(59:61))
C
C
C         variable type - integer, character, real, etc
          IF (ATRIB1(LOOP1) .EQ. 'C') THEN
            OLINE(63:63) = 'C'
            IF (ATRIB2(LOOP1) .NE. 0) THEN
              OLINE(64:64) = '*'
              IF (ATRIB2(LOOP1) .EQ. -1) THEN
                OLINE(65:65) = 'V'
                CPOS = 66
              ELSE
                CALL INTCHR (ATRIB2(LOOP1), 6, 1, SLEN, TINT(1))
                CALL CARVAR (SLEN, TINT(1), SLEN, OLINE(65:65+SLEN-1))
                CPOS = 66 + SLEN
              END IF
            ELSE
              CPOS = 64
            END IF
          ELSE IF (ATRIB1(LOOP1) .EQ. 'I') THEN
            OLINE(63:63) = 'I'
            IF (ATRIB2(LOOP1) .NE. 0) THEN
              OLINE(64:64) = '*'
              CALL INTCHR (ATRIB2(LOOP1), 6, 1, SLEN, TINT(1))
              CALL CARVAR (SLEN, TINT, SLEN, OLINE(65:65+SLEN-1))
              CPOS = 65 + SLEN
            ELSE
              CPOS = 64
            END IF
          ELSE IF (ATRIB1(LOOP1) .EQ. 'R') THEN
            OLINE(63:63) = 'R'
            IF (ATRIB2(LOOP1) .NE. 0) THEN
              OLINE(64:64) = '*'
              CALL INTCHR (ATRIB2(LOOP1), 6, 1, SLEN, TINT(1))
              CALL CARVAR (SLEN, TINT, SLEN, OLINE(65:65+SLEN-1))
              CPOS = 65 + SLEN
            ELSE
              CPOS = 64
            END IF
          ELSE IF (ATRIB1(LOOP1) .EQ. 'X') THEN
            OLINE(63:63) = 'X'
            CPOS = 64
          ELSE IF (ATRIB1(LOOP1) .EQ. 'L') THEN
            OLINE(63:63) = 'L'
            CPOS = 64
          ELSE IF (ATRIB1(LOOP1) .EQ. 'D') THEN
            OLINE(63:63) = 'D'
            CPOS = 64
          END IF
          IF (CPOS .LT. 67) CPOS = 67
C
          IF (DIM1(LOOP1) .NE. 0) THEN
C           first dimension
            OLINE(CPOS:CPOS) = '('
            CPOS = CPOS + 1
            IF (DIM1(LOOP1) .EQ. -1) THEN
              OLINE(CPOS:CPOS) = 'V'
              CPOS = CPOS + 1
            ELSE
              CALL INTCHR (DIM1(LOOP1), 10, 1, SLEN, TINT(1))
              CALL CARVAR (SLEN, TINT(1), SLEN, INTBUF(1:10))
              OLINE(CPOS:CPOS+SLEN-1) = INTBUF
              CPOS = CPOS + SLEN
            END IF
C
            IF (DIM2(LOOP1) .NE. 0) THEN
C             second dimension
              OLINE(CPOS:CPOS) = ','
              CPOS = CPOS + 1
              IF (DIM2(LOOP1) .EQ. -1) THEN
                OLINE(CPOS:CPOS) = 'V'
                CPOS = CPOS + 1
              ELSE
                CALL INTCHR (DIM2(LOOP1), 10, 1, SLEN, TINT(1))
                CALL CARVAR (SLEN, TINT(1), SLEN, INTBUF(1:10))
                OLINE(CPOS:(CPOS+SLEN-1)) = INTBUF
                CPOS = CPOS + SLEN
              END IF
C
              IF (DIM3(LOOP1) .NE. 0) THEN
C               third dimension
                OLINE(CPOS:CPOS) = ','
                CPOS = CPOS + 1
                IF (DIM3(LOOP1) .EQ. -1) THEN
                  OLINE(CPOS:CPOS) = 'V'
                  CPOS = CPOS + 1
                ELSE
                  CALL INTCHR (DIM3(LOOP1), 10, 1, SLEN, TINT(1))
                  CALL CARVAR (SLEN, TINT(1), SLEN, INTBUF(1:10))
                  OLINE(CPOS:(CPOS+SLEN-1)) = INTBUF
                  CPOS = CPOS + SLEN
                END IF
C
                IF (DIM4(LOOP1) .NE. 0) THEN
C                 fourth dimension
                  OLINE(CPOS:CPOS) = ','
                  CPOS = CPOS + 1
                  IF (DIM4(LOOP1) .EQ. -1) THEN
                    OLINE(CPOS:CPOS) = 'V'
                    CPOS = CPOS + 1
                  ELSE
                    CALL INTCHR (DIM4(LOOP1), 10, 1, SLEN, TINT(1))
                    CALL CARVAR (SLEN, TINT(1), SLEN, INTBUF(1:10))
                    OLINE(CPOS:(CPOS+SLEN-1)) = INTBUF
                    CPOS = CPOS + SLEN
                  END IF
                END IF
              END IF
            END IF
            OLINE(CPOS:CPOS) = ')'
          END IF
          WRITE(OUNIT,'(A)') OLINE
 800    CONTINUE
C
        WRITE(OUNIT, 2002)
      END IF
C
C     close and delete the scratch file opened on TUNIT1
      CLOSE (TUNIT1)
C     close the sort file
      CLOSE (TUNIT2)
C
C     if debuging is turned off delete the sort output file
      IF (DEBUG .EQ. 0) THEN
        CALL DOCDEL (SRTNAM, ERCOD2)
        IF (ERCOD2 .NE. 0)
     +    CALL ERROUT (' Error in call to DOCDEL','DUMPX')
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   DEFIMP
C
C     + + + PURPOSE + + +
C     This routine sets the implied ANSI fortran default variable
C     types which can be modified by the IMPLICIT statement
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'cimp.inc'
      INCLUDE 'dimp.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER LOOP
C
C     + + + END SPECIFICATIONS + + +
C
      DO 100 LOOP = 1, 26
        ALPHAB(LOOP) = 'R'
 100  CONTINUE
C
      DO 200 LOOP = 9, 14
        ALPHAB(LOOP) = 'I'
 200  CONTINUE
C
      IMPFLG = .FALSE.
      RETURN
      END
C
C
C
      SUBROUTINE   REVIMP (
     O                     NXTFLG, ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to revise the default variable types as implied
C     by the IMPLICIT statement.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER ERRCOD
      LOGICAL NXTFLG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     NXTFLG - flag to tell calling routine that no additional
C              tokens in the token buffer need to be processed
C              .TRUE. if processing of current buffer is complete
C              .FALSE. otherwise
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'cimp.inc'
      INCLUDE 'dimp.inc'
      INCLUDE 'ctokea.inc'
      INCLUDE 'ctokeb.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     SVAL, EVAL, LOOP, LOOP2
      CHARACTER*1 VTYPE, LETTER(26)
      LOGICAL     OPARIN, CPARIN, INVALV, INVALS, INVALC,
     +            AGAIN, NEWRNG, IMPNON
C
C     + + + EXTERNALS + + +
      EXTERNAL ERROUT, DUMPBF, BUMPTK
C
C     + + + DATA INITIALIZATIONS + + +
      DATA LETTER/'A','B','C','D','E','F','G','H','I','J','K','L','M',
     +            'N','O','P','Q','R','S','T','U','V','W','X','Y','Z'/
C
C     + + + END SPECIFICATIONS + + +
C
C     The value of the variable VTYPE will be set to
C     one of the following:
C
C         'D' = double precision
C         'R' = real
C         'I' = integer
C         '2' = integer*2
C         '4' = integer*4
C         'J' = integer*?
C         'C' = character
C         'L' = logical
C         'X' = complex
C
C
C     initialize flag which indicates if IMPLICIT applies to
C     a range of values or a single value - default is .FALSE.
      ERRCOD = 0
      INVALV = .FALSE.
      INVALS = .FALSE.
      INVALC = .FALSE.
      IMPNON = .FALSE.
      NXTFLG = .FALSE.
C
 100  CONTINUE
        AGAIN = .FALSE.
        OPARIN = .FALSE.
        CPARIN = .FALSE.
C       get next token
        CALL BUMPTK(1,ERRCOD)
        IF (ERRCOD .EQ.0) THEN
          SVAL   = 0
          EVAL   = 0
          VTYPE = ' '
          IF (TOKEN(1:TOKLEN) .EQ. 'NONE') THEN
C           set flag to indicate that NONE was found
            IMPNON = .TRUE.
            NXTFLG = .TRUE.
            AGAIN  = .FALSE.
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'INTEGER') THEN
            VTYPE = 'I'
            CALL BUMPTK(1,ERRCOD)
            IF (TOKEN(1:TOKLEN) .EQ. '(') THEN
              OPARIN = .TRUE.
            ELSE IF (TOKEN(1:TOKLEN) .EQ. '*') THEN
              CALL BUMPTK(1,ERRCOD)
              IF (TOKEN(1:TOKLEN) .EQ. '2') THEN
C               set variable type to '2' = integer*2
                VTYPE = '2'
              ELSE IF (TOKEN(1:TOKLEN) .EQ. '4') THEN
C               set variable type to '4' = integer*4
                VTYPE = '4'
              ELSE
C               set variable type to 'J' = undetermined
C               integer type
                VTYPE = 'J'
              END IF
            END IF
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'DOUBLE') THEN
            CALL BUMPTK(1,ERRCOD)
            IF (TOKEN(1:TOKLEN) .EQ. 'PRECISION') THEN
              VTYPE = 'D'
            ELSE
              ERRCOD = -1
            END IF
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'COMPLEX') THEN
C           set variable type to 'X' = complex
            VTYPE = 'X'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'REAL') THEN
C           set variable type to 'R' = real
            VTYPE = 'R'
            CALL BUMPTK(1,ERRCOD)
            IF (TOKEN(1:TOKLEN) .EQ. '(') THEN
              OPARIN = .TRUE.
            ELSE IF (TOKEN(1:TOKLEN) .EQ. '*') THEN
              CALL BUMPTK(1,ERRCOD)
              IF (TOKEN(1:TOKLEN) .EQ. '4') THEN
C               set variable type to 'R' = real
                VTYPE = 'R'
              ELSE IF (TOKEN(1:TOKLEN) .EQ. '8') THEN
C               set variable type to 'D' = double precision
                VTYPE = 'D'
              END IF
            END IF
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'LOGICAL') THEN
C           set variable type to 'L' = logical
            VTYPE = 'L'
          ELSE IF (TOKEN(1:TOKLEN) .EQ. 'CHARACTER') THEN
C           set variable type to 'C' = character
            VTYPE = 'C'
          ELSE
            ERRCOD = -1
            INVALV = .TRUE.
          END IF
C
          IF (ERRCOD .NE. 0) THEN
C           invalid statement
            INVALS = .TRUE.
          ELSE IF (.NOT.NXTFLG) THEN
C           found variable type
C
            IF (.NOT.OPARIN) THEN
C             load next token prior to checking for open parin
              CALL BUMPTK(1,ERRCOD)
              IF (ERRCOD .NE. 0) INVALS = .TRUE.
            ENDIF
C
            IF (TOKEN(1:TOKLEN) .NE. '(') THEN
C             open parin does not follow variable type
C             invalid statement
              INVALS = .TRUE.
              ERRCOD = -1
            ELSE
C             found open parin, get next token
              CALL BUMPTK(1,ERRCOD)
              IF (ERRCOD .NE. 0) THEN
C               invalid statement
                INVALS = .TRUE.
              ELSE
 300            CONTINUE
                  SVAL   = 0
                  EVAL   = 0
                  NEWRNG = .FALSE.

                  IF (TTYPE .EQ. 'C' .AND. TOKLEN .EQ. 1) THEN
C                   found alpha character
                    LOOP = 0
C                   determine character position in ALPHAB array
 400                CONTINUE
                      LOOP = LOOP + 1
                      IF (LOOP .LE. 26) THEN
                        IF (LETTER(LOOP) .EQ. TOKEN) THEN
C                         set starting value equal to first ALPHAB element
C                         affected
                          SVAL   = LOOP
C                         set ALPHAB element to new variable type default
                          ALPHAB(LOOP) = VTYPE
                        END IF
                      ELSE
C                       invalid character
                        INVALC = .TRUE.
                        ERRCOD = -1
                      END IF
                    IF (ERRCOD .EQ. 0 .AND. LETTER(LOOP) .NE. TOKEN)
     +                  GOTO 400
                  ELSE
C                   invalid statement
                    INVALS = .TRUE.
                    ERRCOD = -1
                  END IF
C
                  IF (ERRCOD .EQ. 0) THEN
C                   get next token
 600                CONTINUE
                      CPARIN = .FALSE.
                      CALL BUMPTK(1,ERRCOD)
                      IF (ERRCOD .NE. 0) THEN
                        INVALS = .TRUE.
                      ELSE
                        IF (TOKEN(1:TOKLEN) .EQ. ')') THEN
C                         found close parin
                          CPARIN = .TRUE.
                          CALL BUMPTK(1,ERRCOD)
                          IF (ERRCOD .EQ. 0) THEN
                            IF (TOKEN(1:TOKLEN) .EQ. ',') THEN
C                             found comma after close parin
C                             set flag to loop up for next variable type
                              AGAIN = .TRUE.
                            ELSE
C                             invalid token follows close parin
                              INVALS = .TRUE.
                              ERRCOD = -1
                            END IF
                          ELSE
C                           no tokens follow close parin, end of line
                            NXTFLG = .TRUE.
                          END IF
C
                        ELSE IF (TOKEN(1:TOKLEN) .EQ. ',') THEN
C                         found a comma, get next token
                          CALL BUMPTK(1,ERRCOD)
                          IF (ERRCOD .EQ. 0) THEN
                            NEWRNG = .TRUE.
                          ELSE
                            ERRCOD = -1
                            INVALS = .TRUE.
                          END IF
C
                        ELSE IF (TOKEN(1:TOKLEN) .EQ. '-'
     +                           .AND. SVAL .NE. 0) THEN
C                         found "range-of-values" indication -
C                         a dash between characters
                          CALL BUMPTK(1,ERRCOD)
                          IF (ERRCOD .NE. 0) THEN
                            INVALS = .TRUE.
                          ELSE
                            IF (TTYPE .EQ. 'C' .AND. TOKLEN .EQ. 1) THEN
C                             found alpha character
C                             determine character position in ALPHAB array
                              LOOP = 0
 800                          CONTINUE
                                LOOP = LOOP + 1
                                IF (LOOP .LE. 26) THEN
                                  IF (LETTER(LOOP) .EQ. TOKEN) THEN
C                                   set ending range value equal to
C                                   last ALPHAB element
                                    EVAL   = LOOP
C                                   set default variable types in ALPHAB
                                    DO 700 LOOP2 = SVAL, EVAL, 1
                                      ALPHAB(LOOP2) = VTYPE
 700                                CONTINUE
                                  END IF
                                ELSE
C                                 non-alphabetic character
                                  INVALC = .TRUE.
                                  ERRCOD = -1
                                END IF
                              IF (ERRCOD .EQ. 0 .AND. EVAL .EQ. 0)
     +                            GOTO 800
                            ELSE
C                             token was not a character or was more
C                             than 1 character
                              INVALS = .TRUE.
                              ERRCOD = -1
                            END IF
                          END IF
                        ELSE
                          INVALS = .TRUE.
                          ERRCOD = -1
                        END IF
                      END IF
                    IF (.NOT.CPARIN) GOTO 600
                  END IF
                IF (NEWRNG) GOTO 300
              END IF
            END IF
          END IF
        ELSE
          ERRCOD = -1
          INVALS = .TRUE.
        END IF
      IF (AGAIN) GOTO 100
C
      IF (INVALV) THEN
        CALL ERROUT (' Invalid variable type in IMPLICIT stmt','REVIMP')
        CALL DUMPBF
      END IF
C
      IF (INVALC) THEN
        CALL ERROUT (' Invalid char in IMPLICIT statement','REVIMP')
        CALL DUMPBF
      END IF
C
      IF (INVALS) THEN
        CALL ERROUT (' Invalid IMPLICIT Statement','REVIMP')
        CALL DUMPBF
      END IF
C
C     set flag to indicate non-default IMPLICIT values are in use
      IF (.NOT.IMPNON) IMPFLG = .TRUE.
C
      RETURN
      END
C
C
C
      SUBROUTINE   BUMPTK
     I                    (REL,
     O                     ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to load token into token buffer
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER REL, ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     REL    - position of token to load relative to position
C              of current token
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed, next token does not exist
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokea.inc'
      INCLUDE 'ctokeb.inc'
C
C     + + + EXTERNALS + + +
      EXTERNAL LOADBF
C
C     + + + END SPECIFICATIONS + + +
C
      TNUM = TNUM + REL
      IF (TNUM .GT. 0 .AND. TNUM .LE. ITEMS) THEN
C       requested token exists
        CALL LOADBF(ERRCOD)
      ELSE
C       requested token does not exist
        ERRCOD = -1
      END IF
      RETURN
      END
C
C
C
      SUBROUTINE   ADVANC
     I                    (ALPH,
     O                     ERRCOD)
C
C     + + + PURPOSE + + +
C     routine to loop through tokens in token buffer and search
C     for character = ALPH.  If found, the value of TNUM is set
C     to the token position where token was found
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER     ERRCOD
      CHARACTER*1 ALPH
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ALPH   - alphnumeric character to search for
C     ERRCOD - error code returned to calling program
C               0 - operation was successful
C              -1 - operation failed, requested token not found
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokeb.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ERRFLG
      LOGICAL GETOUT
C
C     + + + EXTERNALS + + +
      EXTERNAL BUMPTK
C
C     + + + END SPECIFICATIONS + + +
C
      GETOUT = .FALSE.
 100  CONTINUE
        CALL BUMPTK(1,ERRFLG)
        IF (ERRFLG .EQ. 0) THEN
C         process token
          IF (TOKEN(1:1) .EQ. ALPH .AND. TOKLEN .EQ. 1) THEN
C           this is the character being sought
            ERRCOD = 0
            GETOUT = .TRUE.
          END IF
        ELSE
C         no more tokens to look at
          ERRCOD = -1
          GETOUT = .TRUE.
        END IF
      IF (.NOT.GETOUT) GOTO 100
      RETURN
      END
