C
C
C
      SUBROUTINE   MAKXRF
     I                   (FNAME, SOUR, DINTLN, INTFIL, XRFNAM, SRTNAM,
     I                    PATH, PTHLEN,
     O                    ERRCOD)
C
C     + + + PURPOSE + + +
C     This routine creates a cross reference map for the source code
C     contained in the file = FNAME.
C
C     An output file containing the cross reference map is created
C     and given the name nnnnnnnn.XRF where nnnnnnnn = the first 8 or
C     fewer characters of the input file name preceding the first '.'
C     (dot) in the filename passed in FNAME.
C
C     This routine works with ANSI Standard Fortran 77 code.
C
C     NOTE - All INCLUDE files must be in the same directory as
C            the source code files or this routine will abort!
C
C     + + + HISTORY + + +
C     This routine was written in order to provide portability of the
C     SYSDOC system.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       DINTLN, PTHLEN, ERRCOD
      CHARACTER*1   SOUR(8)
      CHARACTER*(*) FNAME,  XRFNAM, SRTNAM, INTFIL, PATH
C
C     + + + ARGUMENT DEFINITIONS + + +
C     FNAME  - name of source code file to be processed
C     SOUR   - name of source code file void of any suffix
C     DINTLN - default integer length (2 or 4)
C     INTFIL - path name of file containing list of valid INTRINSICS
C     XRFNAM - name of cross reference file
C     SRTNAM - name of temporary file used to sort cross reference
C              information
C     PATH   - path name exclusive of FNAME
C     PTHLEN - length of path name
C     ERRCOD - error code returned to calling routine
C               0 if operation was successful
C              -1 if operation failed
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'ctokea.inc'
      INCLUDE 'dtokea.inc'
      INCLUDE 'ctokeb.inc'
      INCLUDE 'dtokeb.inc'
      INCLUDE 'ctokec.inc'
      INCLUDE 'dtokec.inc'
      INCLUDE 'cdebug.inc'
      INCLUDE 'ddebug.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      IUNIT, OUNIT, LOOP1, PARRAY, GOBACK,
     +             TUNIT1, TUNIT2, ERRFLG, IOSTA0
      CHARACTER*8  SOUR2
      CHARACTER*64 INAME, INAME2
      CHARACTER*80 LINE
      LOGICAL      NXTFLG, ENTFLG, SPCFLG, DATFLG, EXEFLG, STFFLG,
     +             INCFLG, INFLG1, INFLG2, CONTIN, DONFLG, GETOUT,
     +             EOFFLG, COMENT, NEXT, IOPEN, OOPEN
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + EXTERNALS + + +
      EXTERNAL ZIP, GUNIT, ERROUT, LDINTR,
     +         DEFIMP, FTKBUF, LOADBF, FNDENT, GINCNM,
     +         FNDSPC, DUMPX, DUMPBF, FNDDAT, FNDEXE
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT ('   Invalid characters in columns 1-5 ',/,
     +        '   File = ',A,'Line = ',I4,/)
 2010 FORMAT (A80)
 2020 FORMAT (I5,1X,A80)
 2030 FORMAT (/,'  Processing Routine Number',I3,
     +          '  in File ',A64)
 2090 FORMAT ('      PROGRAM   ',A8)
 2100 FORMAT(8A1)
C
C     + + + END SPECIFICATIONS + + +
C
      GETOUT = .FALSE.
      IOPEN  = .FALSE.
      OOPEN  = .FALSE.
      WRITE (SOUR2,2100) SOUR
C
C     initialize default integer to length 2 (Short Integers) if not
C     passed from calling program
      IF (DINTLN .NE. 2 .AND. DINTLN .NE. 4) THEN
C       default integer length is INTEGER*2
        CINTLN = 2
      ELSE
C       default integer length was set by user
        CINTLN = DINTLN
      END IF
C
C     load compiler specific INTRINSICs
      CALL LDINTR (INTFIL, ERRCOD)
      IF (ERRCOD .NE. 0) THEN
        GETOUT = .TRUE.
        CALL ERROUT (' Error in call to LDINTR','MAKXRF')
      END IF
C
      IF (.NOT.GETOUT) THEN
C       determine default IMPLICIT variable values
        CALL DEFIMP
C
        LCOUNT = 0
        PRLINE = 0
        EOFFLG = .FALSE.
        RTNNUM = 1
C
C       open source code file
        CALL GUNIT (IUNIT, ERRCOD)
        OPEN (UNIT=IUNIT, FILE=FNAME, STATUS='OLD',
     +        ACCESS='SEQUENTIAL', IOSTAT=IOSTA0)
        IF (IOSTA0 .EQ. 0) THEN
C         input file is opened,
          IOPEN = .TRUE.
C         now open output file
          CALL GUNIT (OUNIT, ERRCOD)
          OPEN (UNIT=OUNIT, FILE=XRFNAM, ACCESS='SEQUENTIAL',
     +          IOSTAT=IOSTA0)
          IF (IOSTA0.EQ. 0) THEN
C           output file is open
            OOPEN = .TRUE.
          ELSE
            CALL ERROUT ('Error opening output file','MAKXRF')
            ERRCOD = -1
            GETOUT = .TRUE.
          END IF
        ELSE
          CALL ERROUT (' Error opening input file','MAKXRF')
          ERRCOD = -1
          GETOUT = .TRUE.
        END IF
      END IF
C
      IF (.NOT.GETOUT) THEN
C       loop to process program modules
 200    CONTINUE
          WRITE (*, 2030) RTNNUM, FNAME(1:64)
          SYMNUM = 0
C         routine entry flag
          ENTFLG = .TRUE.
C         parameter counter
          PARCNT = 0
C         specification flag
          SPCFLG = .FALSE.
C         data definition flag
          DATFLG = .FALSE.
C         statement function flag
          STFFLG = .FALSE.
C         executable code flags
          EXEFLG = .FALSE.
          YESEXE = .FALSE.
C         flag to indicate that a subroutine need not be called again
          DONFLG = .FALSE.
C         end-of-line flag
          EOLFLG = .FALSE.
C
C         flags to indicate that an INCLUDE or $INSERT
C         file is being processed
          INCFLG = .FALSE.
C         first level INCLUDE file
          INFLG1 = .FALSE.
C         second level (nested) include file
          INFLG2 = .FALSE.
C
          CALL ZIP (' ', LINE)
C
C         Loop to read each line of the source code
 300      CONTINUE
            COMENT = .FALSE.
            NEXT   = .FALSE.
            READ (IUNIT, 1000, IOSTAT=IOSTA0) LINE
            IF (IOSTA0 .EQ. 0) THEN
              IF (.NOT. INCFLG) LCOUNT = LCOUNT + 1
              PRLINE = LCOUNT
              IF (.NOT.YESEXE .AND. .NOT.INCFLG) THEN
                WRITE(OUNIT, 2010)LINE
              END IF
              IF (DEBUG .GT. 1) WRITE (*, 2020) PRLINE, LINE
C
              IF (LINE(1:1) .EQ. 'C') THEN
C               comment line, no processing required
                COMENT = .TRUE.
                NEXT   = .TRUE.
              ELSE
C               fill token buffer (TKBUFF)
                CALL FTKBUF (LINE, 1, ERRCOD)
                IF (ERRCOD .NE. 0) THEN
                  CALL ERROUT (' Error in call to FTKBUF','MAKXRF')
                  ERRCOD = -1
                  GETOUT = .TRUE.
                ELSE IF (ITEMS .EQ. 0) THEN
C                there are no items on the current line, read next line
                 NEXT = .TRUE.
                ELSE
C                 a valid line has been found - look ahead in source
C                 file to determine if continuation lines exist
                  GOBACK = 0
 400              CONTINUE
                    CONTIN = .FALSE.
                    COMENT = .FALSE.
                    READ (IUNIT, '(A)', IOSTAT=IOSTA0) LINE
                    IF (IOSTA0 .EQ. 0) THEN
                      IF (.NOT. INCFLG) LCOUNT = LCOUNT + 1
                      IF (LINE(1:1) .EQ. 'C') THEN
C                       comment line, loop until the
C                       next executable statement is found.
                        GOBACK = GOBACK + 1
                        COMENT = .TRUE.
                      ELSE IF (LINE(1:7) .EQ. '$INSERT') THEN
C                       found a PRIME COMPUTER Fortran INCLUDE file
                        GOBACK = GOBACK + 1
                      ELSE IF (LINE(6:6) .NE. ' ') THEN
C                       continuation line
                        CONTIN = .TRUE.
                        IF (.NOT.YESEXE .AND. .NOT.INCFLG) THEN
                          WRITE (OUNIT, 2010) LINE
                        END IF
                        IF (DEBUG .GT. 1) WRITE (*, 2020) PRLINE, LINE
                        GOBACK = GOBACK + 1
C                       append token buffer with continuation line
                        PARRAY = ITEMS + 1
                        CALL FTKBUF (LINE, PARRAY, ERRCOD)
                        IF (ERRCOD .NE. 0) THEN
                          CALL ERROUT (' Error in call to FTKBUF',
     +                                 'MAKXRF')
                          CALL DUMPBF
                          GETOUT = .TRUE.
                        END IF
C                       reset GOBACK counter
                        GOBACK = 0
C                       look at next line
                      ELSE
                        GOBACK = GOBACK + 1
                      END IF
                    ELSE IF (IOSTA0 .GT. 0) THEN
C                     error during read
                      ERRCOD = -1
                      CALL ERROUT (' Error during READ','MAKXRF')
                      GETOUT = .TRUE.
                    ELSE
                      EOFFLG = .TRUE.
                    END IF
                  IF (.NOT.GETOUT .AND. .NOT.EOFFLG
     +                .AND. (COMENT .OR. CONTIN) ) GOTO 400
C
                  IF (.NOT.GETOUT .AND. GOBACK .GT. 0) THEN
C                   reposition input file to location of
C                   line being processed
                    DO 500 LOOP1 = 1, GOBACK, 1
                      BACKSPACE IUNIT
 500                CONTINUE
C                   reset line counters
                    IF (.NOT. INCFLG) LCOUNT = LCOUNT - GOBACK
                    GOBACK = 0
                    NEXT = .FALSE.
                  END IF
                END IF
              END IF
            ELSE IF (IOSTA0 .GT. 0) THEN
              CALL ERROUT (' Error during Read','MAKXRF')
              CALL DUMPBF
              GETOUT = .TRUE.
            ELSE
              EOFFLG = .TRUE.
            END IF
C
C           end look-ahead process
C           NOTE: contents of LINE buffer are not valid beyond
C           this point - use only data located in token buffer
C
C           all source code lines processed beyond this point are
C           executable fortran code
C
            IF (ENTFLG .AND. .NOT.NEXT
     +          .AND. .NOT.GETOUT .AND. .NOT.EOFFLG) THEN
C             process routine entry information
              TNUM = 1
              CALL LOADBF(ERRFLG)
C
              IF (TSTART .GT. 6) THEN
C               first token starts in column 7 or greater
C               call routine to locate routine entry information
                CALL FNDENT (SOUR2, NXTFLG, ERRCOD)
                IF (ERRCOD .NE. 0) THEN
                  CALL ERROUT (' Error in call to FNDENT','MAKXRF')
                  CALL DUMPBF
                  GETOUT = .TRUE.
                ELSE IF (.NOT.NXTFLG) THEN
                  NEXT = .FALSE.
                  BACKSPACE OUNIT
                  READ (OUNIT, 1000) LINE
                  IF (RTNNUM .EQ. 1) WRITE(OUNIT, 2090) SOUR2
                  WRITE(OUNIT, 2010)LINE
                ELSE
                  NEXT = .TRUE.
                END IF
              ELSE
                CALL ERROUT (' Invalid Entry Point','MAKXRF')
                CALL DUMPBF
                GETOUT = .TRUE.
              END IF
C
              NXTFLG = .FALSE.
              ENTFLG = .FALSE.
              SPCFLG = .TRUE.
            END IF
C
            IF (SPCFLG .AND. .NOT.NEXT
     +          .AND. .NOT.GETOUT .AND. .NOT.EOFFLG) THEN
C             process source code specification statements
              TNUM = 1
              CALL LOADBF(ERRFLG)
C
C             determine if this is an INCLUDE or $INSERT file
              IF (TOKEN(1:TOKLEN) .EQ. 'INCLUDE' .OR.
     +            ( TOKEN(1:TOKLEN) .EQ. '$' .AND.
     +            NTOKEN .EQ. 'INSERT') ) THEN
                IF (.NOT. INFLG1) THEN
C                 set flag to indicate this is level 1 INCLUDE
                  INFLG1 = .TRUE.
                  TUNIT1 = IUNIT
                  INCFLG = .TRUE.
                ELSE
C                 this is second level of nesting for INCLUDE
                  INFLG2 = .TRUE.
                  TUNIT2 = TUNIT1
                  TUNIT1 = IUNIT
                  INCFLG = .TRUE.
                END IF
C
C               call routine to get INCLUDE filename
                CALL GINCNM (PATH, PTHLEN, INAME, INAME2, ERRCOD)
                IF (ERRCOD .NE. 0) THEN
                  CALL ERROUT (' Error in call to GINCNM','MAKXRF')
                  CALL DUMPBF
                  GETOUT = .TRUE.
                ELSE
C                 open the INCLUDE file
                  CALL GUNIT (IUNIT, ERRCOD)
                  OPEN (UNIT=IUNIT, FILE=INAME, STATUS='OLD',
     +                   ACCESS='SEQUENTIAL', IOSTAT=ERRCOD)
                    IF (ERRCOD .NE. 0) THEN
C                     failed with INAME, try INAME2
                      OPEN (UNIT=IUNIT, FILE=INAME2, STATUS='OLD',
     +                    ACCESS='SEQUENTIAL', IOSTAT=ERRCOD)
                      IF (ERRCOD .NE. 0) THEN
C                       include file does not exist
                        CALL ERROUT (' Error opening INCLUDE file',
     +                               'MAKXRF')
                        CALL DUMPBF
                        GETOUT = .TRUE.
                      END IF
                    END IF
                  NEXT = .TRUE.
                END IF
              END IF
              IF (.NOT.NEXT) THEN
C               call routine to process specification statements
                CALL FNDSPC (DONFLG, NXTFLG, ERRCOD)
                IF (ERRCOD .EQ. 0) THEN
                  IF (.NOT.DONFLG .AND. NXTFLG) THEN
                    NEXT = .TRUE.
                  ELSE IF (DONFLG) THEN
                    SPCFLG = .FALSE.
                    DATFLG = .TRUE.
                  END IF
                ELSE
                  CALL ERROUT (' Error in call to FNDSPC','MAKXRF')
                  CALL DUMPBF
                  GETOUT = .TRUE.
                END IF
              END IF
            END IF
C
            IF (DATFLG .AND. .NOT.NEXT
     +          .AND. .NOT.GETOUT .AND. .NOT.EOFFLG) THEN
C             process data initialization section of source code
              TNUM = 1
              CALL LOADBF(ERRFLG)
              IF (TSTART .GT. 6 .AND. TOKEN(1:TOKLEN) .EQ.'DATA') THEN
                CALL FNDDAT (DONFLG, NXTFLG, ERRCOD)
                IF (ERRCOD .EQ. 0 .AND. NXTFLG) THEN
                  NEXT = .TRUE.
                ELSE
                  CALL ERROUT (' Error in call to FNDDAT','MAKXRF')
                  WRITE(*,2000) FNAME, PRLINE
                  CALL DUMPBF
                  GETOUT = .TRUE.
                END IF
              ELSE
                DATFLG = .FALSE.
                STFFLG = .TRUE.
              END IF
            END IF
C
            IF ( (STFFLG .OR. EXEFLG) .AND. .NOT.NEXT
     +           .AND. .NOT.GETOUT .AND. .NOT.EOFFLG) THEN
C             this is a hook to add processing for source code
C             statement functions, however, this code is not
C             required to meet SYSDOC requirements - no processing
C             occurs
              STFFLG = .FALSE.
              EXEFLG = .TRUE.
            END IF
C
            IF (EXEFLG .AND. .NOT.NEXT
     +          .AND. .NOT.GETOUT .AND. .NOT.EOFFLG) THEN
C             process executable source code lines
              TNUM = 1
              IF (TKBUFF(2) .EQ. 'FORMAT' .AND. (.NOT.YESEXE)) THEN
C               found a 'FORMAT' statement preceeding executable code
C               (these lines get printed to the cross reference file)
                NEXT = .TRUE.
              ELSE IF (.NOT. YESEXE) THEN
C               this is the first line of executable code, backspace
C               the output unit by one line so that the next write to
C               the output unit overwrites it in the xref file since
C               the xref file is not to contain executable code
                YESEXE = .TRUE.
                BACKSPACE OUNIT
              END IF
C
              IF (.NOT.NEXT) THEN
C               call routine to find and process executable code
                CALL FNDEXE (DONFLG, NXTFLG, ERRCOD)
                IF (ERRCOD .NE. 0) THEN
                  WRITE(*,2000) FNAME, PRLINE
                  CALL ERROUT (' Error in call to FNDEXE','MAKXRF')
                  CALL DUMPBF
                  GETOUT = .TRUE.
                ELSE IF (.NOT.DONFLG .AND. NXTFLG) THEN
                  NEXT = .TRUE.
                ELSE IF (DONFLG .AND. NXTFLG) THEN
C                 finished with routine
                  EXEFLG = .FALSE.
C                 read ahead and eliminate any blank comment
C                 lines between program modules so they do not
C                 get written to output file
                  GOBACK = 0
 600              CONTINUE
                    COMENT = .FALSE.
                    READ (IUNIT, '(A)', IOSTAT = IOSTA0) LINE
                    IF (IOSTA0 .EQ. 0) THEN
                      IF (.NOT. INCFLG) LCOUNT = LCOUNT + 1
C                     fill token buffer (TKBUFF)
                      CALL FTKBUF (LINE, 1, ERRCOD)
                        IF (ERRCOD .NE. 0)
     +                      CALL ERROUT (' Error in call to FTKBUF',
     +                      'MAKXRF')
                      IF (ITEMS .EQ. 0) THEN
C                       no items on current line
C                       consider it a blank comment line
                        COMENT = .TRUE.
                      ELSE IF (LINE(1:1) .EQ. 'C') THEN
C                       comment line
                        COMENT = .TRUE.
                      ELSE
                        GOBACK = GOBACK + 1
                      END IF
                    ELSE IF (IOSTA0 .LT. 0) THEN
C                     end-of-file detected
                      EOFFLG = .TRUE.
                    END IF
                  IF (COMENT .AND. .NOT.EOFFLG) GOTO 600
C
                  IF (.NOT.EOFFLG) THEN
                    IF (GOBACK .GT. 0) THEN
C                     reposition input file to location of
C                     non-blank comment line
                      DO 700 LOOP1 = 1, GOBACK, 1
                        BACKSPACE IUNIT
 700                  CONTINUE
                    END IF
C                   reset line counters
                    IF (.NOT. INCFLG) LCOUNT = LCOUNT - 1
                    GOBACK = 0
                    NEXT = .FALSE.
                  END IF
                ELSE
                  CALL ERROUT (' Error in call to FNDEXE','MAKXRF')
                  CALL DUMPBF
                  GETOUT = .TRUE.
                END IF
              END IF
            END IF
            IF (EOFFLG) THEN
C             end-of-file was encountered during a read
              IF (INFLG2) THEN
C               EOF occurred while reading an INCLUDE or $INSERT file
                CLOSE (IUNIT)
                IUNIT  = TUNIT1
                TUNIT1 = TUNIT2
                INFLG2 = .FALSE.
                EOFFLG = .FALSE.
                NEXT = .TRUE.
              ELSE IF (INFLG1) THEN
                CLOSE (IUNIT)
                IUNIT  = TUNIT1
                INFLG1 = .FALSE.
                INCFLG = .FALSE.
                EOFFLG = .FALSE.
                NEXT = .TRUE.
              END IF
            END IF
          IF (.NOT.GETOUT .AND. .NOT.EOFFLG .AND. NEXT) GOTO 300
C
          IF (.NOT.GETOUT) THEN
C           dump xref information to output file
            CALL DUMPX (SRTNAM, OUNIT, ERRCOD)
            RTNNUM = RTNNUM + 1
            ENTFLG = .TRUE.
C           parameter counter
            PARCNT = 0
          END IF
        IF (.NOT.EOFFLG .AND. .NOT.GETOUT) GOTO 200
      END IF
      IF (OOPEN) CLOSE (OUNIT)
      IF (IOPEN) CLOSE (IUNIT)
      RETURN
      END
