C
C
C
      SUBROUTINE   PIKSRT
     I                   (IUNIT, OUNIT, IKEYS, ICOL1, ICOL2,
     I                    LONG, NRECS,
     O                    ERRCOD)
C
C     + + + PURPOSE + + +
C     Routine to sort records in file opened on file unit IUNIT by
C     by supplied keys into ascending order with the results written
C     to the file opened on file unit OUNIT.
C
C     + + + HISTORY + + +
C     09/02/94  kmf  rewritten using selection sort
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER   IKEYS
      INTEGER   IUNIT, OUNIT, ICOL1(IKEYS), ICOL2(IKEYS),
     +          LONG, NRECS, ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT  - file unit number of input file
C     OUNIT  - file unit number of output file
C     IKEYS  - contains the number of sort keys to sort on.
C              If NKEYS = I, then the sort is performed on
C              the entire record.
C     ICOL1  - integer array which contains the first column
C              number of each sort key
C     ICOL2  - integer array which contains the last column number
C              of each sort key
C     LONG   - length of records to be sorted
C     NRECS  - number of records to be sorted
C     ERRCOD - contains an error code which is returned to the
C              calling program.
C               0 if operation was successful
C              -1 if operation failed
C
C     + + + LOCAL VARIABLES + + +
      INTEGER   NUNIT, IOSTA0, IOSTA1, STATFL,
     $          I, J, RECM, REC, RECI, LNGTHM, LNGTHT, LENKEY,
     $          LNGEST, NKEYS, NCOL1(6), NCOL2(6)
      CHARACTER*250 BUFM, KEYM, BUFT, KEYT, BUFI
C
C     + + + INTRINSICS + + +
      INTRINSIC LLT
C
C     + + + EXTERNALS + + +
      EXTERNAL CONSTR, GUNIT, ERROUT
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (A)
 2010 FORMAT ( I10 )
C
C     + + + END SPECIFICATIONS + + +
C
      IF (IKEYS .LT. 6  .AND.  LONG .LE. 240) THEN
C       number of keys and record length are ok
        LNGEST = LONG + 10
        LENKEY = 0
        DO 50 I = 1, IKEYS
          NCOL2(I) = ICOL2(I)
          NCOL1(I) = ICOL1(I)
          LENKEY = LENKEY + (NCOL2(I) - NCOL1(I) + 1)
 50     CONTINUE
        NKEYS = IKEYS + 1
        NCOL2(NKEYS) = LONG + 10
        NCOL1(NKEYS) = LONG + 1
C       open direct access scratch file for the sort
        ERRCOD = 0
        NUNIT = 70
        CALL GUNIT(NUNIT, ERRCOD)
        OPEN (UNIT = NUNIT,
     $      STATUS = 'SCRATCH',
     $      ACCESS = 'DIRECT',
     $        RECL = LNGEST,
     $      IOSTAT = IOSTA0)
        IF (IOSTA0 .GT. 0) THEN
C         error occurred during open
          CALL ERROUT('Error in OPEN', 'PIKSRT')
          ERRCOD =  -1
        ELSE
C         write contents of input file to scratch file
          STATFL = 0
          I = 0
 100      CONTINUE
            I = I + 1
            READ (IUNIT,1000,IOSTAT=IOSTA1) BUFT(1:LONG)
            IF (IOSTA1 .LT. 0) THEN
C             end-of-file encountered
              STATFL = 1
            ELSE IF (IOSTA1 .GT. 0) THEN
C             error during read
              STATFL = -1
            ELSE
C             save record
              REC = I
              WRITE (BUFT(LONG+1:LNGEST),2010) REC
              WRITE (NUNIT, REC = I) BUFT(1:LNGEST)
            END IF
          IF ( I .LT. NRECS  .AND.  STATFL .EQ. 0) GO TO 100
          IF (STATFL .NE. 0) THEN
C           error reading input file
            CALL ERROUT(' Error during READ','PIKSRT')
          END IF
        END IF
      ELSE
C       too many keys or record too long
        ERRCOD = -1
      END IF
      IF (STATFL .GE. 0  .AND.  ERRCOD .EQ. 0) THEN
C       get key length
        LENKEY = 0
        DO 200 I = 1, NKEYS
          LENKEY = LENKEY + (NCOL2(I) - NCOL1(I) + 1)
 200    CONTINUE
C       sort records
        DO 440 I = 1, NRECS-1
C         assume record I is the alphabetically lowest
          RECI = I
          RECM = RECI
          REC  = RECI
          READ (NUNIT,REC=REC) BUFM(1:LNGEST)
          BUFI(1:LNGEST) = BUFM(1:LNGEST)
          CALL CONSTR ( BUFM, NKEYS, NCOL1, NCOL2, 
     O                  KEYM, LNGTHM, ERRCOD )
          DO 420 J = I+1, NRECS
C           compare all records past I for lowest
            REC = J
            READ (NUNIT,REC=REC) BUFT(1:LNGEST)
            CALL CONSTR ( BUFT, NKEYS, NCOL1, NCOL2,
     O                    KEYT, LNGTHT, ERRCOD )
            IF ( LLT ( KEYT, KEYM ) ) THEN
C             new minimum, save pointer and update minimum
              RECM = J
              BUFM(1:LNGEST) = BUFT(1:LNGEST)
              KEYM(1:LENKEY) = KEYT(1:LENKEY)
            END IF
 420      CONTINUE
C         save minimum to sorted file
          WRITE (OUNIT,2000) BUFM(1:LONG)
          IF (RECM .NE. RECI) THEN
C           save record i to minimum record
            REC = RECM
            WRITE (NUNIT,REC=REC) BUFI(1:LNGEST)
          END IF
 440    CONTINUE
C       save largest (last) value to sorted file
        WRITE (OUNIT,2000) BUFT(1:LONG)
C       close scratch file
        CLOSE ( NUNIT )
      END IF
C
      RETURN
      END
