C
C
C
      SUBROUTINE   SRTSYM (
     I                      TUNIT1, TUNIT2, NKEYS, NCOL1, NCOL2,
     O                      ERRCOD)
C
C     + + + PURPOSE + + +
C     Routine to sort records in file opened on file unit TUNIT1
C     by supplied keys into ascending order with the results written
C     to file opened on file unit OFILE.
C
C     + + + HISTORY + + +
C     This routine was written to provide portablility to source
C     code which requires a sort routine.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER  TUNIT1, TUNIT2, NKEYS,
     +         NCOL1(NKEYS), NCOL2(NKEYS), ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     TUNIT1 - file unit upon which the input file is opened
C     TUNIT2 - file unit upon which the output file is opened
C     NKEYS  - contains the number of sort keys to sort on.
C              If NKEYS = 0, then the sort is performed on
C              the entire record.
C     NCOL1  - integer array which contains the first column
C               number of each sort key
C     NCOL2  - integer array which contains the last column number
C              of each sort key
C     ERRCOD - contains an error code which is returned to the
C              calling program.
C              ERRCOD =  0 if operation was successful
C              ERRCOD = -1 if operation failed
C
C
C     + + + PARAMETERS + + +
      INCLUDE 'psort.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       IUNIT, OUNIT, NRECS, LENGTH, LNGEST, LOOP, IOSTA0
      CHARACTER*250 LINE, LINE2
      LOGICAL       GETOUT, MORREC
C
C     + + + FUNCTIONS + + +
      INTEGER STRLEN
C
C     + + + EXTERNALS + + +
      EXTERNAL STRLEN, ERROUT, PIKSRT, HSORT
C
C     + + + END SPECIFICATIONS + + +
C
C     Initialize temporary variables and arrays
      IUNIT  = TUNIT1
      OUNIT  = TUNIT2
      GETOUT = .FALSE.
C
C     Read input file to determine longest record length and
C     number of records
      LENGTH = 0
      LNGEST = 0
      NRECS  = 0
      MORREC = .TRUE.
 100  CONTINUE
        READ (IUNIT, '(A)', IOSTAT=IOSTA0) LINE
        IF (IOSTA0 .EQ. 0) THEN
C         increment record counter and examine record length
C         to determine maximum record length
          NRECS = NRECS + 1
          LENGTH = STRLEN (LINE)
          IF (LENGTH .GT. LNGEST) LNGEST = LENGTH
          LINE2 = LINE
        ELSE IF( IOSTA0 .LT. 0) THEN
C         end-of-file
          MORREC = .FALSE.
        ELSE
C         error during read
          CALL ERROUT ('Error reading input file','SRTSYM')
          ERRCOD = -1
          GETOUT = .TRUE.
        END IF
      IF (.NOT.GETOUT .AND. MORREC) GOTO 100
C
      IF (.NOT.GETOUT) THEN
        IF (NRECS .LE. 1) THEN
C         no sort is required
          GETOUT = .TRUE.
          IF (NRECS .EQ. 1) THEN
C           write 1 record to output file before exiting
            WRITE (OUNIT, '(A)') LINE2(1:LNGEST)
          END IF
        ELSE IF (NKEYS .EQ. 0) THEN
C         no keys were passed by calling routine
C         entire record is used as the sort key
          NKEYS = 1
          NCOL1(1) = 1
          NCOL2(1) = LNGEST
        ELSE IF (NKEYS .EQ. 1 .AND. NCOL1(1) .EQ. 0) THEN
C         one key was passed and the key is equal to the length
C         of the record, therefore entire record is the sort key
          NCOL1(1) = 1
          NCOL2(1) = LNGEST
        END IF
      END IF
C
      IF (.NOT.GETOUT) THEN
C       determine maximum record length
        DO 200 LOOP = 1, NKEYS, 1
          IF (NCOL1(LOOP) .GT. LNGEST) LNGEST = NCOL1(LOOP)
          IF (NCOL2(LOOP) .GT. LNGEST) LNGEST = NCOL2(LOOP)
 200    CONTINUE
C
        REWIND IUNIT
C
C       call sort routine
        IF (LNGEST .LT. LENREC .AND. NRECS .LT. LENARY) THEN
          CALL HSORT (1, IUNIT, OUNIT, NKEYS, NCOL1, NCOL2,
     +        NRECS, LNGEST, ERRCOD)
        ELSE
          CALL PIKSRT (IUNIT, OUNIT, NKEYS, NCOL1, NCOL2, LNGEST,
     +        NRECS, ERRCOD)
        END IF
      END IF
      RETURN
      END
C
C
C
      SUBROUTINE   DMPBUF (
     I                      BUFNUM)
C
C     + + + PURPOSE + + +
C     routine to copy contents of main buffer to temp xref buffer
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER BUFNUM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     BUFNUM - buffer location from where to retrieve information
C
C     + + + COMMON BLOCKS + + +
      INCLUDE 'cxref.inc'
      INCLUDE 'ctokec.inc'
C
C     + + + END SPECIFICATIONS + + +
C
      TMODUL = MODULE(BUFNUM)
      TMODTY = MODTYP(BUFNUM)
      TMODNU = MODNUM(BUFNUM)
      TARGCN = ARGCNT(BUFNUM)
      TSYMNM = SYMNAM(BUFNUM)
      TCLASS = CLASS(BUFNUM)
      TDEFIN = DEFIND(BUFNUM)
      TATRB1 = ATRIB1(BUFNUM)
      TATRB2 = ATRIB2(BUFNUM)
      TATRB3 = ATRIB3(BUFNUM)
      TDIM1  = DIM1(BUFNUM)
      TDIM2  = DIM2(BUFNUM)
      TDIM3  = DIM3(BUFNUM)
      TDIM4  = DIM4(BUFNUM)
      TCOMNM = COMNAM(BUFNUM)
      TLINEN = LINENO(BUFNUM)
      TLNUSE = LINUSE(BUFNUM)
      TAPOS  = ARGPOS(BUFNUM)
      RETURN
      END
C
C
C
      SUBROUTINE   ERROUT
     I                   (STRNG1, CRTN)
C
C     + + + PURPOSE + + +
C     Routine to print out an error message and then return to the
C     calling routine.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) STRNG1
      CHARACTER*(*) CRTN
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRNG1 - contains a hopefully meaningful message passed from
C              the calling routine.
C     CRTN   - contains the name of the calling routine.
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (/,A,' generated in ',A)
C
C     + + + END SPECIFICATIONS + + +
C
      WRITE (*,2000) STRNG1, CRTN
      RETURN
      END
C
C
C
      INTEGER FUNCTION   STRLEN
     I                          (LINE)
C
C     + + + PURPOSE + + +
C     Function which returns the length of a character string
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) LINE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LINE   -  contains a character string passed from the calling
C               routine.
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + END SPECIFICATIONS + + +
C
      DO 100 STRLEN = LEN(LINE), 1, -1
          IF (LINE (STRLEN:STRLEN) .NE. ' ') RETURN
 100  CONTINUE
      END
C
C
C
      SUBROUTINE   ZIP
     I                 (CZIP,
     M                  CSTR)
C
C     This routine fills the character string CSTR with
C     the given value CZIP.
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*1   CZIP
      CHARACTER*(*) CSTR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     CZIP   - character to fill string
C     CSTR   - character string to be filled
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     LOOP
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + END SPECIFICATIONS + + +
C
      DO 100 LOOP = 1, LEN(CSTR)
        CSTR(LOOP:LOOP) = CZIP
 100  CONTINUE
      RETURN
      END
C
C
C
      SUBROUTINE   GTOKEN
     I                   (LINE, TNUM,
     O                    TOKEN, TOKTYP, TOKLEN, TSTART, TEND,
     O                    EOLFLG, ERRCOD)
C
C     + + + PURPOSE + + +
C     This routine gets the nth token on character string
C     LINE and returns the token in character string TOKEN.
C     Attributes of the TOKEN are placed in TOKTYP.  TOKLEN is
C     the length in characters, of the TOKEN.
C
C     GTOKEN does not recognize blank strings as tokens
C
C     Possible values of TOKTYP are:
C
C       C - Character string is a Character string
C       B - Entire LINE is BLANK (i.e. NO TOKEN found)
C       N - Character string contains only digits (0 - 9)
C           (i.e. it is a number)
C       D - Character string is a delimiter
C       U - Unable to isolate Character string type
C
C     + + + HISTORY + + +
C     This routine was written in support of a portable fortran
C     cross reference generator which is a subsystem of the SYSDOC
C     system.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER ERRCOD, TNUM, TOKLEN, TSTART, TEND
      CHARACTER*(*) LINE, TOKEN
      CHARACTER*1 TOKTYP
      LOGICAL EOLFLG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LINE    - Character string from calling routine
C     TNUM    - Absolute position of the token requested by
C               calling routine (i.e. 3rd token in line)
C     TOKEN   - Character token extracted from LINE and returned
C               to calling routine.
C     TOKTYP  - Type of token returned
C               possible values of TYPE are:
C               C - Character string is a Character string
C               B - Character string is BLANK
C               N - Character string contains all digits (0 - 9)
C                   (i.e. it can be interpreted as a number)
C               D - Character string is a delimiter
C               U - Unable to isolate Character string type
C     TOKLEN  - Character length of returned token
C     TSTART  - Starting Character position of token
C     TEND    - Ending Character position of token
C     EOLFLG  - Logical Flag - set to TRUE if End of Line encountered
C     ERRCOD  - Error code returned to calling routine
C               ERRCOD =  0 if operation was successful
C               ERRCOD = -1 if operation failed
C               ERRCOD = -2 if requested TOKEN is out of range
C                         (i.e. TNUM is greater than number of TOKENs)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER BEGCOL, LOOP, TOKPOS
      LOGICAL GOTTOK, BLANK
C
C     + + + INTRINSICS + + +
      INTRINSIC LEN
C
C     + + + EXTERNALS + + +
      EXTERNAL PARSER
C
C     + + + END SPECIFICATIONS + + +
C
      TOKTYP = 'B'
      EOLFLG = .FALSE.
      ERRCOD = 0
      BEGCOL = 1
      TOKPOS = BEGCOL
      GOTTOK = .FALSE.
C
C     read tokens until target token is retrieved
C
      LOOP = 0
 100  CONTINUE
        LOOP = LOOP + 1
        IF (LOOP .LE. TNUM) THEN
 200      CONTINUE
            BLANK = .FALSE.
C           get token which is closest to (begins in, or follows)
C           column position TOKPOS
            CALL PARSER (TOKPOS, LINE, TOKEN, TOKLEN,
     +                   TOKTYP, EOLFLG, ERRCOD)
            IF (ERRCOD .EQ. 0) THEN
C             token successfully retrieved
              BEGCOL = TOKPOS
              TOKPOS = TOKPOS + TOKLEN
              IF (TOKTYP .EQ. 'B') THEN
C               token is blank, do not count it
                BLANK = .TRUE.
              ELSE IF ( LOOP .EQ. TNUM) THEN
C               target token has been retrieved
                GOTTOK = .TRUE.
              END IF
            ELSE
C             requested token is out of range
              ERRCOD = -2
            END IF
          IF (BLANK .AND. ERRCOD .EQ. 0) GOTO 200
        END IF
      IF (.NOT.GOTTOK .AND. ERRCOD .EQ. 0) GOTO 100
      IF (GOTTOK) THEN
        TSTART = BEGCOL
        TEND   = BEGCOL + TOKLEN - 1
        ERRCOD = 0
      END IF
      RETURN
      END
