C
C
C
      SUBROUTINE   DOCDEL
     I                   (FILENM,
     O                    ERRFLG)
C
C     + + + PURPOSE + + +
C     This routine deletes the file equal to the filename
C     contained in FILENM.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       ERRFLG
      CHARACTER*(*) FILENM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     FILENM - name of file to be deleted
C     ERRFLG - error code returned to calling routine
C               0 if operation was successful - file was deleted
C              -1 if operation failed - file does not exist
C              -2 if operation failed - file exists, not deleted
C
C     + + + LOCAL VARIABLES + + +
      INTEGER IUNIT
      LOGICAL YES
C
C     + + + EXTERNALS + + +
      EXTERNAL GUNIT, ERROUT
C
C     + + + END SPECIFICATIONS + + +
C
      ERRFLG = -2
C
C     Call INQUIRE to see if file exists.
      INQUIRE (FILE=FILENM, EXIST=YES)
      IF (YES) THEN
C       get an available file unit number
        CALL GUNIT (IUNIT, ERRFLG)
        IF (ERRFLG .EQ. 0) THEN
C         GUNIT returned a good unit number - open the file
          OPEN (UNIT=IUNIT, FILE=FILENM, STATUS='OLD', IOSTAT=ERRFLG)
          IF (ERRFLG .EQ. 0) THEN
C           close and delete the file
            CLOSE (UNIT=IUNIT, STATUS='DELETE')
C           double-check to see if delete operation was successful
            INQUIRE (FILE=FILENM, EXIST=YES)
            IF (.NOT.YES) THEN
C             Operation was successful ... file was deleted
              ERRFLG = 0
            ELSE
C             file exists, display error message and set return code
              CALL ERROUT ('Document not deleted','DOCDEL')
              ERRFLG = -2
            END IF
          ELSE
C           open failed, display error message and return
C           leave ERRFLG equal to the value returned by call to OPEN
            CALL ERROUT ('Error calling OPEN','DOCDEL')
          END IF
        ELSE
C         did not get valid file unit number, display message and return
          CALL ERROUT ('Error calling GUNIT','DOCDEL')
        END IF
      ELSE
C       If file does not exist, set ERRFLG and return
        ERRFLG = -1
      END IF
      RETURN
      END
C
C
C
      SUBROUTINE   CONSTR
     I                   (LINE, SSTRGS, ICOL1, ICOL2,
     O                    SUBSTR, LENGTH, ERRCOD)
C     + + + PURPOSE + + +
C     This routine returns a character string which is the
C     concatonation of multiple substrings extracted from LINE.
C     The concatonation of substrings CANNOT exceed the
C     length of the input LINE.  There can be a total of 64
C     substrings concatonated.
C
C     + + + HISTORY + + +
C     This routine was written to support the SYSDOC SUBSRT routine
C     It's original use was to concatonate multiple subkeys within
C     a record prior to sorting the records.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       SSTRGS
      INTEGER       ICOL1(SSTRGS), ICOL2(SSTRGS), ERRCOD
      CHARACTER*(*) LINE, SUBSTR
C
C     + + + ARGUMENT DEFINITIONS + + +
C     LINE   - contains the character string to be disected into
C              substrings
C     SSTRGS - integer number of substrings to be extracted from LINE
C     ICOL1  - integer array containing the starting column of each
C              substring in LINE
C     ICOL2  - integer array containing the ending column of each
C              substring in LINE
C     SUBSTR - concatenated substring returned to calling routine
C     LENGTH - length of returned substring
C     ERRCOD - error code returned to calling routine
C               0 if operation was successful
C              -1 if operation failed
C
C     + + + LOCAL VARIABLES + + +
      INTEGER LENGTH, LOOP1, IPTR1(64), IPTR2(64)
C
C     + + + EXTERNALS + + +
C     EXTERNAL CHRCHR
C
C     + + + END SPECIFICATIONS + + +
C
C     write (*,*) ' --> constr:    line = ', line
C     write (*,*) '              sstrgs = ', sstrgs
C     write (*,*) '               icol1 = ', icol1
C     write (*,*) '               icol2 = ', icol2
      LENGTH = 0
      ERRCOD = -1
C
      IF (SSTRGS .EQ. 1) THEN
C       there is only one substring, extract it and
C       determine the length
        SUBSTR(1:(ICOL2(1) - ICOL1(1)) + 1)  =
     +      LINE(ICOL1(1):ICOL2(1))
        LENGTH = (ICOL2(1) - ICOL1(1)) + 1
        ERRCOD = 0
      ELSE
C       there are multiple substrings to concatonate
C       initilize first substring pointer to point to the first
C       array position in the output character array
        IPTR1(1) = 1
C       loop once for each substring (SSTRGS)
        DO 100 LOOP1 = 1, SSTRGS, 1
C         determine ending pointer for substring
          IPTR2(LOOP1) =  IPTR1(LOOP1) +
     +        (ICOL2(LOOP1) - ICOL1(LOOP1))
C         move substring into output character array
          SUBSTR(IPTR1(LOOP1):IPTR2(LOOP1)) =
     +        LINE(ICOL1(LOOP1):ICOL2(LOOP1))
C         determine output array starting position for next substring
          IF (LOOP1 .LT. SSTRGS) THEN
C           there are more substrings to process
C           set starting pointer to last substring ending pointer + 1
            IPTR1(LOOP1+1) = IPTR2(LOOP1) + 1
          ELSE
C           otherwise, this is the last substring, capture length of
C           returned string
            LENGTH = IPTR2(LOOP1)
            ERRCOD = 0
          END IF
 100    CONTINUE
C       write (*,*) ' <-- constr:  substr = ', substr
C       write (*,*) '              length = ', length
C       write (*,*) '              errcod = ', errcod
      END IF
C
      RETURN
      END
C
C
C
      SUBROUTINE   SUBSRT
     I                   (IFILE, OFILE, NKEYS, NCOL1, NCOL2,
     O                    RTNERR)
C
C     + + + PURPOSE + + +
C     Routine to sort records in file IFILE by supplied keys
C     into ascending order with the results written to file
C     OFILE.
C
C     + + + HISTORY + + +
C     This routine was written to provide portablility to source
C     code which requires a sort routine.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       NKEYS
      INTEGER       NCOL1(NKEYS), NCOL2(NKEYS), RTNERR
      CHARACTER*(*) IFILE, OFILE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IFILE  - contains the file name of the input file
C     OFILE  - contains the file name of the output file
C     NKEYS  - contains the number of sort keys to sort on.
C              If NKEYS = 0, then the sort is performed on
C              the entire record.
C     NCOL1  - integer array which contains the first column
C              number of each sort key
C     NCOL2  - integer array which contains the last column number
C              of each sort key
C     RTNERR - contains an error code which is returned to the
C              calling program.
C               0 if operation was successful
C              -1 if operation failed
C
C     + + + PARAMETERS + + +
      INCLUDE 'psort.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER       IUNIT, OUNIT, NRECS, LENGTH, LNGEST, LOOP,
     +              ERRCOD, IOSTA0, STATFL
      CHARACTER*250 LINE, SAVLIN
C
C     + + + FUNCTIONS + + +
      INTEGER STRLEN
C
C     + + + EXTERNALS + + +
      EXTERNAL STRLEN, ERROUT, OPENSQ, PIKSRT, HSORT
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (2X,'Begin sort - Input file-',A10,
     +        1X,' Output file-',A10,/,
     +        2X,' Number of records=',I5,'     Record length=',I4)
 2005 FORMAT ( A )
 2010 FORMAT ('  Sort not required',/)
 2020 FORMAT ('  End sort',/)
 2030 FORMAT ('  Sort FAILED - ERRCOD =',I3,/)
C
C     + + + END SPECIFICATIONS + + +
C
C     write (*,*) ' --> subsrt:  ifile = ', ifile
C     write (*,*) '              ofile = ', ofile
C     write (*,*) '              nkeys = ', nkeys
C     write (*,*) '              ncol1 = ', ncol1
C     write (*,*) '              ncol2 = ', ncol2
C     Initialize temporary variables and arrays
      LENGTH = 0
      LNGEST = 0
      RTNERR = -1
C
C     Open input file
C     write (*,*) '              opening files'
      CALL OPENSQ (1, IFILE, IUNIT, ERRCOD)
C     write (*,*) '              iunit = ', iunit
C     write (*,*) '             errcod = ', errcod
      IF (ERRCOD .EQ. 0) THEN
C       input file opened successfully, open output file
        CALL OPENSQ (5, OFILE, OUNIT, ERRCOD)
C       write (*,*) '              ounit = ', ounit
C       write (*,*) '             errcod = ', errcod
        IF (ERRCOD .NE. 0) THEN
C         problem opening output file, close input file
          CALL ERROUT ('Error opening output file','SUBSRT')
          CLOSE (IUNIT)
        END IF
      ELSE
C       problem opening input file
        CALL ERROUT ('Error opening input file','SUBSRT')
      END IF
C
      IF (ERRCOD .EQ. 0) THEN
C       determine number of records in file and maximum record length
        NRECS  = 0
        STATFL = 0
 100    CONTINUE
          READ (IUNIT, 1000, IOSTAT=IOSTA0) LINE
          IF (IOSTA0 .EQ. 0) THEN
C           good read, increment record count and check length of line
            NRECS = NRECS + 1
            LENGTH = STRLEN (LINE)
            IF (LENGTH .GT. LNGEST) LNGEST = LENGTH
            SAVLIN = LINE
          ELSE IF (IOSTA0 .GT. 0) THEN
C           read error, close files and abandon sort
            CALL ERROUT ('Error reading input file','SUBSRT')
            CLOSE (IUNIT)
            CLOSE (OUNIT)
            STATFL = -1
            ERRCOD = -1
          ELSE
C           end of file reached, rewind file for sort
            REWIND IUNIT
            STATFL = 1
          END IF
        IF (STATFL .EQ. 0) GOTO 100
      END IF
      IF (ERRCOD .EQ. 0) THEN
C       successfully read input file, continue processing
        WRITE (*,2000) IFILE,OFILE,NRECS,LNGEST
        IF (NRECS .GT. 0) THEN
C         file contained information, verify keys and lengths
          IF (NKEYS .EQ. 0) THEN
C           no keys, entire record is used as the sort key
            NKEYS = 1
            NCOL1(1) = 1
            NCOL2(1) = LNGEST
          ELSE IF (NKEYS .EQ. 1 .AND. NCOL1(1) .EQ. 0) THEN
C           one key, length of the record, entire record is sort key
            NCOL1(1) = 1
            NCOL2(1) = LNGEST
          ELSE
C           check starting and ending columns against record length
C           determined during read process above
            DO 200 LOOP = 1, NKEYS, 1
C             if starting column is beyond previously determined
C             length, reset length to corresponding ending column
C             value
              IF (NCOL1(LOOP) .GT. LNGEST .AND. LNGEST .NE. 0)
     +          LNGEST = NCOL2(LOOP)
C             if ending column is beyond previously determined
C             length, reset length ending column value
              IF (NCOL2(LOOP) .GT. LNGEST .AND. LNGEST .NE. 0)
     +          LNGEST = NCOL2(LOOP)
 200        CONTINUE
          END IF
C         write (*,*) '              check lenth of records'
C         write (*,*) '             lngest =', lngest
C         write (*,*) '              nrecs =', nrecs
C
C         sort the file
          IF (NRECS .GT. 1) THEN
C           more than 1 record, sort file
C           write (*,*) '              calling sort routine'
C           write (*,*) '              nrecs = ', nrecs
C           write (*,*) '             lngest = ', lngest
C           write (*,*) '             errcod = ', errcod
            IF (LNGEST .LE. LENREC .AND. NRECS .LE. LENARY) THEN
C             records will fit in in-memory array, use heapsort
C             write (*,*) '              calling hsort'
              CALL HSORT (1, IUNIT, OUNIT, NKEYS, NCOL1, NCOL2,
     +                     NRECS, LNGEST, ERRCOD)
            ELSE
C             records will not fit in memory array, use bubble sort
C             write (*,*) '              calling piksrt'
              CALL PIKSRT (IUNIT, OUNIT, NKEYS, NCOL1, NCOL2,
     +                      LNGEST, NRECS, ERRCOD)
            END IF
C           write (*,*) '              sort completed'
C           write (*,*) '              nrecs = ', nrecs
C           write (*,*) '             lngest = ', lngest
C           write (*,*) '             errcod = ', errcod
            IF (ERRCOD .EQ. 0) THEN
C             sort was successful
              WRITE (*,2020)
              RTNERR = 0
            ELSE
C             sort failed
              WRITE (*,2030) ERRCOD
            END IF
          ELSE
C           one record, no sort required, save record
            WRITE (*,2010)
            WRITE (OUNIT, 2005) SAVLIN(1:LNGEST)
          END IF
        ELSE
C         no records - no sort required
          WRITE (*,2010)
        END IF
C       close the input and output files
C       write (*,*) '              closing files'
C       write (*,*) '              iunit = ', iunit
        CLOSE (IUNIT)
C       write (*,*) '              ounit = ', ounit
        CLOSE (OUNIT)
C       write (*,*) '              files closed'
      END IF
C     write (*,*) '                leaving subsrt'
C
      RETURN
      END
C
C
C
      SUBROUTINE   HSORT
     I                   (IREL, IUNIT, OUNIT, NKEYS, NCOL1, NCOL2,
     I                    NRECS,
     M                    LNGEST,
     O                    ERRCOD)
C
C     + + + PURPOSE + + +
C     Routine to sort records in file opened on file unit IUNIT
C     by supplied keys into ascending order with the results written
C     to the file opened on file unit OUNIT.
C
C     + + + HISTORY + + +
C     This routine was written to provide portablility to source
C     code which requires a sort routine.  The algorithm used is
C     the Heapsort algorithm invented by J.W.J.Williams.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       NKEYS
      INTEGER       IREL, IUNIT, OUNIT, NCOL1(NKEYS),
     +              NCOL2(NKEYS), LNGEST, NRECS, ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IREL   - integer flag used to indicate whether or not the
C              relative positioning of records with identical keys
C              is to be preserved as was found in the input file
C               0 - original relative positioning is not required
C               1 - preserve original relative record positions
C              Note: if relative positioning is not required, this
C                    sort routine requires 1/2 the CPU time required
C                    otherwise
C     IUNIT  - file unit where input file is opened
C     OUNIT  - file unit where output file is opened
C     NKEYS  - contains the number of sort keys to sort on.
C              If NKEYS = 0, then the sort is performed on
C              the entire record.
C     NCOL1  - integer array which contains the first column
C              number of each sort key
C     NCOL2  - integer array which contains the last column number
C              of each sort key
C     LNGEST - physical length of records to be sorted, modified
C              to the logical length of the records if logical length
C              is longer than physical record length
C     NRECS  - number of records to be sorted
C     ERRCOD - contains an error code which is returned to the
C              calling program.
C               0 if operation was successful
C              -1 if operation failed
C
C     + + + PARAMETERS + + +
      INCLUDE 'psort.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      INDEX1, INDEX2, INDEX3, INDEX4, INTLEN,
     +             LOOP1,  LOOP2,  IOSTA0, SUBCNT, LOOP3,  OFFSET,
     +             KEYLEN, LNGTH1, LNGTH2, INDXAR(LENARY), IART
      CHARACTER*(LENREC) CARRAY(LENARY), KEY1, KEY2,
     +                   TEMP1, LINE1, LINE2
      CHARACTER*10 IOFMT
      CHARACTER*1  CTEMP(7)
      LOGICAL      ONEPAS, SRTFL1, SRTFL2, SRTFL3, SRTFL4, SRTFL5
C
C     + + + INTRINSICS + + +
      INTRINSIC LLT
C
C     + + + EXTERNALS + + +
      EXTERNAL ERROUT, INTCHR, ZIP, CONSTR, CARVAR
C
C     + + + DATA INITIALIZATIONS + + +
      DATA IOFMT/'(A       )'/
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (A)
C
C     + + + END SPECIFICATIONS + + +
C
      IF (NRECS .GT. 1) THEN
C       more than 1 record, sort required
C       initialize temporary variables and arrays
        ONEPAS = .FALSE.
        CALL ZIP (' ',LINE1)
        CALL ZIP (' ',LINE2)
C
C       check to see if calling routine passed sort key starting
C       and ending column values which exceed the physical record
C       length passed
        DO 100 LOOP2 = 1, NKEYS, 1
          IF (NCOL1(LOOP2) .GT. LNGEST) LNGEST = NCOL1(LOOP2)
          IF (NCOL2(LOOP2) .GT. LNGEST) LNGEST = NCOL2(LOOP2)
 100    CONTINUE
C
C       Construct I/O format statement -
C       the character string IOFMT is initialized to '(A       )'
C       convert integer (LNGEST) to character string (CTEMP)
C       and get length of character string
        CALL INTCHR (LNGEST, 7, 1, INTLEN, CTEMP)
C
C       move the character string (CTEMP) into the I/O Format
C       statement (IOFMT) beginning in position 3.  This will
C       create a character string which is subsequently used
C       as the output format for the records being sorted.
C       (e.g. FORMAT (A125    )).
        CALL CARVAR (INTLEN, CTEMP, 7, IOFMT(3:9))
C
        REWIND IUNIT
C
C       Read input file into memory - character array (CARRAY)
C       initialize variables and index array
        DO 150 LOOP2 = 1,NRECS,1
          READ (IUNIT, FMT=IOFMT, IOSTAT=IOSTA0) CARRAY(LOOP2)
          INDXAR(LOOP2) = LOOP2
 150    CONTINUE
C
        IF (IOSTA0 .EQ. 0) THEN
C         multiple records exist, sort required
C         determine key characteristics
          IF (NKEYS .EQ. 0) THEN
C           no keys were passed by calling routine
C           use entire record as the sort key
            NKEYS = 1
            NCOL1(1) = 1
            NCOL2(1) = LNGEST
C           only one sort pass is necessary
            ONEPAS = .TRUE.
          ELSE IF (NKEYS .EQ. 1) THEN
C           one key was passed
            IF (NCOL1(1) .EQ. 0) THEN
C             key columns are undefined
C             use the entire record as the sort key
              NCOL1(1) = 1
              NCOL2(1) = LNGEST
C             only one sort pass is necessary
              ONEPAS = .TRUE.
            ELSE IF ((NCOL2(1)-NCOL1(1)+1) .EQ. LNGEST) THEN
C             the key is equal to the length of the record
C             use the entire record as the sort key
C             only one sort pass is necessary
              ONEPAS = .TRUE.
            END IF
          END IF
C
C         if calling routine does not require this sort routine to
C         maintain original relative positioning of records with
C         identical keys, the sort will be completed in one pass
          IF (IREL .EQ. 0) ONEPAS = .TRUE.
C
C         The following sort is a "heapsort"
C
C         To perform a heapsort, all elements must first by arranged
C         into a "heap".
C
C         For purposes of explaining what a heap is, it helps to
C         mentally picture a business organization whereby the
C         organizational structure (heap) is basically a binary tree
C         structure with the top member (sort element) being the
C         chairman of the board.  The chairman of the board has two
C         supervisors.  Each supervisor has two supervisees.  Each
C         supervisee has two supervisees.  And so on down through
C         the levels of the hierarchy.  When the boss retires, the
C         top ranking supervisor is promoted.  Then the top ranking
C         supervisees of the now promoted supervisees is promoted
C         to take his place and so on down through the organization.
C
C         To create the organization (heap) in the first place, imagine
C         that there are N/2 employees (i.e. half the number of elements
C         to be sorted) on the production line, but no supervisors.
C         A supervisor is hired from outside the company to supervise
C         two workers.  If he is less capable than one of his workers,
C         that one is promoted in his place, and he joins the production
C         line.  After every two workers has a supervisor, supervisors
C         are now hired to supervise the supervisors.  If the newly
C         hired supervisor is less capable than one of his supervisees,
C         then he is demoted until he is located in his proper place in
C         the organization.
C
C         Each employee is brought in at the top of the tree and then
C         immediately sifted down to their proper place with the more
C         capable employes being promoted.
C
C         In the following heapsort code, the same "sift-down" code
C         can be used for the initial creation of the heap and for the
C         subsequent retirement-and-promotion phase.  One execution of
C         the heapsort code represents the entire life-cycle of a giant
C         corporation: N/2 workers are hired; N/2 potential supervisors
C         are hired; there is a sifting up in the ranks, a sort of
C         Parkinson's Law; finally, in due course, each of the original
C         employees gets promoted to the chairman of the board.  When
C         the last employee becomes the chairman of the board, the
C         organization is sorted.
C
C         This is a N log(sub2) N process compared to a bubble sort
C         which is an extremely inefficient N**2 process.
C
C
C         Set up heapsort index variables
          INDEX1 = NRECS / 2 + 1
          INDEX2 = NRECS
          KEYLEN = 0
C
C         determine concatonated key length
          DO 200 LOOP1 = 1, NKEYS, 1
            KEYLEN = KEYLEN + (NCOL2(LOOP1) - NCOL1(LOOP1) + 1)
 200      CONTINUE
C
C         set sort execution status flag
          SRTFL1 = .TRUE.
C
C         Perform Heapsort on first key
 250      CONTINUE
            IF (INDEX1 .GT. 1) THEN
C             still in hiring phase
              INDEX1 = INDEX1 - 1
              TEMP1 = CARRAY(INDEX1)
C             save original element position
              IART = INDXAR(INDEX1)
            ELSE
C             now in retirement-and-promotion phase
C             clear a space at end of array
              TEMP1 = CARRAY(INDEX2)
C             save original element position
              IART = INDXAR(INDEX2)
C             retire the top of the heap into space at end of array
              CARRAY(INDEX2) = CARRAY(1)
              INDXAR(INDEX2) = INDXAR(1)
C             decrease the size of the organization
              INDEX2 = INDEX2 - 1
C             done with the last promotion
              IF (INDEX2 .EQ. 1) THEN
C               the least capable employee
                CARRAY(1) = TEMP1
                INDXAR(1) = IART
C               sort complete - set sort execution status flag
                SRTFL1 = .FALSE.
              END IF
            END IF
C
            IF (SRTFL1) THEN
C             continue sort process
C             whether we are in the hiring phase or promotion phase,
C             we here set up to sift down the LINE1 element to its
C             proper level
              INDEX3 = INDEX1
              INDEX4 = INDEX1 + INDEX1
C             set sort execution status flag
              SRTFL2 = .TRUE.
 300          CONTINUE
                IF (INDEX4 .LE. INDEX2) THEN
                  IF (INDEX4 .LT. INDEX2) THEN
                    CALL CONSTR (CARRAY(INDEX4), NKEYS, NCOL1, NCOL2,
     &                            KEY1, LNGTH1, ERRCOD)
                    CALL CONSTR (CARRAY(INDEX4+1), NKEYS, NCOL1, NCOL2,
     &                            KEY2, LNGTH2, ERRCOD)
C                   compare element to the better underling
                    IF (LLT (KEY1 (1:LNGTH1) , KEY2 (1:LNGTH2)))
     &                   INDEX4 = INDEX4 + 1
                  END IF
                    CALL CONSTR (TEMP1, NKEYS, NCOL1, NCOL2,
     &                            KEY1, LNGTH1, ERRCOD)
                    CALL CONSTR (CARRAY(INDEX4), NKEYS, NCOL1, NCOL2,
     &                            KEY2, LNGTH2, ERRCOD)
C                 demotion
                  IF (LLT (KEY1(1:LNGTH1), KEY2(1:LNGTH2))) THEN
                    CARRAY(INDEX3) = CARRAY(INDEX4)
                    INDXAR(INDEX3) = INDXAR(INDEX4)
                    INDEX3 = INDEX4
                    INDEX4 = INDEX4 + INDEX4
                  ELSE
C                   this is elements organization level, set INDEX4
C                   to terminate sift-down
                    INDEX4 = INDEX2 + 1
                  END IF
                ELSE
                  SRTFL2 = .FALSE.
                END IF
C             if sort not complete loop up and continue processing
              IF (SRTFL2) GOTO 300
C             otherwise, put last element into its proper slot
              CARRAY(INDEX3) = TEMP1
              INDXAR(INDEX3) = IART
            END IF
C         if sort not complete loop up and continue process
          IF (SRTFL1) GOTO 250
C
C         otherwise, first sort pass is complete
C
          IF (.NOT. ONEPAS) THEN
C           more than one pass is required, sort by index array
C
C           A side effect of the speed of the heapsort algorithm is that
C           records with identical keys may be shuffled from the order
C           in which they were input.  Although this is not wrong, it
C           does cause problems if the user is anticipating otherwise
C           in their code.
C
C           Perform 2nd pass heapsort on index array to place all
C           records with identical keys in their original order according
C           to the index array which was constructed during the first pass.
C
            SUBCNT = 1
            DO 500 LOOP3 = 1, NRECS, 1
 350          CONTINUE
C               set sort execution status flag
                SRTFL3 = .TRUE.
                LINE1 = CARRAY(LOOP3)
                IF (LOOP3 .EQ. NRECS) THEN
                  CALL ZIP ('*',LINE2)
                ELSE
                  LINE2 = CARRAY(LOOP3+1)
                END IF
                CALL CONSTR (LINE1, NKEYS, NCOL1, NCOL2,
     &                        KEY1, LNGTH1, ERRCOD)
                CALL CONSTR (LINE2, NKEYS, NCOL1, NCOL2,
     &                        KEY2, LNGTH2, ERRCOD)
                IF (KEY1(1:LNGTH1) .EQ. KEY2(1:LNGTH2)) THEN
                  SUBCNT = SUBCNT + 1
                ELSE IF (SUBCNT .GT. 1) THEN
C
C                 Set up heapsort index variables
                  OFFSET = LOOP3 - SUBCNT
                  INDEX1 = SUBCNT / 2 + 1
                  INDEX2 = SUBCNT
C
C                 set sort execution status flags
                  SRTFL4 = .TRUE.
C
C                 Heapsort on subset of character array elements
 400              CONTINUE
                    IF (INDEX1 .GT. 1) THEN
C                     still in hiring phase
                      INDEX1 = INDEX1 - 1
                      TEMP1 = CARRAY(OFFSET+INDEX1)
                      IART = INDXAR(OFFSET+INDEX1)
                    ELSE
C                     now in retirement-and-promotion phase
C                     clear a space at end of array
                      TEMP1 = CARRAY(OFFSET+INDEX2)
                      IART = INDXAR(OFFSET+INDEX2)
C                     retire the top of the heap into space at end of array
                      CARRAY(OFFSET+INDEX2) = CARRAY(OFFSET+1)
                      INDXAR(OFFSET+INDEX2) = INDXAR(OFFSET+1)
C                     decrease the size of the organization
                      INDEX2 = INDEX2 - 1
C                     done with the last promotion
                      IF (INDEX2 .EQ. 1) THEN
C                       the least capable employee
                        CARRAY(OFFSET+1) = TEMP1
                        INDXAR(OFFSET+1) = IART
C                       sort complete - set sort execution status flag
                        SRTFL4 = .FALSE.
                      END IF
                    END IF
C
                    IF (SRTFL4) THEN
C                     continue sort process
C                     whether we are in the hiring phase or promotion
C                           phase, we here set up to sift down the LINE1
C                           element to its proper level
                      INDEX3 = INDEX1
                      INDEX4 = INDEX1 + INDEX1
C                     set sort execution status flags
                      SRTFL5 = .TRUE.
 450                  CONTINUE
                        IF (INDEX4 .LE. INDEX2) THEN
                          IF (INDEX4 .LT. INDEX2) THEN
C                           compare element to the better underling
                            IF (INDXAR (OFFSET+INDEX4) .LT.
     +                           INDXAR (OFFSET+INDEX4+1))
     +                           INDEX4 = INDEX4 + 1
                          END IF
C                         demotion
                          IF (IART .LT. INDXAR(OFFSET+INDEX4)) THEN
                            CARRAY(OFFSET+INDEX3)=CARRAY(OFFSET+INDEX4)
                            INDXAR(OFFSET+INDEX3)=INDXAR(OFFSET+INDEX4)
                            INDEX3 = INDEX4
                            INDEX4 = INDEX4 + INDEX4
                          ELSE
C                           this is elements organization level, set
C                           INDEX4 to terminate sift-down
                            INDEX4 = INDEX2 + 1
                          END IF
                        ELSE
                          SRTFL5 = .FALSE.
                        END IF
C                     if sort not complete loop up and continue processing
                      IF (SRTFL5) GOTO 450
C                     Put last element into its proper slot
                      CARRAY(OFFSET+INDEX3) = TEMP1
                      INDXAR(OFFSET+INDEX3) = IART
                    END IF
C                 if sort not complete loop up and continue process
                  IF (SRTFL4) GOTO 400
C                 otherwise, sort on subset is complete - reinitialize
C                 subset record counter and loop up to continue sorting
                  SUBCNT = 1
                  SRTFL3 = .FALSE.
                END IF
              IF (.NOT.SRTFL3) GOTO 350
C           second sort pass complete
 500        CONTINUE
          END IF
C
C         sort complete - write memory character array to output file
          DO 550 LOOP2 = 1,NRECS,1
            TEMP1 = CARRAY (LOOP2)
            WRITE (OUNIT, FMT=IOFMT) TEMP1(1:LNGEST)
 550      CONTINUE
C
        ELSE
C         Error during read
          CALL ERROUT ('Error reading input file','HSORT')
          ERRCOD = -1
        END IF
      ELSE
C       one or zero records, no sort required
        IF (NRECS .EQ. 1)
C         only 1 record, write it to output file
     +      WRITE (OUNIT, 2000) CARRAY(1)
      END IF
      RETURN
      END

