C
C
C
      PROGRAM   SYSDOC
C
C     + + + PURPOSE + + +
C     Program to produce system level documentation for programs
C     written following the Office of Surface Water coding
C     convention.
C       jlkittle 3/86, 3/87, 4/87, 3/88, 9/88
C       kmflynn 9/89, bschell 3/92
C
C     + + + COMMON BLOCKS + + +
C
      INCLUDE 'cfiles.inc'
      INCLUDE 'cdebug.inc'
      INCLUDE 'ddebug.inc'
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      NKEY,ICOL1(5),ICOL2(5),FOPT, ERRCOD, IOSTA0,
     +             IOSTA1, OPTNAM,OPTREF,OPTMER,OPTIND,DINTLN,CLEAN
      CHARACTER*14 FILENM
      LOGICAL      DELFLG, GETOUT
C
C     + + + EXTERNALS + + +
      EXTERNAL     BREF,BNAM,BMER,DOCDEL,SUBSRT,ZIP
      EXTERNAL     OPTSET, OPTOPT, DOCSRM
C
C     + + + DATA INITIALIZATIONS + + +
      DATA   FOPT / 10 /
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (7I5)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (' Error from DOCDEL when attempting to delete file',/,
     &        ' File name = ',A,/,' SYSDOC Aborted')
 2010 FORMAT (/,' Begin SYSDOC')
 2015 FORMAT ( //, I5, ' = stxref',
     $          /, I5, ' = stargf',
     $          /, I5, ' = merge ',
     $          /, I5, ' = index ',
     $          /, I5, ' = intlen',
     $          /, I5, ' = debug ',
     $          /, I5, ' = clean ' )
 2070 FORMAT (/,' Processing Intermediate Files')
 2075 FORMAT (/,' Sorting Intermediate Files')
 2080 FORMAT (/,' Compiling SYSDOC Output files _.out, _.com _.int, ',
     $          ' _.unk, _.ina, and _.ins' )
 2090 FORMAT (/,' Sorting Routine Names by Type and Name')
 2100 FORMAT (/,' End SYSDOC')
 2150 FORMAT (' SYSDOC ABORTED')
C
C     + + + END SPECIFICATIONS + + +
C
      GETOUT = .FALSE.
      CALL ZIP(' ',INTFIL)
      CALL ZIP (' ',FILENM)
      ERRCOD = 0
      DELFLG = .TRUE.
      WRITE(*,2010)
C
C     set processing and formatting options
      CALL OPTSET
C
C     get processing options
      CALL OPTOPT ( OPTREF, OPTNAM, OPTMER, OPTIND,
     O              DINTLN, DEBUG, CLEAN )
      WRITE (*,2015) OPTREF, OPTNAM, OPTMER, OPTIND,
     $               DINTLN, DEBUG, CLEAN
C
      IF (OPTREF.NE.0) THEN
C       create the following 6 files:
C
C       CALL.SD   names, etc.  of routines called from a given routine
C       NAM.SD    names of routines processed
C       ARG.SD    arguments of routines processed
C       CUSE.SD   common block usage of routines processed
C       DOC.SD    documentation (comments) of routines processed
C       ADEF.SD   definition of arguments of routines processed
C
C       first delete any old files
        CALL DOCSRM ( ERRCOD, DELFLG, FILENM )
        IF (DELFLG) THEN
C         all old files, if they existed, were deleted successfully
C
C         read the FORTRAN complier listing files specified in
C         sysdoc input file (file=INP.SD) and create the files:
C
C         CALL.SD   names, etc.  of routines called from a given routine
C         NAM.SD    names of routines processed
C         ARG.SD    arguments of routines processed
C         CUSE.SD   common block usage of routines processed
C         DOC.SD    documentation (comments) of routines processed
C         ADEF.SD   definition of arguments of routines processed
C
          CALL BREF
     I             (CINP,CCALL,CNAM,CARG,CCUSE,CDOC,CADEF,DINTLN,
     I              INTFIL,
     O              ERRCOD)
          IF (ERRCOD .NE. 0) THEN
            GETOUT = .TRUE.
            WRITE(*,2150)
          END IF
          IF (.NOT.GETOUT) THEN
            WRITE(*,2075)
C
C           sort called routines by name(p) and caller(s)
            NKEY    = 3
            ICOL1(1)= 2
            ICOL2(1)= 9
            ICOL1(2)= 24
            ICOL2(2)= 33
            ICOL1(3)= 15
            ICOL2(3)= 22
            CALL SUBSRT (CCALL,CSCALU,NKEY,ICOL1,ICOL2,ERRCOD)
C
C           sort called routines by caller(p)
            NKEY    = 1
            ICOL1(1)= 15
            ICOL2(1)= 22
            CALL SUBSRT (CCALL,CSCALD,NKEY,ICOL1,ICOL2,ERRCOD)
C
C           sort names of routines processed by name(p)
            NKEY    = 1
            ICOL1(1)= 2
            ICOL2(1)= 9
            CALL SUBSRT (CNAM,CSNAM,NKEY,ICOL1,ICOL2,ERRCOD)
C
C           sort arguments by variable name(p) and routine(s)
            NKEY    = 2
            ICOL1(1)= 17
            ICOL2(1)= 24
            ICOL1(2)= 2
            ICOL2(2)= 9
            CALL SUBSRT (CARG,CSARG,NKEY,ICOL1,ICOL2,ERRCOD)
C
C           sort definitions of arguments by variable name(p)
C           and routine(s)
            NKEY    = 2
            ICOL1(1)= 2
            ICOL2(1)= 7
            ICOL1(2)= 73
            ICOL2(2)= 80
            CALL SUBSRT (CADEF,CSADEF,NKEY,ICOL1,ICOL2,ERRCOD)
C
C           sort use of common by routine(p) and common block(s)
            NKEY    = 2
            ICOL1(1)= 2
            ICOL2(1)= 9
            ICOL1(2)= 15
            ICOL2(2)= 22
            CALL SUBSRT (CCUSE,CSCUSE,NKEY,ICOL1,ICOL2,ERRCOD)
C
C           sort use of common by common block(p), variable name(s1)
C           and routine(s2)
            NKEY    = 3
            ICOL1(1)= 15
            ICOL2(1)= 22
            ICOL1(2)= 24
            ICOL2(2)= 31
            ICOL1(3)= 2
            ICOL2(3)= 9
            CALL SUBSRT (CCUSE,CSCUSX,NKEY,ICOL1,ICOL2,ERRCOD)
C
C           sort documentation by routine(p)
            NKEY    = 1
            ICOL1(1)= 2
            ICOL2(1)= 9
            CALL SUBSRT (CDOC,CSDOC,NKEY,ICOL1,ICOL2,ERRCOD)
          END IF
        END IF
      END IF
C
      IF (.NOT.GETOUT .AND. OPTNAM.NE.0 .AND. DELFLG) THEN
C       read the names of arguments in file SARG.SD
C       and the definitions of variables in SADEF.SD
C       and writes file FARG.SD which contains variable
C       names and definitions if available
C
        WRITE(*,2070)
        CALL BNAM
     I           (CSARG,CSADEF,CFARG)
        WRITE(*,2075)
C
C       sort arguments by routine(p) and argument number(s)
        NKEY    = 2
        ICOL1(1)= 2
        ICOL2(1)= 9
        ICOL1(2)= 15
        ICOL2(2)= 16
        CALL SUBSRT (CFARG,CSFARG,NKEY,ICOL1,ICOL2,ERRCOD)
C
C       delete intermediate file
        CALL DOCDEL (CFARG, ERRCOD)
        IF (ERRCOD .EQ. -2) THEN
          FILENM = CFARG
          DELFLG = .FALSE.
        END IF
      END IF
C
      IF (.NOT.GETOUT .AND. OPTMER.NE.0 .AND. DELFLG) THEN
C       merge files SCALLU.SD, SNAM.SD,
C       SCALLD.SD, SFARG.SD, SCUSE.SD, SCUSEX.SD,
C       SDOC.SD and additional generic text and formatting
C       to produce file OUT.SD which is the system level
C       documentation.
C
C       first delete old output files
        CALL DOCDEL (COUT, ERRCOD)
        IF (ERRCOD .EQ. -2) THEN
          FILENM = COUT
          DELFLG = .FALSE.
        ELSE
          CALL DOCDEL (COUTI, ERRCOD)
          IF (ERRCOD .EQ. -2) THEN
            FILENM = COUTI
            DELFLG = .FALSE.
          ELSE
            CALL DOCDEL (COUTU, ERRCOD)
            IF (ERRCOD .EQ. -2) THEN
              FILENM = COUTU
              DELFLG = .FALSE.
            ELSE
              CALL DOCDEL (COUTC, ERRCOD)
              IF (ERRCOD .EQ. -2) THEN
                FILENM = COUTC
                DELFLG = .FALSE.
              ELSE
                CALL DOCDEL (CIND, ERRCOD)
                IF (ERRCOD .EQ. -2) THEN
                  FILENM = CIND
                  DELFLG = .FALSE.
                END IF
              END IF
            END IF
          END IF
        END IF
        IF (DELFLG) THEN
C         all old files, if they existed, were deleted successfully
C         merge reference listings to create sysdoc report
          WRITE(*,2080)
          CALL BMER
     I            (CPARM,CSCALU,CSNAM,COUT,COUTI,COUTC,COUTU,CSCALD,
     1             CSFARG,CSCUSE,CSCUSX,CSDOC,CIND)
        END IF
      END IF
C
      IF (CLEAN .NE. 0) THEN
C       delete all intermediate files
        CALL DOCSRM ( ERRCOD, DELFLG, FILENM )
      END IF
C
      IF (.NOT.GETOUT .AND. OPTIND.NE.0 .AND. DELFLG) THEN
C       first delete old index file
        CALL DOCDEL (CSIND, ERRCOD)
        IF (ERRCOD .EQ. -2) THEN
          FILENM = CSIND
          DELFLG = .FALSE.
        END IF
C
        IF (DELFLG) THEN
C         old index file, if it existed, was deleted successfully
C         sort routine names by type and name
          WRITE(*,2090)
          NKEY    = 1
          ICOL1(1)= 2
          ICOL2(1)= 9
          CALL SUBSRT (CIND,CSIND,NKEY,ICOL1,ICOL2,ERRCOD)
        END IF
      END IF
C
      IF (.NOT.GETOUT .AND. .NOT. DELFLG) THEN
C       error deleting old files
        WRITE (*, 2000) FILENM
      ELSE
C       sysdoc completed successfully
        WRITE(*,2100)
      END IF
      STOP
      END
C
C
C
      SUBROUTINE   DOCSRM
     O                   ( ERRCOD, DELFLG, FILENM )
C
C     + + + PURPOSE + + +
C     Removes all intermediate files
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER      ERRCOD
      CHARACTER*14 FILENM
      LOGICAL      DELFLG
C
C     + + + COMMONS + + + 
      INCLUDE 'cfiles.inc'
C
C     + + + EXTERNALS + + +
      EXTERNAL DOCDEL
C
C     + + + END SPECIFICATIONS
C
C     assume everything will be ok
      ERRCOD = 0
      DELFLG  = .TRUE.
      FILENM = '              '
C
C     delete the files
      CALL DOCDEL (CCALL, ERRCOD)
      IF (ERRCOD .EQ. -2) THEN
        FILENM = CCALL
        DELFLG = .FALSE.
      ELSE
        CALL DOCDEL (CNAM, ERRCOD)
        IF (ERRCOD .EQ. -2) THEN
          FILENM = CNAM
          DELFLG = .FALSE.
        ELSE
          CALL DOCDEL (CARG, ERRCOD)
          IF (ERRCOD .EQ. -2) THEN
            FILENM = CARG
            DELFLG = .FALSE.
          ELSE
            CALL DOCDEL (CCUSE, ERRCOD)
            IF (ERRCOD .EQ. -2) THEN
              FILENM = CCUSE
              DELFLG = .FALSE.
            ELSE
              CALL DOCDEL (CDOC, ERRCOD)
              IF (ERRCOD .EQ. -2) THEN
                FILENM = CDOC
                DELFLG = .FALSE.
              ELSE
                CALL DOCDEL (CADEF, ERRCOD)
                IF (ERRCOD .EQ. -2) THEN
                  FILENM = CADEF
                  DELFLG = .FALSE.
                ELSE
                  CALL DOCDEL (CSADEF, ERRCOD)
                  IF (ERRCOD .EQ. -2) THEN
                    FILENM = CSADEF
                    DELFLG = .FALSE.
                  ELSE
                    CALL DOCDEL (CSARG, ERRCOD)
                    IF (ERRCOD .EQ. -2) THEN
                      FILENM = CSARG
                      DELFLG = .FALSE.
                    ELSE
                      CALL DOCDEL (CSCALD, ERRCOD)
                      IF (ERRCOD .EQ. -2) THEN
                        FILENM = CSCALD
                        DELFLG = .FALSE.
                      ELSE
                        CALL DOCDEL (CSCALU, ERRCOD)
                        IF (ERRCOD .EQ. -2) THEN
                          FILENM = CSCALU
                          DELFLG = .FALSE.
                        ELSE
                          CALL DOCDEL (CSCUSE, ERRCOD)
                          IF (ERRCOD .EQ. -2) THEN
                            FILENM = CSCUSE
                            DELFLG = .FALSE.
                          ELSE
                            CALL DOCDEL (CSCUSX, ERRCOD)
                            IF (ERRCOD .EQ. -2) THEN
                              FILENM = CSCUSX
                              DELFLG = .FALSE.
                            ELSE
                              CALL DOCDEL (CSDOC, ERRCOD)
                              IF (ERRCOD .EQ. -2) THEN
                                FILENM = CSDOC
                                DELFLG = .FALSE.
                              ELSE
                                CALL DOCDEL (CSFARG, ERRCOD)
                                IF (ERRCOD .EQ. -2) THEN
                                  FILENM = CSFARG
                                  DELFLG = .FALSE.
                                ELSE
                                  CALL DOCDEL (CSIND, ERRCOD)
                                  IF (ERRCOD .EQ. -2) THEN
                                    FILENM = CSIND
                                    DELFLG = .FALSE.
                                  ELSE
                                    CALL DOCDEL (CSNAM, ERRCOD)
                                    IF (ERRCOD .EQ. -2) THEN
                                      FILENM = CSNAM
                                      DELFLG = .FALSE.
                                    END IF
                                  END IF
                                END IF
                              END IF
                            END IF
                          END IF
                        END IF
                      END IF
                    END IF
                  END IF
                END IF
              END IF
            END IF
          END IF
        END IF
      END IF
C
      RETURN
      END
