C
C
C
      SUBROUTINE   GUNIT
     M                   (IUNIT,
     O                    ERRCOD)
C
C     + + + PURPOSE + + +
C     This routine determines the next available fortran file unit
C     If IUNIT is non-zero when this routine is called, routine
C     tests to see if unit=IUNIT is available.  If unit=IUNIT is not
C     available this routine returns the next higher unit number
C     which is available.
C
C     This routine sets ERRCOD=0 if operation was successful
C     If routine fails (i.e. no units available) ERRCOD is set to 1
C     or the value of ERRCOD returned by the IOSTAT directive in the
C     call to INQUIRE.
C
C     + + + HISTORY + + +
C     This routine was written to be used whenever a fortran file
C     unit number is required.  It is extremely useful to use this
C     routine to avoid file unit conflicts in large applications.
C     This routine also makes the code more portable since file
C     unit numbers are flexible and therefore will not conflict
C     with operating system differences.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER IUNIT, ERRCOD
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IUNIT   - Fortran unit number passed from calling routine
C     ERRCOD  - error code returned to calling routine
C                0 operation was successful
C               -1 operation failed
C
C     + + + LOCAL VARIABLES + + +
      INTEGER OUNIT, MAXUNT
      LOGICAL YES, GOTUN
C
C     + + + END SPECIFICATIONS + + +
C
C     set maximum file units to test
      MAXUNT = 128
C     set unit number to 1 if unit passed by calling routine
C     out of range
      IF (IUNIT .LE. 0 .OR. IUNIT .GT. MAXUNT) IUNIT = 1
      OUNIT = IUNIT
      GOTUN = .FALSE.
C
 100  CONTINUE
        INQUIRE (UNIT=OUNIT, OPENED=YES, IOSTAT=ERRCOD)
        IF (ERRCOD .EQ. 0) THEN
C         call to INQUIRE was successful
          IF (YES) THEN
C           unit is in use, increment unit number
            OUNIT = OUNIT + 1
            IF (OUNIT .GT. MAXUNT) THEN
C             unit number exceeds max allowed, no units are available
              ERRCOD = -1
            END IF
          ELSE
C           unit is available
            IUNIT  = OUNIT
            GOTUN  = .TRUE.
          END IF
        END IF
      IF (.NOT.GOTUN .AND. ERRCOD .EQ. 0) GOTO 100
C
      RETURN
      END
C
C
C
      SUBROUTINE   OPENSQ
     I                    (IOTYPE, FILENM,
     O                     FUNIT , ERRCOD)
C
C     + + + PURPOSE + + +
C     Routine to check to see if file = FILENM exists, and then
C     open it on an available file unit.
C
C     + + + HISTORY + + +
C     This routine was written to support portable source code.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       IOTYPE, FUNIT, ERRCOD
      CHARACTER*(*) FILENM
C
C     + + + ARGUMENT DEFINITIONS + + +
C     IOTYPE  - contains a number which indicates if the file
C               should be opened for input or output
C               IOTYPE = 1 if old file should be opened for input
C               IOTYPE = 2 if new file should be opened for output
C               IOTYPE = 3 if new file should be opened for input
C               IOTYPE = 4 if old file should be opened for appended
C                        output
C               IOTYPE = 5 if file should be opened and if it
C                        already existed, overwritten with new output
C     FILENM  - file to open if it exists
C     FUNIT   - contains the file unit number on which the file
C               was opened.  This number is returned to the
C               calling program.
C     ERRCOD  - error code indicating success or failure of the
C               operation.
C                0 if successful
C               -1 if operation failed
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     ERRFLG
      CHARACTER*1 YORN
      LOGICAL     INFILE, INUSE, FEXIST, OVERWR, BADNAM,
     +            PROMPT, APPFIL, GETOUT
C
C     + + + EXTERNALS + + +
      EXTERNAL GUNIT, ERROUT, ZIP
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (A)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT ('Enter input file name or <CR> to exit: ')
 2001 FORMAT ('Enter output file name or <CR> to exit: ')
 2002 FORMAT ('File in use')
 2003 FORMAT ('File ',A,' already exists, Overwrite?')
 2004 FORMAT ('File ',A,' does not exist')
C
C     + + + END SPECIFICATIONS + + +
C
      FEXIST = .FALSE.
      YORN = 'N'
      ERRCOD = 0
      PROMPT = .FALSE.
      IF (IOTYPE .GT. 0 .AND. IOTYPE .LT. 6) THEN
C       this is a legal I/O type
C
        IF (IOTYPE .EQ. 1) THEN
C         old file to be opened for input
          APPFIL = .FALSE.
          INFILE = .TRUE.
          OVERWR = .FALSE.
        ELSE IF (IOTYPE .EQ. 2) THEN
C         new file to be opened for output
          APPFIL = .FALSE.
          INFILE = .FALSE.
          OVERWR = .FALSE.
        ELSE IF (IOTYPE .EQ. 3) THEN
C         new file to be opened for input
          APPFIL = .FALSE.
          INFILE = .TRUE.
          OVERWR = .FALSE.
        ELSE IF (IOTYPE .EQ. 4) THEN
C         old file to be opened for appended output
          APPFIL = .TRUE.
          INFILE = .FALSE.
          OVERWR = .FALSE.
          INFILE = .FALSE.
        ELSE IF (IOTYPE .EQ. 3) THEN
C         file to be opened and, if it already existed
C         overwritten with new output
          APPFIL = .FALSE.
          INFILE = .FALSE.
          OVERWR = .TRUE.
        END IF
C
C       If FILENM is blank, set flag to prompt for file name
        IF (FILENM(1:10) .EQ. '          ') PROMPT = .TRUE.
C
 100    CONTINUE
          BADNAM = .FALSE.
          GETOUT = .FALSE.
          IF (PROMPT) THEN
C           prompt user for filename
            IF (INFILE) THEN
              WRITE (*,2000)
            ELSE
              WRITE (*,2001)
            END IF
            CALL ZIP (' ', FILENM)
            READ (*,1000) FILENM
            IF (FILENM(1:4) .EQ. '    ') THEN
C             user elected to exit
              CALL ZIP (' ',FILENM)
              ERRCOD = -1
              GETOUT = .TRUE.
            END IF
          END IF
C
          IF (.NOT.GETOUT) THEN
C           Call INQUIRE to get status of file
            INQUIRE (FILE=FILENM, OPENED=INUSE, EXIST=FEXIST,
     +               IOSTAT=ERRFLG)
            IF (INUSE) THEN
C             file is already in use
              IF (PROMPT) THEN
                WRITE (*,2002) FILENM
                BADNAM = .TRUE.
              ELSE
                IF (ERRFLG .EQ. 0) ERRFLG = -1
                GETOUT = .TRUE.
              END IF
            END IF
C
            IF (.NOT.BADNAM .AND. (.NOT.GETOUT)) THEN
              IF (.NOT.INFILE .AND. FEXIST) THEN
C               output file exists, prompt to make sure it is
C               supposed to be overwritten.
                IF (PROMPT) THEN
                  WRITE (*,2003) FILENM
                  READ (*,1000) YORN
                  IF (YORN .EQ. 'Y' .OR. YORN .EQ. 'y') THEN
C                   user elected to overwrite existing file
                    OVERWR = .TRUE.
                  ELSE
C                   user does not want to overwrite existing file
                    BADNAM = .TRUE.
                  END IF
                END IF
              END IF
            END IF
C
            IF (.NOT.BADNAM .AND. (.NOT.GETOUT)) THEN
              IF (.NOT.FEXIST .AND. (APPFIL .OR. OVERWR)) THEN
C               file is supposed to exist, but does not
                IF (PROMPT) THEN
                  WRITE (*,2004) FILENM
                  BADNAM = .TRUE.
                ELSE
                  ERRCOD = -1
                  GETOUT = .TRUE.
                END IF
              END IF
            END IF
C
            IF (.NOT.BADNAM .AND. (.NOT.GETOUT)) THEN
C             get available file unit number and
C             then open the file
              CALL GUNIT (FUNIT, ERRCOD)
              IF (ERRCOD .NE. 0) THEN
C               Error generated when attempting to get unit number
C               Print error message and exit.
                IF (FUNIT.NE.0) THEN
                  ERRCOD = -1
                  CALL ERROUT ('Error in call to GUNIT','OPENSQ')
                ELSE
                  ERRCOD = -1
                  CALL ERROUT ('No file units available','OPENSQ')
                END IF
              END IF
            END IF
C
            IF (.NOT.BADNAM .AND. (.NOT.GETOUT)) THEN
C             Open file
              IF (IOTYPE .EQ. 1) THEN
C               old file to be opened for input
                OPEN (UNIT=FUNIT, FILE=FILENM, STATUS='OLD',
     +                ACCESS='SEQUENTIAL', IOSTAT=ERRFLG)
              ELSE IF (IOTYPE .EQ. 2) THEN
C               new file to be opened for output
                OPEN (UNIT=FUNIT, FILE=FILENM, STATUS='NEW',
     +                ACCESS='SEQUENTIAL', IOSTAT=ERRFLG)
              ELSE IF (IOTYPE .EQ. 3) THEN
C               new file to be opened for input
                OPEN (UNIT=FUNIT, FILE=FILENM, STATUS='NEW',
     +                ACCESS='SEQUENTIAL', IOSTAT=ERRFLG)
              ELSE IF (IOTYPE .EQ. 4) THEN
C               old file to be opened for appended output
                OPEN (UNIT=FUNIT, FILE=FILENM, STATUS='OLD',
     +                ACCESS='SEQUENTIAL', IOSTAT=ERRFLG)
              ELSE IF (IOTYPE .EQ. 5) THEN
C               file to be opened and positioned to TOF
                OPEN (UNIT=FUNIT, FILE=FILENM,
     +                ACCESS='SEQUENTIAL', IOSTAT=ERRFLG)
              END IF
              IF (ERRFLG .NE. 0) THEN
                CALL ERROUT('Error in call to OPEN','OPENSQ')
                ERRFLG = -1
              END IF
            END IF
          END IF
        IF (BADNAM) GOTO 100
      ELSE
        CALL ERROUT('Invalid argument - 1 through 5 only','OPENSQ')
        ERRCOD = -1
      END IF
C
      RETURN
      END

