C
C
C
      SUBROUTINE   PARSER
     I                    (BCOL, LINE,
     O                     STRING, STRLEN, TYPE, EOLFLG, ERRCOD)
C
C     + + + PURPOSE + + +
C     This routine parses the character string (LINE) and
C     returns the first full substring found to the right of
C     the character position (BCOL) passed by calling routine.
C
C     Delimiters are considered to be substrings of length
C     one (1).  However, PARSER will first check to see
C     if there are multiple sequential occurances of the
C     delimiter characters and will then return the count
C     of sequential delimiter characters found.
C
C     If a blank string is returned to the calling program and
C     the value of STRLEN is not equal to 0, it means that PARSER
C     found STRLEN blank characters.
C
C     The file, PDELIM.INC, contains the list of valid delimiters.
C
C     + + + HISTORY + + +
C     This routine was written to work with the MAKXRF routine which
C     is used to create a cross-reference listing for FORTRAN
C     source code.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER BCOL, STRLEN, ERRCOD
      CHARACTER*(*) LINE, STRING
      CHARACTER*1   TYPE
      LOGICAL       EOLFLG
C
C     + + + ARGUMENT DEFINITIONS + + +
C     BCOL     - first character position to be examined
C                BCOL is passed from the calling routine
C     LINE     - character string to be parsed
C     STRING   - character substring returned to calling routine
C     STRLEN   - length of substring returned to calling routine
C     TYPE     - type of character string found
C                possible values of TYPE are:
C                C - Character string is a Character string
C                B - Character string is BLANK
C                N - Character string contains all digits (0 - 9)
C                    (i.e. it can be interpreted as a number)
C                D - Character string is a delimiter
C                U - Unable to isolate Character string type
C     EOLFLG   - logical flag - true if End of Line detected
C     ERRCOD   - error code returned to calling routine
C                ERRCOD =  0 if operation was successful
C                ERRCOD = -1 if operation failed
C
C     + + + LOCAL VARIABLES + + +
      INTEGER     IPOS1, IPOS2, DELPTR, LENARY, TLOOP, NDELS,
     +            LCAPS, HCAPS, LLOWS, HLOWS, LDIGIT, HDIGIT, ITEMP
      CHARACTER*1 TCHAR, DELIMS(17)
      LOGICAL     DELSTR, REPEAT, GOTSUB, GOTTYP, MORDEL
C
C     + + + INTRINSICS + + +
      INTRINSIC ICHAR, LEN, CHAR
C
C     + + + EXTERNALS + + +
      EXTERNAL ZIP
C
C     + + + DATA INITIALIZATIONS + + +
      DATA DELIMS /' ','(',')',',','.',':',';','''','"',
     +             '/',' ','+','-','*','=','#','%'/
C
C     DELIMS(1)   = Blank
C     DELIMS(2)   = Left Parenthesis (
C     DELIMS(3)   = Right Parenthesis )
C     DELIMS(4)   = Comma ,
C     DELIMS(5)   = Period/Dot .
C     DELIMS(6)   = Colon :
C     DELIMS(7)   = Semicolon ;
C     DELIMS(8)   = Single Quote '
C     DELIMS(9)   = Double Quote "
C     DELIMS(10)  = Forward Slash /
C     DELIMS(11)  = Back Slash \
C     DELIMS(12)  = Plus Sign +
C     DELIMS(13)  = Dash -
C     DELIMS(14)  = Asterisk *
C     DELIMS(15)  = Equal Sign =
C     DELIMS(16)  = Pound Sign #
C     DELIMS(17)  = Percent Sign %
C
C     + + + END SPECIFICATIONS + + +
C
C     use CHAR to set special character backslash
      DELIMS(11) = CHAR ( 92 )
C
C     initialize local variables
      ERRCOD = 0
      NDELS  = 17
      IPOS1  = 0
      IPOS2  = 0
      EOLFLG = .FALSE.
C
C     TYPE is unknown until proven otherwise
      TYPE = 'U'
      GOTTYP = .FALSE.
C
C     establish valid range of character values for the CPU on
C     which this program is executing.
C
C     uppercase:
      LCAPS = ICHAR('A')
      HCAPS = ICHAR('Z')
      IF (LCAPS .GT. HCAPS) THEN
        ITEMP = HCAPS
        HCAPS = LCAPS
        LCAPS = ITEMP
      END IF
C
C     lower case
      LLOWS = ICHAR('a')
      HLOWS = ICHAR('z')
      IF (LLOWS .GT. HLOWS) THEN
        ITEMP = HLOWS
        HLOWS = LLOWS
        LLOWS = ITEMP
      END IF
C
C     digits
      LDIGIT = ICHAR('0')
      HDIGIT = ICHAR('9')
      IF (LDIGIT .GT. HDIGIT) THEN
        ITEMP = HDIGIT
        HDIGIT = LDIGIT
        LDIGIT = ITEMP
      END IF
C
C     determine array length of LINE array
      LENARY = LEN(LINE)
C     initialize STRING to blank
      CALL ZIP (' ', STRING)
C
      IF (BCOL .LE. LENARY) THEN
C       the beginning character position is within the bounds of
C       array, loop and examine each character in LINE until a
C       substring has been found.
        DELSTR = .FALSE.
        STRLEN = 1
        IPOS1  = BCOL - 1
 100    CONTINUE
          GOTSUB = .FALSE.
          IPOS1  = IPOS1 + 1
          IF (IPOS1 .LE. LENARY) THEN
C           compare character against the possible delimiters
            DELPTR = 1
            MORDEL = .TRUE.
 200        CONTINUE
              IF (DELPTR .LE. NDELS) THEN
C               compare character against delimeter
                IF (LINE(IPOS1:IPOS1) .EQ. DELIMS(DELPTR)) THEN
C                 delimiter has been found
                  GOTSUB = .TRUE.
                  IF (IPOS1 .EQ. BCOL) THEN
C                   delimiter begins in first position, therefore
C                   this is a delimiter substring
                    DELSTR = .TRUE.
C                   substring type is 'D'elimiter
                    TYPE = 'D'
                    GOTTYP = .TRUE.
                    IF (LINE(IPOS1:IPOS1) .EQ. ' ') THEN
C                     delimiter is a space (blank), therefore
C                     substring type is 'B'lank
                      TYPE = 'B'
                    END IF
                    IF (DELPTR .EQ. 1 .OR. DELPTR .EQ. 8 .OR.
     +                  DELPTR .EQ. 10) THEN
C                     delimeter is a blank, a single quote, or
C                     a forward slash - loop to determine if there
C                     are repeating delimiters
                      IPOS2 = IPOS1
 300                  CONTINUE
                        REPEAT = .FALSE.
                        IPOS2 = IPOS2 + 1
                        IF (IPOS2 .LE. LENARY) THEN
C                         not at end of array
                          IF (LINE(IPOS2:IPOS2) .EQ.
     +                        DELIMS(DELPTR)) THEN
C                           character is same as previous delimiter
                            STRLEN = STRLEN + 1
                            REPEAT = .TRUE.
                          END IF
                        ELSE
C                         end of array
                          EOLFLG = .TRUE.
                        END IF
                      IF (REPEAT) GOTO 300
                    END IF
                  ELSE
C                   delimiter was found after end of substring
C                   decrement substring length counter
                    STRLEN = STRLEN - 1
                  END IF
C                 check to see if this is last character, if so,
C                 set EOLFLG flag
                  IF (IPOS1 .GE. LENARY .OR. IPOS2 .GE. LENARY)
     +                EOLFLG = .TRUE.
                ELSE
C                 increment delimiter array pointer
                  DELPTR = DELPTR + 1
                END IF
              ELSE
C               no more delimiters to compare against
                MORDEL = .FALSE.
              END IF
            IF (MORDEL .AND. (.NOT.GOTSUB)) GOTO 200
          END IF
C       if substring has not been found then delimiter or
C       end of line has not been reached, increment substring counter
        IF (.NOT.GOTSUB) STRLEN = STRLEN + 1
        IF (.NOT.GOTSUB) GOTO 100
C
C       set returned character string equal to substring found in line
        STRING(1:STRLEN) = LINE(BCOL:BCOL+(STRLEN-1))
        IF (.NOT.GOTTYP) THEN
C         string type not yet determined
C         loop to see if any characters in string are
C         letters of the alphabet.  If so, set string type
C         equal to character (C).
          TLOOP = 0
 400      CONTINUE
            TLOOP = TLOOP + 1
            TCHAR = STRING(TLOOP:TLOOP)
C           check to see if character is alpha
            IF (ICHAR(TCHAR) .GE. LCAPS .AND.
     +          ICHAR(TCHAR) .LE. HCAPS) TYPE = 'C'
            IF (ICHAR(TCHAR) .GE. LLOWS .AND.
     +          ICHAR(TCHAR) .LE. HLOWS) TYPE = 'C'
            IF (TYPE .EQ. 'C') GOTTYP = .TRUE.
          IF (TLOOP .LE. STRLEN .AND. (.NOT.GOTTYP)) GOTO 400
          IF (.NOT.GOTTYP) THEN
C           string contains no alphabet characters, loop to see
C           if all characters are digits
            TLOOP = 0
 500        CONTINUE
              TLOOP = TLOOP + 1
              IF (TLOOP .LE. STRLEN) THEN
                TCHAR = STRING(TLOOP:TLOOP)
                IF (ICHAR(TCHAR) .GE. LDIGIT .AND.
     +              ICHAR(TCHAR) .LE. HDIGIT) THEN
                  TYPE = 'N'
                ELSE
                  TYPE = 'U'
                  GOTTYP = .TRUE.
                END IF
              END IF
            IF (TLOOP .LE. STRLEN .AND. (.NOT.GOTTYP)) GOTO 500
          END IF
        END IF
      ELSE
C       the character position passed to PARSER is greater
C       than the maximum length of the character string
        ERRCOD = -1
      END IF
      RETURN
      END
C
C
C
      SUBROUTINE   PULNAM
     I                   (FNAME,
     O                    SOUR,XNAME,SRTNAM,PATH,PTHLEN,MAKEX,ERRCOD)
C
C     + + + PURPOSE + + +
C     Routine to extract group name from file name and build
C     filenames for cross reference and temporary sort files.
C     Also determines if a pre-existing cross reference file
C     is being processed.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER       ERRCOD, PTHLEN
      CHARACTER*(*) FNAME, XNAME, SRTNAM, PATH
      CHARACTER*1   SOUR(8)
      LOGICAL       MAKEX
C
C     + + + ARGUMENT DEFINTIONS + + +
C     FNAME  - path name of group file
C     SOUR   - name of group file
C     XNAME  - name of cross reference file
C     SRTNAM - name of temporary sort file
C     PATH   - path name exclusive of file name
C     PTHLEN - length of pathname
C     MAKEX  - flag indicating if group file is an existing
C              cross reference file
C     ERRCOD - error code returned to calling routine
C               0 if operation was successful
C              -1 if operation failed
C              -2 if operation failed, FNAME was blank
C
C     + + + LOCAL VARIABLES + + +
      INTEGER      J, LENNAM, JLEN, LOCDOT, LOCPTH, LNSOUR
      CHARACTER*1  FNAME1(64), PATH1(64), BKS
      LOGICAL      FNDDOT, FNDPTH
C
C     + + + FUNCTIONS + + +
      INTEGER      LENSTR
C
C     + + + INTRINSICS + + +
      INTRINSIC    CHAR
C
C     + + + EXTERNALS + + +
      EXTERNAL     LENSTR, ZIPC, CHRCHR, ZIP
C
C     + + + INPUT FORMATS + + +
 1000 FORMAT (64A1)
C
C     + + + OUTPUT FORMATS + + +
 2000 FORMAT (64A1)
C
C     + + + END SPECIFICATIONS + + +
C
      ERRCOD = -1
      MAKEX  = .TRUE.
      FNDDOT = .FALSE.
      FNDPTH = .FALSE.
      CALL ZIPC (8,' ',SOUR)
      CALL ZIPC (64,' ',FNAME1)
      CALL ZIPC (64,' ',PATH1)
      CALL ZIP (' ',XNAME)
      CALL ZIP (' ',SRTNAM)
      CALL ZIP (' ',PATH)
C     use CHAR to set special character backslash
      BKS = CHAR ( 92 )
      READ (FNAME,1000) FNAME1
      READ (FNAME,1000) PATH1
      JLEN = LENSTR(64,FNAME1)
      IF (JLEN .EQ. 0) THEN
C       blank filename
        ERRCOD = -2
      ELSE
        J = JLEN
C       build filenames
 15     CONTINUE
          IF (.NOT.FNDDOT) THEN
            IF (FNAME1(J) .EQ. '.') THEN
              FNDDOT = .TRUE.
              LOCDOT = J
            END IF
          END IF
          IF (.NOT.FNDPTH) THEN
            IF (FNAME1(J) .EQ. '/' .OR.
     +          FNAME1(J) .EQ. BKS .OR.
     +          FNAME1(J) .EQ. '>' ) THEN
              FNDPTH = .TRUE.
              LOCPTH = J
            END IF
          END IF
          J = J - 1
        IF (J .GT. 0) GOTO 15
C
        IF (FNDDOT .AND. FNDPTH) THEN
C         pathname with dot
          IF (LOCDOT .LT. LOCPTH) THEN
C           dot is to the left of the path delimeter and
C           therefore not part of filename
C           the filename does not have a dot suffix
            LENNAM = JLEN - LOCPTH
          ELSE
C           dot is to the right of the path delimeter and
C           therefore is part of the filename
C           the filename has a dot suffix
            LENNAM = LOCDOT - LOCPTH - 1
          END IF
          PTHLEN = LOCDOT - LENNAM - 1
        ELSE IF (FNDDOT .AND. .NOT.FNDPTH) THEN
C         filename with suffix, no pathname
          LENNAM = JLEN - (JLEN-LOCDOT) - 1
          LOCPTH = 0
          PTHLEN = 0
        ELSE IF (.NOT.FNDDOT .AND. FNDPTH) THEN
C         pathname with no suffix
          LENNAM = JLEN - LOCPTH
          LOCDOT = JLEN + 1
          PTHLEN = JLEN - LENNAM
        ELSE
C         no dots and no pathname
          LENNAM = JLEN
          LOCPTH = 0
          LOCDOT = JLEN + 1
          PTHLEN = 0
        END IF
        LNSOUR = LENNAM
        IF (LNSOUR .GT. 8) LNSOUR = 8
        CALL CHRCHR(LNSOUR,FNAME1(LOCPTH+1),SOUR(1))
        CALL ZIPC (64-PTHLEN,' ',PATH1(PTHLEN+1))
        WRITE (PATH,2000) PATH1
C
        IF (FNDDOT) THEN
C         check for existance of XRF suffix
          IF (LOCDOT .LE. 60 .AND.
     +       (FNAME(LOCDOT:LOCDOT+3) .EQ. '.XRF' .OR.
     +        FNAME(LOCDOT:LOCDOT+3) .EQ. '.xrf')) THEN
            XNAME = FNAME
            MAKEX = .FALSE.
          END IF
        END IF
C
        IF (MAKEX) THEN
C         create output file name (XNAME) and temporary
C         sort file name (SRTNAM) from input filename (FNAME)
C         append .XRF and .SRT to filenames
C         fill extra space with blanks
          WRITE(XNAME,2000) SOUR
          SRTNAM = XNAME
          XNAME(LENNAM+1:LENNAM+4) = '.XRF'
          SRTNAM(LENNAM+1:LENNAM+4) = '.SRT'
        END IF
        ERRCOD = 0
      END IF
      RETURN
      END
